/***************************************************************************
****************************************************************************

This is a part of the xModule Source code.

Copyright (C) 2003-2004 The LS-Universe Team

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

****************************************************************************
****************************************************************************/

#include "stdafx.h"
#include "ButtonIcon.h"

ButtonIcon::ButtonIcon()
{
	defaulticon.showIcon = true;
	defaulticon.bigIcon = false;
	defaulticon.iconSize = 16;
	defaulticon.iconX = "0";
	defaulticon.iconY = "";
	
	defaulticon.icon_saturation = 255;
	defaulticon.icon_hueintensity = 0;
	defaulticon.icon_huecolor = RGB(255, 255, 255);
}

ButtonIcon::~ButtonIcon()
{
}

void ButtonIcon::subconfigure(string type, ICONSETTINGS &workSettings, ICONSETTINGS &defaultSettings)
{
	workSettings.showIcon = GetxRCBool(type, "ShowIcon", defaultSettings.showIcon);
	workSettings.bigIcon = GetxRCBool(type, "UseBigIcon", defaultSettings.bigIcon);
	workSettings.iconSize = GetxRCInt(type, "IconSize", defaultSettings.iconSize, 8, 64);
	workSettings.iconX = GetxRCString(type, "IconX", defaultSettings.iconX);
	if ( defaulticon.iconY.empty() )
		workSettings.iconY = GetxRCString(type, "IconY", ("-" + IntToString(workSettings.iconSize/2) + "c"));
	else
		workSettings.iconY = GetxRCString(type, "IconY", defaulticon.iconY);
	workSettings.icon_saturation = GetxRCInt(type, "IconSaturationIntensity", defaultSettings.icon_saturation, 0, 255);
	workSettings.icon_hueintensity = GetxRCInt(type, "IconHueIntensity", defaultSettings.icon_hueintensity, 0, 255);
	workSettings.icon_huecolor = GetxRCColor(type, "IconHueColor", defaultSettings.icon_huecolor);
}

void ButtonIcon::configure(const string &name)
{
	subconfigure((name + "Entry"), entry, defaulticon);

	subconfigure((name + "ActiveEntry"), selentry, entry);

	subconfigure((name + "Folder"), folder, entry);

	subconfigure((name + "ActiveFolder"), selfolder, (GetxRCBool(name, "ActiveFolderUseFolderDefault") ? folder : selentry));
}

void ButtonIcon::apply(HDC hDC, int x, int y, int width, int height, int buttonType, const string &iconPath, const HWND iconhWnd, const LPITEMIDLIST iconpidl, const string &DefaultIcon, bool useAlphaMap)
{
	int iconX = 0;
	int iconY = 0;
	int iconSize = 16;
	bool bigIcon = false;

	int _saturation = 255;
	int _hueIntensity = 0;
	COLORREF _hueColor = RGB(255, 255, 255);

	if (buttonType == 1)
	{
		if (!entry.showIcon)
			return;
		iconX = xParseCoord(entry.iconX, 0, width);
		iconY = xParseCoord(entry.iconY, 0, height);

		iconSize = entry.iconSize;
		bigIcon = entry.bigIcon;
		_saturation = entry.icon_saturation;
		_hueIntensity = entry.icon_hueintensity;
		_hueColor = entry.icon_huecolor;
	}
	else if (buttonType == 2)
	{
		if (!selentry.showIcon)
			return;
		iconX = xParseCoord(selentry.iconX, 0, width);
		iconY = xParseCoord(selentry.iconY, 0, height);

		iconSize = selentry.iconSize;
		bigIcon = selentry.bigIcon;
		_saturation = selentry.icon_saturation;
		_hueIntensity = selentry.icon_hueintensity;
		_hueColor = selentry.icon_huecolor;
	}
	else if (buttonType == 3)
	{
		if (!folder.showIcon)
			return;
		iconX = xParseCoord(folder.iconX, 0, width);
		iconY = xParseCoord(folder.iconY, 0, height);

		iconSize = folder.iconSize;
		bigIcon = folder.bigIcon;
		_saturation = folder.icon_saturation;
		_hueIntensity = folder.icon_hueintensity;
		_hueColor = folder.icon_huecolor;
	}
	else if (buttonType == 4)
	{
		if (!selfolder.showIcon)
			return;
		iconX = xParseCoord(selfolder.iconX, 0, width);
		iconY = xParseCoord(selfolder.iconY, 0, height);

		iconSize = selfolder.iconSize;
		bigIcon = selfolder.bigIcon;
		_saturation = selfolder.icon_saturation;
		_hueIntensity = selfolder.icon_hueintensity;
		_hueColor = selfolder.icon_huecolor;
	}

	HICON hIcon = NULL;
	
	if ( !iconPath.empty() )
		hIcon = GetIconFromPathEx(iconPath.c_str(), bigIcon);
	else if ( iconhWnd != NULL )
		hIcon = GetTaskIcon(iconhWnd, bigIcon);
	else if ( iconpidl != NULL )
	{
		SHFILEINFO shfi;
		SHGetFileInfo((LPCSTR) iconpidl, 0, &shfi, sizeof(SHFILEINFO),
							  SHGFI_ATTRIBUTES | (bigIcon ? SHGFI_LARGEICON : SHGFI_SMALLICON) |
							  SHGFI_PIDL | SHGFI_ICON );
		hIcon = shfi.hIcon;
	}
	else 
		return;

	if ( hIcon == NULL && stricmp(iconPath.c_str(), ".none") != NULL )
		hIcon = GetIconFromPathEx(DefaultIcon.c_str(), bigIcon);
	
	if ( hIcon == NULL )
		return;

	if ( useAlphaMap || IsOS(OS_XP) || _saturation != 255 || _hueIntensity != 0 )
	{
		BITMAPINFO alpha;
			ZeroMemory( &alpha.bmiHeader, sizeof(BITMAPINFOHEADER) );
			alpha.bmiHeader.biWidth=iconSize;      // Set size you need
			alpha.bmiHeader.biHeight=iconSize;    // Set size you need
			alpha.bmiHeader.biPlanes=1;
			alpha.bmiHeader.biBitCount=32;            // Can be 8, 16, 32 bpp or other number
			alpha.bmiHeader.biSizeImage=0;
			alpha.bmiHeader.biSize=sizeof(BITMAPINFOHEADER);
			alpha.bmiHeader.biClrUsed= 0;
			alpha.bmiHeader.biClrImportant= 0;
			VOID *pvBits;

		HDC hdcBGCOPY = CreateCompatibleDC(hDC);
		HBITMAP	hbmBGCOPY = CreateDIBSection( hdcBGCOPY,
										&alpha,
										DIB_RGB_COLORS,
										&pvBits,
										NULL,
										0 );
		SelectObject(hdcBGCOPY, hbmBGCOPY);

		//Copy RealBG in Result Background
		BitBlt( hdcBGCOPY, 0, 0, iconSize, iconSize, hDC, x+iconX, y+iconY, SRCCOPY );

		HDC hdcBuffer = CreateCompatibleDC(hDC);
		HBITMAP	hbmBuffer = CreateDIBSection( hdcBuffer,
										&alpha,
										DIB_RGB_COLORS,
										&pvBits,
										NULL,
										0 );
		SelectObject(hdcBuffer, hbmBuffer);

		//Copy RealBG in Result Background
		if ( IsOS(OS_2KXP) )
			BitBlt(hdcBuffer, 0, 0, iconSize, iconSize, hDC, x+iconX, y+iconY, SRCCOPY );

		//Draw Icon
		DrawIconEx(hdcBuffer, 0, 0, hIcon, iconSize, iconSize, 0, NULL, DI_NORMAL );

		HDC hdcMask = CreateCompatibleDC(hdcBuffer);
		HBITMAP	hbmMask = CreateDIBSection( hdcMask,
										&alpha,
										DIB_RGB_COLORS,
										&pvBits,
										NULL,
										0 );
		SelectObject(hdcMask, hbmMask);

		//Draw Mask of the Icon
		DrawIconEx(hdcMask, 0, 0, hIcon, iconSize, iconSize, 0, NULL, DI_MASK);

		if ( IsOS(OS_XP) )
		{
			BITMAP bm;
			GetObject(hbmBuffer, sizeof(BITMAP), &bm);

			BITMAP bmmask;
			GetObject(hbmMask, sizeof(BITMAP), &bmmask);

			BITMAP bmorg;
			GetObject(hbmBGCOPY, sizeof(BITMAP), &bmorg);

			BYTE *pPixel = (LPBYTE)bm.bmBits;
			BYTE *pPixelMask = (LPBYTE)bmmask.bmBits;
			BYTE *pPixelOrg = (LPBYTE)bmorg.bmBits;

			BYTE r = 0;
			BYTE g = 0;
			BYTE b = 0;
			bool skip = false;

			for (int i = 0; i < bm.bmHeight * bm.bmWidth; ++i)
			{
				//Filter Mask and insert OriginalBG to preserve AlphaChannel
				if ( pPixelMask[0] == 255 && pPixelMask[1] == 255 && pPixelMask[2] == 255 )
				{
					pPixel[0] = pPixelOrg[0];	
					pPixel[1] = pPixelOrg[1];	
					pPixel[2] = pPixelOrg[2];	
					pPixel[3] = pPixelOrg[3];	

					skip = true;
				}
				//For colorizing without AlphaMap
				if ( !skip && pPixelOrg[0] == 255 && pPixelOrg[1] == 0 && pPixelOrg[2] == 255 && pPixel[3] != 255 && pPixel[3] != 0 )
				{
					pPixel[0] = pPixelOrg[0];	
					pPixel[1] = pPixelOrg[1];	
					pPixel[2] = pPixelOrg[2];	
					pPixel[3] = pPixelOrg[3];	

					skip = true;
				}

				if ( skip )
				{
					pPixel+= 4;
					pPixelMask+= 4;
					pPixelOrg+= 4;

					skip = false;
					continue;
				}

				r = pPixel[0];
				g = pPixel[1];
				b = pPixel[2];

				//AlphaMap Colorizing
				//Saturation Effect
				if( _saturation != 255 )
				{
					BYTE gray = (BYTE)(r * 0.3086 + g * 0.6094 + b * 0.0820);
					r = (BYTE)((r * _saturation + gray * (255 - _saturation) + 255) >> 8);
					g = (BYTE)((g * _saturation + gray * (255 - _saturation) + 255) >> 8);
					b = (BYTE)((b * _saturation + gray * (255 - _saturation) + 255) >> 8);
				}
				// Hue effect
				if( _hueIntensity != 0 )
				{
					// The B & R values of the hue is swapped here, can somebody tell me why it only works that way?
					// test normal
					r = (BYTE)((r * (255 - _hueIntensity) + GetBValue(_hueColor) * _hueIntensity ) >> 8);
					g = (BYTE)((g * (255 - _hueIntensity) + GetGValue(_hueColor) * _hueIntensity ) >> 8);
					b = (BYTE)((b * (255 - _hueIntensity) + GetRValue(_hueColor) * _hueIntensity ) >> 8);
				}

				pPixel[0] = r;
				pPixel[1] = g;
				pPixel[2] = b;
				//Fix for Icons without AlphaChannel (all is 0!), with AlphaChannel 0 is Mask
				if ( pPixel[3] == 0 )
					pPixel[3] = 255;

				pPixel+= 4;
				pPixelMask+= 4;
				pPixelOrg+= 4;
			}

			BitBlt( hDC, x+iconX, y+iconY, iconSize, iconSize, hdcBuffer, 0, 0, SRCCOPY);
		}
		else
		{
			int iy = 0;
			int ix = 0;
			COLORREF cl = 0;
			BYTE r = 0;
			BYTE g = 0;
			BYTE b = 0;
			for(iy = 0; iy < iconSize; iy++)
			{
				for(ix = 0; ix < iconSize; ix++)
				{
					// Loop through all transparent pixels...
					// If the mask-pixel is white or grey(if that is possible?), then break
					if( !GetPixel(hdcMask, ix, iy) )
					{
						cl = GetPixel(hdcBuffer, ix, iy);
						r = GetRValue(cl);
						g = GetGValue(cl);
						b = GetBValue(cl);

						// Saturation effect
						if( _saturation != 255 )
						{
							BYTE gray = (BYTE)(r * 0.3086 + g * 0.6094 + b * 0.0820);

							r = (BYTE)((r * _saturation + gray * (255 - _saturation) + 255) >> 8);
							g = (BYTE)((g * _saturation + gray * (255 - _saturation) + 255) >> 8);
							b = (BYTE)((b * _saturation + gray * (255 - _saturation) + 255) >> 8);
						}

						// Hue effect
						if( _hueIntensity != 0 )
						{
							r = (BYTE)((r * (255 - _hueIntensity) + GetRValue(_hueColor) * _hueIntensity + 255) >> 8);
							g = (BYTE)((g * (255 - _hueIntensity) + GetGValue(_hueColor) * _hueIntensity + 255) >> 8);
							b = (BYTE)((b * (255 - _hueIntensity) + GetBValue(_hueColor) * _hueIntensity + 255) >> 8);
						}

						SetPixelV(hdcBGCOPY, ix, iy, RGB(r,g,b));
					}
				}
			}

			BitBlt( hDC, x+iconX, y+iconY, iconSize, iconSize, hdcBGCOPY, 0, 0, SRCCOPY);
		}
				
		hbmMask = (HBITMAP) SelectObject(hdcMask, hbmMask);
		DeleteObject(hbmMask);
		DeleteDC(hdcMask);

		hbmBuffer = (HBITMAP) SelectObject(hdcBuffer, hbmBuffer);
		DeleteObject(hbmBuffer);
		DeleteDC(hdcBuffer);

		hbmBGCOPY = (HBITMAP) SelectObject(hdcBGCOPY, hbmBGCOPY);
		DeleteObject(hbmBGCOPY);
		DeleteDC(hdcBGCOPY);
	}
	else
		DrawIconEx(hDC, x+iconX, y+iconY, hIcon, iconSize, iconSize, 0, NULL, DI_NORMAL);

	if (hIcon)
		DestroyIcon(hIcon);
}
