/***************************************************************************
****************************************************************************

This is a part of the xModule Source code.

Copyright (C) 2003-2004 The LS-Universe Team

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

****************************************************************************
****************************************************************************/

#include "stdafx.h"
#include "Popup.h"

#include "ContextMenu.h"

#include "PopupTexture.h"
#include "ButtonTexture.h"
#include "ButtonIcon.h"
#include "ButtonFont.h"

#include <shlwapi.h>
#include <algorithm>

//////////////////////////////////////////////////////////////////////
// Defines
//////////////////////////////////////////////////////////////////////

extern PopupList popupList;
extern PopupList visiblepopupList;

extern int screenX;
extern int screenY;

extern int infoExport;

extern bool compatibilitymode;

extern bool cacheAllPopupsAtStartup;

extern int bitsperpixel;

#define GET_X_LPARAM(lp) ((int)(short)LOWORD(lp))
#define GET_Y_LPARAM(lp) ((int)(short)HIWORD(lp))

#define WM_MOUSEWHEEL 0x020A

  #define WM_XBUTTONDOWN 0x020B
  #define WM_XBUTTONUP 0x020C
  //#define WM_XBUTTONDBLCLK 0x020D

  #define MK_XBUTTON1 0x0020
  #define MK_XBUTTON2 0x0040
  #define XBUTTON1 0x0001
  #define XBUTTON2 0x0002

// undocumented SwitchToThisWindow API
void (WINAPI *SwitchToThisWindow)(HWND, int);

//Alpha Transparency
typedef DWORD (WINAPI *PSLWA)(HWND, DWORD, BYTE, DWORD);
static PSLWA pSetLayeredWindowAttributes = NULL;

typedef BOOL (WINAPI *PULW)(HWND hwnd, HDC hdcDst, POINT *pptDst, SIZE *psize, HDC hdcSrc, POINT *pptSrc, COLORREF crKey, BLENDFUNCTION *pblend, DWORD dwFlags);
static PULW pUpdateLayeredWindow = NULL;

static bool alphaInitialized = false;

#define TIMER_MOUSETRACK	1
#define TIMER_ALPHAFADEIN	2
#define TIMER_ALPHAFADEOUT	3
#define TIMER_UPDATE		4
#define TIMER_ACTIVE		5
#define TIMER_HOVER			6
#define TIMER_CHORDING		7

extern PopupTexture *defaultpopuptexture;
extern ButtonTexture *defaultbuttontexture;
extern ButtonIcon *defaultbuttonicon;
extern ButtonFont *defaultbuttonfont;

//////////////////////////////////////////////////////////////////////
// Konstruktion/Destruktion
//////////////////////////////////////////////////////////////////////

Popup::Popup(const string &name, string subname, PopupTexture *parentpopuptexture, ButtonTexture *parentbuttontexture, ButtonIcon *parentbuttonicon, ButtonFont *parentbuttonfont):actualx(0),actualy(0),actualwidth(0),actualheight(0)
{
	//Specifies the Cfg Prefix
	this->name = name;

	//Specifies the internal name (Caption/Title) plus "sub_" Prefix
	this->subname = subname;

		CString help = subname.c_str();
		help.Replace("sub_", "");

		CString help1 = help.Left(1);
		help1.MakeUpper();
	
	//Specifies the Caption/Title
	this->captionname = (help1 + help.Right(help.GetLength()-1));

	//Bang which calls this Popup (only Top Level Popups have this)
	BangToCall = "";
	//All *popup lines which belong to this Popup (will be deleted after Parse!)
	popupcontentList.clear();

	hWnd = 0;
	hInstance = 0;

	isTaskPopup = false;
	hasHookedWindows = false;
	hasInfoEntry = false;

	moveButtonDown = false;

	dropShadow = false;

	alwaysOnTop = true;

	openToLeft = false;

	pParentPopup = NULL;
	pParentEntry = NULL;

	startentry = NULL;
	lastactiveentry = NULL;
	hoverentry = NULL;

	lastkeyentry = -1;

	PinnedMode = false;

	defaultDC = NULL;
	defaultBitmap = NULL;

	alphaMap = false;
	//layeredFlag = false;

	currentPart = 0;
	maxPart = 0;
	maxMenuBreak = 0;

	dynamicPopup = false;
	rootPath = "";

	showExtensions = false;
	noshowExtensions.clear();

	execSoundPlayed = false;

	UpdateAlpha = 0;
	alphaFadeStep = 4;
	alphaFadeDelay = 10;

	mouseInside = false;
	mousePressed = false;
	mouseMoved = false;
	oldmovetracker.x = 0;
	oldmovetracker.y = 0;

	manualMove = true;
	
	visible = false;

	inChordingTimeOut = false;

	shaded = -1;

	alwaysOnTop = false;

	entryCount = -1;

	//Only Create Painting Classes, if it's a TopLevel Popup
	//-> Nothing is sent to the "SubPopup"
	if (parentpopuptexture == NULL)
		popuptexture = new PopupTexture();
	else
		popuptexture = parentpopuptexture;

	if (parentbuttontexture == NULL)
		buttontexture = new ButtonTexture();
	else
		buttontexture = parentbuttontexture;

	if (parentbuttonicon == NULL)
		buttonicon = new ButtonIcon();
	else
		buttonicon = parentbuttonicon;

	if (parentbuttonfont == NULL)
		buttonfont = new ButtonFont();
	else
		buttonfont = parentbuttonfont;

	SwitchToThisWindow = (void (WINAPI *)(HWND, int)) 
							GetProcAddress(GetModuleHandle("USER32.DLL"), "SwitchToThisWindow");
}

Popup::~Popup()
{
	if (hWnd && ToolTips)
	{
		updateHint(hWnd,"", false, true);
		DestroyWindow(ToolTips);
		ToolTips = 0;
	}

	if (!BangToCall.empty() /*-> TopLevel Popup*/ && stricmp(BangToCall.c_str(), "dynamicdummy") != NULL )
	{
		RemoveBangCommand(BangToCall.c_str());
		
		//Reposition Bang
		string bang = BangToCall;
		bang.append("Reposition");
		RemoveBangCommand(bang.c_str());
		
		//Pin Bang == Show
		bang = BangToCall;
		bang.append("Pin");
		RemoveBangCommand(bang.c_str());

		//UnPin Bang == Hide
		bang = BangToCall;
		bang.append("UnPin");
		RemoveBangCommand(bang.c_str());

		//Toggle PinnedMode == Toggle
		bang = BangToCall;
		bang.append("TogglePin");
		RemoveBangCommand(bang.c_str());

		//Toggle ShadeMode == Toggle
		bang = BangToCall;
		bang.append("ToggleShade");
		RemoveBangCommand(bang.c_str());

		//Toggle Visiblity == Toggle
		bang = BangToCall;
		bang.append("Toggle");
		RemoveBangCommand(bang.c_str());
	}

	for(PopupListIterator plit = subpopupList.begin(); plit != subpopupList.end(); plit++)
	{
		(*plit)->hide();
		delete *plit;
	}

	subpopupList.clear();

	for (EntryListIterator eit = entrylist.begin(); eit != entrylist.end(); ++eit)
		delete (*eit);

	entrylist.clear();

	tasklist.clear();

	// shutdown timer
	if(IsWindow(hWnd))
	{
		KillTimer(hWnd, TIMER_MOUSETRACK);
		KillTimer(hWnd, TIMER_ALPHAFADEIN);
		KillTimer(hWnd, TIMER_ALPHAFADEOUT);
		KillTimer(hWnd, TIMER_UPDATE);
		KillTimer(hWnd, TIMER_ACTIVE);
		KillTimer(hWnd, TIMER_HOVER);
		KillTimer(hWnd, TIMER_CHORDING);
	}

	if(IsWindow(hWnd))
	{
		DestroyWindow(hWnd);
	}

	if(defaultBitmap)
	{
		defaultBitmap = (HBITMAP) SelectObject(defaultDC, defaultBitmap);
		DeleteObject(defaultBitmap);
	}
	if (defaultDC)
		DeleteDC(defaultDC);

	if (!BangToCall.empty() /*-> TopLevel Popup*/) 
	{	
		if (popuptexture && popuptexture != defaultpopuptexture)
			delete popuptexture;
		if (buttontexture && buttontexture != defaultbuttontexture)
			delete buttontexture;
		if (buttonicon && buttonicon != defaultbuttonicon)
			delete buttonicon;
		if (buttonfont && buttonfont != defaultbuttonfont)
			delete buttonfont;
	}
}

void Popup::load(HINSTANCE hInstance)
{
	this->hInstance = hInstance;
	
	if (IsWindow(hWnd))	// window already exists...
		return;

	if ( IsOS(OS_XP) && GetxRCBool(name, "DropShadow") )
		dropShadow = true;

	hWnd = CreateWindowEx(WS_EX_TOOLWINDOW,
		( dropShadow ? "xPopupXP" : "xPopup" ),
		name.c_str(),
		WS_POPUP | WS_CLIPCHILDREN,
		actualx, actualy,		
		actualwidth, actualheight,
		GetLitestepWnd(), 
		0,
		hInstance,
		this);
		
	if (hWnd)
	{
		SetWindowLong(hWnd, GWL_USERDATA, magicDWord);

		reconfigure();
		
		setAlwaysOnTop(true);
	}
	else
		delete this;
}

void Popup::reconfigure()
{
	NameValuePair captionPositionValues[] = {
			{ ".none", 0 },
			{ ".top", 1 },
			{ ".bottom", 2 },
			{ ".right", 1 },
			{ ".left", 2 },
			{ 0, 0 }
		};

	NameValuePair quicklaunchValues[] = {
			{ ".none", 0 },
			{ ".horizontal", 1 },
			{ ".vertical", 2 },
			{ 0, 0 }
		};

	NameValuePair alphaFadeValues[] = {
			{ ".none", 0 },
			{ "false", 0 },
			{ ".allpopups", 1 },
			{ "true", 1 },
			{ ".mainpopups", 2 },
			{ 0, 0}
		};

	NameValuePair clickactionValues[] = {
			{ ".none", 0 },
			{ ".pin", 1 },
			{ ".unpin", 2 },
			{ ".togglepin", 3 },
			{ ".toggleshade", 4 },
			{ ".toggleontop", 5 },
			{ ".move", 6 },
			{ ".close", 7 },
			{ 0, 0}
		};

	if (!BangToCall.empty() /*-> TopLevel Popup*/ && stricmp(name.c_str(), "xpopup") != NULL)
	{
		popuptexture->configure(name);
		buttontexture->configure(name);
		buttonicon->configure(name);
		buttonfont->configure(name);
	}

	transparencyNeededAlways = buttontexture->getTrueTransparent();

	//---------------------------
	//Get DefaultIcons
	//---------------------------
	DefaultIcon = GetxRCString(name, "DefaultIcon", "");		
	DefaultFolderIcon = GetxRCString(name, "FolderIcon", "");

	SystemParametersInfo(SPI_GETWORKAREA, 0, &customWorkArea, 0);

	string workArea = GetxRCLine(name, "WorkArea", "");
	if ( !workArea.empty() )
	{
		char one[32], two[32], three[32], four[32];
		char *wbuffers[] = {one, two, three, four};

		if ( LCTokenize( workArea.c_str(), wbuffers, 4, NULL) == 4 )
		{
			customWorkArea.left = xParseCoord(one, customWorkArea.left, screenX);
			customWorkArea.right = xParseCoord(two, customWorkArea.right, screenX);
			customWorkArea.top = xParseCoord(three, customWorkArea.top, screenY);
			customWorkArea.bottom = xParseCoord(four, customWorkArea.bottom, screenY);
		}
	}

	restrictToWorkArea = GetxRCBool(name, "RestrictToWorkArea");

	showCaption = GetxRCNamedValue(name , "ShowCaption", captionPositionValues, 1);
	showExtensions = GetxRCBool(name , "ShowExtensions");
	if ( showExtensions )
		noshowExtensions = split(GetxRCString(name, "ShowExtensions", ""), "|");

	pinnedPopupNotOnTop = GetxRCBool(name , "PinnedPopupNotOnTop");
	HideAllowed = GetxRCBool(name , "ManualUnPin", true);
	MoveAllowed = GetxRCBool(name , "Moveable", true);
	ShadeAllowed = GetxRCBool(name , "Shadeable", true);
	AutoPin = GetxRCBool(name , "PinOnMove", true);

	folderOpenOnClick = GetxRCBool(name , "FolderOpenOnClick");

	openToLeft = GetxRCBool(name, "OpenToLeft");

	quicklaunchMode = GetxRCNamedValue(name, "QuicklaunchMode", quicklaunchValues, 0);
	quicklaunchItemSize = GetxRCInt(name, "QuicklaunchItemSize", 32, 16, 64);
	quicklaunchWrapCount = GetxRCInt(name, "QuicklaunchWrapCount", 100, 1);
	quicklaunchOpenTo = GetxRCNamedValue(name, "QuicklaunchOpenTo", captionPositionValues, 1);

	openSound = GetxRCString(name, "OpenSound", "");
	closeSound = GetxRCString(name, "CloseSound", "");
	subSound = GetxRCString(name, "SwitchSound", "");
	execSound = GetxRCString(name, "ActionSound", "");

	dropCommand = GetxRCLine(name, "OnDrop", "");
	openCommand = GetxRCLine(name, "OnOpen", "");
	closeCommand = GetxRCLine(name, "OnClose", "");
	shadeCommand = GetxRCLine(name, "OnShade", "");
	unshadeCommand = GetxRCLine(name, "OnUnShade", "");

	titleLClickAction = GetxRCNamedValue(name, "OnCaptionLeftClick", clickactionValues, 6);
	titleMClickAction = GetxRCNamedValue(name, "OnCaptionMiddleClick", clickactionValues, 0);
	titleRClickAction = GetxRCNamedValue(name, "OnCaptionRightClick", clickactionValues, 4);

	//File Actions
	if (dropCommand.empty())
		DragAcceptFiles(hWnd, false);
	else
		DragAcceptFiles(hWnd, true);

	leftBorder = GetxRCInt(name, "LeftBorder", 0, 0);
	topBorder = GetxRCInt(name, "TopBorder", 0, 0);
	rightBorder = GetxRCInt(name, "RightBorder", 0, 0);
	bottomBorder = GetxRCInt(name, "BottomBorder", 0, 0);

	overlapX = GetxRCInt(name, "OverlapX", 0);
	overlapY = GetxRCInt(name, "OverlapY", -topBorder);

	entryHeight = GetxRCInt(name, "EntryHeight", buttontexture->getEntryHeight(), 0);
	folderHeight = GetxRCInt(name, "FolderHeight", buttontexture->getFolderHeight(), 0);
	infoHeight = GetxRCInt(name, "InfoHeight", entryHeight, 0);
	separatorHeight = GetxRCInt(name, "SeparatorHeight", buttontexture->getSeparatorHeight(), 0);

	autoSeparator = GetxRCBool(name , "AutoSeparator");
	autoMenuBreak = GetxRCBool(name , "AutoMenuBreak");

	maxWidth = GetxRCInt(name, "MaxWidth", screenX, 0, customWorkArea.right-customWorkArea.left );
	maxHeight = GetxRCInt(name, "MaxHeight", screenY, 0, customWorkArea.bottom-customWorkArea.top);
	minWidth = GetxRCInt(name, "MinWidth", 0, 0, screenX);
	minHeight = GetxRCInt(name, "MinHeight", max(topBorder+bottomBorder, popuptexture->getMinNeededHeight()), max(topBorder+bottomBorder, popuptexture->getMinNeededHeight()), screenY);
	orgminHeight = minHeight;

	maxEntryWidth = GetxRCInt(name, "MaxEntryWidth", screenX, 0, screenX);
	minEntryWidth = GetxRCInt(name, "MinEntryWidth", 0, 0, screenX);

	xSpacing = GetxRCInt(name, "xSpacing", 0, 0);
	ySpacing = GetxRCInt(name, "ySpacing", 0, 0);

	hideDelay = GetxRCInt(name, "HideDelay", 250, 100);
	trackDelay = GetxRCInt(name, "TrackingDelay", 50, 10);
	openDelay = GetxRCInt(name, "FolderOpenDelay", 1, 1);
	tasksUpdate = GetxRCInt(name, "TasksUpdateInterval", 5000, 1000);

	if ( IsOS(OS_2KXP) )
	{
		alphaTransparency = GetxRCInt(name, "AlphaTransparency", 255, 0, 255);

		if ( !GetxRCBool(name, "UseSolidColors") && !GetxRCString(name, "Image", "").empty() )
			alphaMap = GetxRCBool(name, "AlphaMap");

		char first[32], second[32];
		char *buffers[] = {first, second};

		int numTokens = 0;	
		string config = "";
		//---------------------------------------------------------------
		//---------------------------------------------------------------
		config = GetxRCLine(name, "CustomAlphaFade", "");

		numTokens = LCTokenize( config.c_str(), buffers, 2, NULL);

		if ( config.empty() )
			alphaFade = false;
		else if (numTokens == 1)
		{
			alphaFadeStep = atoi( first );
			if (alphaFadeStep < 1 || alphaFadeStep > 255)
				alphaFadeStep = 2;
			alphaFade = true;
		}
		else if (numTokens == 2)
		{
			alphaFadeStep = atoi( first );
			if (alphaFadeStep < 1 || alphaFadeStep > 255)
				alphaFadeStep = 2;
			alphaFadeDelay = atoi( second );
			if (alphaFadeDelay < 1 || alphaFadeDelay > 500)
				alphaFadeDelay = 10;
			alphaFade = true;
		}

		int value = GetxRCNamedValue(name, "AlphaFade", alphaFadeValues, -1);

		if (value == -1)
			alphaFade = GetxRCBool(name, "AlphaFade", alphaFade);
		else if ( value == 2 )
		{
			if ( BangToCall.empty() )
				alphaFade = false;
			else
				alphaFade = true;
		}
		else if ( value == 1 )
			alphaFade = true;
		//---------------------------------------------------------------
	}
	else if ( !IsOS(OS_2KXP) )
	{
		alphaTransparency = 255;
		alphaFade = false;
		alphaMap = false;
	}

	//---------------------------------------------------------------
	//---------------------------------------------------------------
	if ( IsOS(OS_2KXP) && (alphaTransparency != 255 || alphaFade || alphaMap) )
	{
		ModifyStyleEx(hWnd, 0, WS_EX_LAYERED);

		//layeredFlag = true;

		if (alphaFade)
			UpdateWindowTransparent(hWnd, 0);
		else
			UpdateWindowTransparent(hWnd, alphaTransparency);	
	}

	if ( quicklaunchMode != 0 )
	{
		ToolTips = CreateWindow(TOOLTIPS_CLASS, NULL, TTS_ALWAYSTIP,
								0, 0, 0, 0,
								NULL, NULL, hInstance, NULL);

		SetWindowPos( ToolTips, HWND_TOPMOST, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE | SWP_NOACTIVATE);
		
		if (ToolTips != 0)
		{
			TOOLINFO ti;
			RECT clientRect;

			GetClientRect(hWnd, &clientRect);
			ti.cbSize = sizeof(TOOLINFO);
			ti.uFlags = TTF_SUBCLASS;
			ti.hwnd = hWnd;
			ti.uId = 0;
			ti.rect = clientRect;
			ti.hinst = hInstance;
			ti.lpszText = 0;
			ti.lParam = 0;

			SendMessage(ToolTips, TTM_ADDTOOL, 0, LPARAM(&ti));
		}

		updateHint(hWnd, "", true);
	}

	updateRegion();
}

//////////////////////////////////////////////////////////////////////
// Styles
//////////////////////////////////////////////////////////////////////

void Popup::setAlwaysOnTop(bool newalwaysOnTop)
{
	if( newalwaysOnTop != alwaysOnTop )
	{
		//ModifyStyle(hWnd, WS_POPUP, WS_CHILD);
		//SetParent(hWnd, alwaysOnTop ? 0 : GetLitestepDesktop());
		//ModifyStyle(hWnd, WS_CHILD, WS_POPUP);

		alwaysOnTop = newalwaysOnTop;
		
		SetWindowPos(hWnd, newalwaysOnTop ? HWND_TOPMOST : HWND_NOTOPMOST,
			0, 0, 0, 0,
			SWP_NOACTIVATE | SWP_NOMOVE | SWP_NOSIZE );
	}
}

/*void Popup::setAlpha(int newAlphaTransparency)
{
	if ( newAlphaTransparency >= 0 && newAlphaTransparency <= 255 && IsOS(OS_2KXP) )
	{
		if (!layeredFlag)
		{
			ModifyStyleEx(hWnd, 0, WS_EX_LAYERED);
			layeredFlag = true;
		}

		alphaTransparency = newAlphaTransparency;

		UpdateWindowTransparent(hWnd, newAlphaTransparency);

		for(PopupListIterator plit = subpopupList.begin(); plit != subpopupList.end(); plit++)
		{
			(*plit)->setAlpha(newAlphaTransparency);
		}
	}
}*/

void Popup::UpdateWindowTransparent(HWND hWnd, int factor)
{
	/* First, see if we can get the API call we need. If we've tried
    * once, we don't need to try again. */
	if (!alphaInitialized)
	{
		HMODULE hDLL = LoadLibrary ("user32");

		pSetLayeredWindowAttributes = 
					(PSLWA) GetProcAddress(hDLL, "SetLayeredWindowAttributes");

		pUpdateLayeredWindow = 
					(PULW) GetProcAddress(hDLL, "UpdateLayeredWindow");

		alphaInitialized = TRUE;
   
		if (pSetLayeredWindowAttributes == NULL || pUpdateLayeredWindow == NULL)
		{
		    ModifyStyleEx(hWnd, WS_EX_LAYERED, 0 );

			alphaTransparency = 255;
			alphaMap = false;
			alphaFade = false;

			return;
		}
	}

	/* Now, we need to set the 'layered window attributes'. This
	 * is where the alpha values get set. */
  	if (alphaMap)
	{
		HDC backgroundDC = NULL;
		HBITMAP backgroundBitmap = NULL;
		HDC bufferDC = NULL;
		HBITMAP bufferBitmap = NULL;

		//------------------------------------------------------------------
		RECT rectPosition;
		GetWindowRect(hWnd, &rectPosition);
		actualx = rectPosition.left;
		actualy = rectPosition.top;
		actualwidth = rectPosition.right - rectPosition.left;
		actualheight = rectPosition.bottom - rectPosition.top;

		// make a rendition of the background
		backgroundDC = CreateCompatibleDC(NULL);
				
		BITMAPINFO alpha;
		ZeroMemory( &alpha.bmiHeader, sizeof(BITMAPINFOHEADER) );
		alpha.bmiHeader.biWidth=actualwidth;      // Set size you need
		alpha.bmiHeader.biHeight=actualheight;    // Set size you need
		alpha.bmiHeader.biPlanes=1;
		alpha.bmiHeader.biBitCount=32;            // Can be 8, 16, 32 bpp or other number
		alpha.bmiHeader.biSizeImage=0;
		alpha.bmiHeader.biSize=sizeof(BITMAPINFOHEADER);
		alpha.bmiHeader.biClrUsed= 0;
		alpha.bmiHeader.biClrImportant= 0;
		VOID *pvBits;

		backgroundBitmap = CreateDIBSection( backgroundDC,
											&alpha,
											DIB_RGB_COLORS,
											&pvBits,
											NULL,
											0 );

		SelectObject(backgroundDC, backgroundBitmap);
				
		//Normal Background Painting
		popuptexture->apply(backgroundDC, 0, 0, actualwidth, actualheight);
			
		// create & keep a cached rendition of the default popupmenu
		if(!defaultBitmap)
		{
			if(!defaultDC)
				defaultDC = CreateCompatibleDC(NULL);

			BITMAPINFO alpha;
			ZeroMemory( &alpha.bmiHeader, sizeof(BITMAPINFOHEADER) );
			alpha.bmiHeader.biWidth=actualwidth;      // Set size you need
			alpha.bmiHeader.biHeight=actualheight;    // Set size you need
			alpha.bmiHeader.biPlanes=1;
			alpha.bmiHeader.biBitCount=32;            // Can be 8, 16, 32 bpp or other number
			alpha.bmiHeader.biSizeImage=0;
			alpha.bmiHeader.biSize=sizeof(BITMAPINFOHEADER);
			alpha.bmiHeader.biClrUsed= 0;
			alpha.bmiHeader.biClrImportant= 0;
			VOID *pvBits;

			defaultBitmap = CreateDIBSection( defaultDC,
											&alpha,
											DIB_RGB_COLORS,
											&pvBits,
											NULL,
											0 );

			SelectObject(defaultDC, defaultBitmap);
				
			// blt background into double buffer
			BitBlt(defaultDC, 0, 0, actualwidth, actualheight, backgroundDC, 0, 0, SRCCOPY);

			//Only Painted if NOT Shaded
			if ( shaded == -1 )
			{
				int state = 0;
				int arrowstate = 0;
				for (EntryListIterator eit = entrylist.begin(); eit != entrylist.end(); ++eit)
				{
					if ( currentPart == (*eit)->hiddenPart && (*eit)->entryType != 2 )
					{
						//initialize
						state = 0;
						arrowstate = 0;

						if ( (*eit)->entryType == 1 )
						{
							state = 10;
						}
						else if ( (*eit)->entryType >= 0 )
						{
							state = 1;
						}
						else if ( (*eit)->entryType < 0 )
						{
							state = 3;
							arrowstate = 31;
						}
						else
							continue;
						
						//Paint Entry Texture
						buttontexture->apply( defaultDC, (*eit)->x, (*eit)->y, (*eit)->width, (*eit)->height, state);
						//Paint Arrow
						if (arrowstate != 0)
						{
							buttontexture->apply( defaultDC, (*eit)->x, (*eit)->y, (*eit)->width, (*eit)->height, arrowstate);
							buttonfont->apply( defaultDC, (*eit)->x, (*eit)->y, (*eit)->width, (*eit)->height, (*eit)->caption, arrowstate );
						}

						//If No Separator, Icon and Text Painting is valid
						if (state != 10)
						{
							buttonicon->apply( defaultDC, (*eit)->x, (*eit)->y, (*eit)->width, (*eit)->height, state, (*eit)->iconPath, (*eit)->hWnd, (*eit)->pidl, DefaultIcon, alphaMap );

							if (quicklaunchMode == 0)
								buttonfont->apply( defaultDC, (*eit)->x, (*eit)->y, (*eit)->width, (*eit)->height, (*eit)->caption, state );
						}
					}
				}
			}

			//Prepare Combined Image
			BITMAP bm;
			GetObject(defaultBitmap, sizeof(BITMAP), &bm);

			if ((bm.bmBits != 0) && (bm.bmBitsPixel == 32))
			{
				BYTE *pPixel = (LPBYTE)bm.bmBits;

				for (int j = 0; j < bm.bmHeight * bm.bmWidth; ++j)
				{
					//Convert Text to Full Opacity
					if (pPixel[3] == 0)
						pPixel[3] = 255;

					pPixel+= 4;
				}
			}
		}

		// double buffer for flicker-free paint
		bufferDC = CreateCompatibleDC(NULL);

		ZeroMemory( &alpha.bmiHeader, sizeof(BITMAPINFOHEADER) );
		alpha.bmiHeader.biWidth=actualwidth;      // Set size you need
		alpha.bmiHeader.biHeight=actualheight;    // Set size you need
		alpha.bmiHeader.biPlanes=1;
		alpha.bmiHeader.biBitCount=32;            // Can be 8, 16, 32 bpp or other number
		alpha.bmiHeader.biSizeImage=0;
		alpha.bmiHeader.biSize=sizeof(BITMAPINFOHEADER);
		alpha.bmiHeader.biClrUsed= 0;
		alpha.bmiHeader.biClrImportant= 0;
			
		bufferBitmap = CreateDIBSection( bufferDC,
											&alpha,
											DIB_RGB_COLORS,
											&pvBits,
											NULL,
											0 );

		SelectObject(bufferDC, bufferBitmap);

		// blt default background into double buffer
		BitBlt(bufferDC, 0, 0, actualwidth, actualheight, defaultDC, 0, 0, SRCCOPY);

		if (showCaption == 1)
			buttonfont->apply( bufferDC, 0, 0, actualwidth, topBorder, captionname, 0);
		else if (showCaption == 2)
			buttonfont->apply( bufferDC, 0, actualheight-bottomBorder, actualwidth, bottomBorder, captionname, 0);

		//Only Painted if NOT Shaded
		if ( shaded == -1 )
		{
			int state = 0;
			int arrowstate = 0;
			if ( quicklaunchMode == 0 && hasInfoEntry )
			{
				for (EntryListIterator eit = entrylist.begin(); eit != entrylist.end(); ++eit)
				{
					if ( currentPart == (*eit)->hiddenPart && (*eit)->entryType == 2 )
					{
						state = 5;

						//Normal Background Painting behind ActiveEntry
						BitBlt(bufferDC, (*eit)->x, (*eit)->y, (*eit)->width, (*eit)->height, backgroundDC, (*eit)->x, (*eit)->y, SRCCOPY);

						//Paint Entry Texture
						buttontexture->apply( bufferDC, (*eit)->x, (*eit)->y, (*eit)->width, (*eit)->height, state);

						buttonfont->apply( bufferDC, (*eit)->x, (*eit)->y, (*eit)->width, (*eit)->height, (*eit)->caption, state );
					}
				}
			}
			for (EntryListIterator eit = entrylist.begin(); eit != entrylist.end(); ++eit)
			{
				//initialize
				state = 0;
				arrowstate = 0;

				if ( currentPart == (*eit)->hiddenPart && (*eit) == hoverentry )
				{
					if ( (*eit)->entryType == 0 )
					{
						state = 2;
					}
					else if ( (*eit)->entryType < 0 )
					{
						state = 4;
						arrowstate = 41;
					}
					else
						continue;
						
					//Normal Background Painting behind ActiveEntry
					BitBlt(bufferDC, (*eit)->x, (*eit)->y, (*eit)->width, (*eit)->height, backgroundDC, (*eit)->x, (*eit)->y, SRCCOPY);

					//Paint Entry Texture
					buttontexture->apply( bufferDC, (*eit)->x, (*eit)->y, (*eit)->width, (*eit)->height, state);
					//Paint Arrow
					if (arrowstate != 0)
					{
						buttontexture->apply( bufferDC, (*eit)->x, (*eit)->y, (*eit)->width, (*eit)->height, arrowstate);
						buttonfont->apply( bufferDC, (*eit)->x, (*eit)->y, (*eit)->width, (*eit)->height, (*eit)->caption, arrowstate );
					}

					//If No Info, Icon is valid
					buttonicon->apply( bufferDC, (*eit)->x, (*eit)->y, (*eit)->width, (*eit)->height, state, (*eit)->iconPath, (*eit)->hWnd, (*eit)->pidl, DefaultIcon, alphaMap );
					if (quicklaunchMode == 0)
						buttonfont->apply( bufferDC, (*eit)->x, (*eit)->y, (*eit)->width, (*eit)->height, (*eit)->caption, state );
				}
			}
		}

		//Hooked Child Window AlphaMap Painting
		if ( hasHookedWindows )
		{
			for(HWND hWndChild = ::GetTopWindow(hWnd); hWndChild != NULL; hWndChild = ::GetNextWindow(hWndChild, GW_HWNDNEXT))
			{
				SendMessage(hWndChild, LM_EXTERNALDRAW, (long) bufferDC, 0 );
			}
		}

		//Prepare Combined Image
		BITMAP bm;
		GetObject(bufferBitmap, sizeof(BITMAP), &bm);

		if ((bm.bmBits != 0) && (bm.bmBitsPixel == 32))
		{
			BYTE *pPixel = (LPBYTE)bm.bmBits;

			for (int j = 0; j < bm.bmHeight * bm.bmWidth; ++j)
			{
				//Convert Text to Full Opacity
				if (pPixel[3] == 0)
					pPixel[3] = 255;

				//Magic Pink
				if (hasHookedWindows && pPixel[0] == 255 && pPixel[1] == 0 && pPixel[2] == 255)
					pPixel[3] = 0;

				//Remove Prepared Alpha (1 back to 0)
				if (pPixel[3] == 1)
					pPixel[3] = 0;

				pPixel+= 4;
			}
		}
		//------------------------------------------------------------------

		POINT ptWindowScreenPosition = {actualx, actualy};
		SIZE szWindow = {actualwidth, actualheight};

		POINT ptSrc = {0,0};

		BLENDFUNCTION blendPixelFunction = { AC_SRC_OVER, 0, factor, AC_SRC_ALPHA };

		pUpdateLayeredWindow(hWnd, NULL, &ptWindowScreenPosition, &szWindow, bufferDC, &ptSrc, 0, &blendPixelFunction, ULW_ALPHA );
	
		if(backgroundBitmap)
		{
			backgroundBitmap = (HBITMAP) SelectObject(backgroundDC, backgroundBitmap);
			DeleteObject(backgroundBitmap);
		}
		if (backgroundDC)
			DeleteDC(backgroundDC);	
		
		if(bufferBitmap)
		{
			bufferBitmap = (HBITMAP) SelectObject(bufferDC, bufferBitmap);
			DeleteObject(bufferBitmap);
		}
		if (bufferDC)
			DeleteDC(bufferDC);	
	}
	else
		pSetLayeredWindowAttributes(hWnd, 0, factor, LWA_ALPHA);
}

//////////////////////////////////////////////////////////////////////
// Popup Creation -> StepParsing, Folder Scanning, Shell Scanning, ...
//////////////////////////////////////////////////////////////////////

void Popup::ScanKeyCodes(ENTRY* processEntry, string processString)
{
	//Ampersand Keyboard navigation
	//------------------------------------------------------------
	char* m_pszTitle = strdup(processString.c_str());
	// search the string for '&', that is keyboard-shortcuts
	int len = lstrlen(m_pszTitle);
	for (int i = 0; i < len; i++)
	{
		if (m_pszTitle[i] == '&' && m_pszTitle[i + 1] != '\0')
		{
			processEntry->keyCode = islower(m_pszTitle[i + 1]) ? toupper(m_pszTitle[i + 1]) : m_pszTitle[i + 1];
			break;
		}
	}
	
	if (processEntry->keyCode == -1)
	{
		processEntry->keyCode = islower(m_pszTitle[0]) ? toupper(m_pszTitle[0]) : m_pszTitle[0];
		processEntry->keyCode1 = islower(m_pszTitle[1]) ? toupper(m_pszTitle[1]) : m_pszTitle[1];
	}
	//------------------------------------------------------------
}

void Popup::ParseStepRC(bool externCall)
{
	StringList hookcontentList;
	StringList settingsList = popupcontentList;

	if (!settingsList.empty() && settingsList.size() == 1)
	{
		char szFirst[MAX_PATH_LENGTH] = { 0 };
		char szSecond[MAX_PATH_LENGTH] = { 0 };
		char szThird[MAX_PATH_LENGTH] = { 0 };

		char *buffer[] = { szFirst, szSecond, szThird };

		int nTokens = LCTokenize((*settingsList.begin()).c_str(), buffer, 2, szThird);

		CString str_third = TrimAndStripQuote((string)szThird).c_str();

		if ( stricmp(str_third.Left(19), "!popupdynamicfolder") == NULL || stricmp(str_third.Left(14), "!dynamicfolder") == NULL || stricmp(str_third.Left(12), "!popupfolder") == NULL 
				|| stricmp(str_third.Left(20), "!xpopupdynamicfolder") == NULL || stricmp(str_third.Left(13), "!xpopupfolder") == NULL)
		{
			CString easyHelper = TrimAndStripQuote( (string)szThird ).c_str();

			// Get FolderPath
			easyHelper = easyHelper.Right(easyHelper.GetLength()-easyHelper.Find(":")-1);

			// Strip surrounding '"'
			easyHelper = TrimAndStripQuote( (string)easyHelper ).c_str();

			ScanSubMenu( (string)easyHelper, -2 );

			return;
		}
		else if ( quicklaunchMode == 0 && (stricmp(str_third, "!popuptasks") == NULL || stricmp(str_third, "!xpopuptasks") == NULL) )
		{
			ScanTasks();

			return;
		}
		else if ( quicklaunchMode == 0 && (stricmp(str_third, "!popupcontrolpanel") == NULL || stricmp(str_third, "!popupprinters") == NULL 
						|| stricmp(str_third, "!popuprecyclebin") == NULL || stricmp(str_third, "!popuprecentdocuments") == NULL
						|| stricmp(str_third, "!popupnetwork") == NULL || stricmp(str_third, "!popupmycomputer") == NULL 
						|| stricmp(str_third, "!xpopupcontrolpanel") == NULL || stricmp(str_third, "!xpopupprinters") == NULL 
						|| stricmp(str_third, "!xpopuprecyclebin") == NULL || stricmp(str_third, "!xpopuprecentdocuments") == NULL
						|| stricmp(str_third, "!xpopupnetwork") == NULL || stricmp(str_third, "!xpopupmycomputer") == NULL) )
		{

			if ( stricmp(str_third, "!popupcontrolpanel") == NULL || stricmp(str_third, "!xpopupcontrolpanel") == NULL )
				ScanShellFolder( CSIDL_CONTROLS );
			else if (stricmp(str_third, "!popupmycomputer") == NULL || stricmp(str_third, "!xpopupmycomputer") == NULL )
				ScanShellFolder( CSIDL_DRIVES );
			else if (stricmp(str_third, "!popupnetwork") == NULL || stricmp(str_third, "!xpopupnetwork") == NULL )
				ScanShellFolder( CSIDL_NETWORK );
			else if (stricmp(str_third, "!popupprinters") == NULL || stricmp(str_third, "!xpopupprinters") == NULL )
				ScanShellFolder( CSIDL_PRINTERS );
			else if (stricmp(str_third, "!popuprecentdocuments") == NULL || stricmp(str_third, "!xpopuprecentdocuments") == NULL )
				ScanShellFolder( CSIDL_RECENT );
			else if (stricmp(str_third, "!popuprecyclebin") == NULL || stricmp(str_third, "!xpopuprecyclebin") == NULL )
				ScanShellFolder( CSIDL_BITBUCKET );
		
			return;
		}
	}
	
	if (!settingsList.empty())
	{
		char szFirst[MAX_PATH_LENGTH] = { 0 };
		char szSecond[MAX_PATH_LENGTH] = { 0 };
		char szThird[MAX_PATH_LENGTH] = { 0 };

		char *buffer[] = { szFirst, szSecond, szThird };

		//Initializing of Values
		int nextXPosition = leftBorder;
		int nextYPosition = topBorder;
		int hiddenPart = 0;
		int menuBreaks = 0;
		int maxNeededWidth = 0;
		int adaptedmaxHeight = 0;
		bool customIcon = false;
		int nTokens = 0;

		//For Folder ~Folder
		//--------------------------------------
		Popup* addsettingPopup = NULL;
		string addsettingCustomIcon = "";
		bool input = false;
		int subfolderopen = 0;
		StringList sendToPopup;
		//--------------------------------------

		//Helper CStrings
		CString str_first = "";
		CString str_second = "";
		CString str_third = "";

		CString easyHelper = "";

		for (StringListIterator sit = settingsList.begin(); sit != settingsList.end(); ++sit)
		{
			//Read Settings line ( *(x)Popup
			nTokens = LCTokenize((*sit).c_str(), buffer, 2, szThird);

			//Icon
			str_first = szFirst;
			str_first.MakeLower();
			str_first.Replace(".icon=", "");
			str_first = TrimAndStripQuote( (string)str_first ).c_str();

			if ( stricmp(str_first, ".noexist") != NULL )
			{
				str_first.Replace("/", "\\");
				str_first.Replace("\\\\", "\\");
				customIcon = true;
			}
			else
				customIcon = false;
		
			//Caption
			str_second = szSecond;
			//Action
			str_third = TrimAndStripQuote( (string) szThird ).c_str();

			//-----------------------------------------------------
			// MenuBreak
			//-----------------------------------------------------
			if ( nTokens >= 2 && (stricmp(str_second, "Menubreak") == NULL || stricmp(str_third, "Menubreak") == NULL) )
			{
				nextXPosition = nextXPosition+maxNeededWidth+xSpacing;

				if (adaptedmaxHeight < nextYPosition-ySpacing+bottomBorder)
					adaptedmaxHeight = nextYPosition-ySpacing+bottomBorder;

				nextYPosition = topBorder;

				menuBreaks++;
			}
			//-----------------------------------------------------
			// Manually Created SubFolder START
			//-----------------------------------------------------
			else if ( nTokens >= 2 && (stricmp(str_second, "Folder") == NULL || stricmp(str_third, "Folder") == NULL) )
			{
				if (input)
				{
					//Add to specific Popup Settings list
					sendToPopup.insert(sendToPopup.end(), *sit);
					subfolderopen++;
				}
				else
				{
					//Search for Popup Start with !New and Create New Popup
					Popup *popup = NULL;
					if (stricmp(str_second, "Folder") == NULL)
					{
						popup = new Popup(name, "sub_dummy", popuptexture, buttontexture, buttonicon, buttonfont);
					}
					if (stricmp(str_third, "Folder") == NULL)
					{
						easyHelper = ("sub_" + str_second);
						popup = new Popup(name, (string)easyHelper, popuptexture, buttontexture, buttonicon, buttonfont);
					}
			
					if (popup)
					{
						subpopupList.insert(subpopupList.end(), popup);
								
						addsettingPopup = popup;
						addsettingCustomIcon = str_first;

						//Allow following settings to be tracked
						input = true;

						continue;
					}
				}
			}
			//-----------------------------------------------------
			// Manually Created SubFolder END and CREATION
			//-----------------------------------------------------
			else if (nTokens >= 1 && (stricmp(str_first, "~Folder") == NULL || stricmp(str_second, "~Folder") == NULL))
			{
				bool closeFolder = false;
				if (input)
				{
					//Add to specific Popup Settings list
					if (subfolderopen > 0)
					{
						sendToPopup.insert(sendToPopup.end(), *sit);
						subfolderopen--;
					}
					else
						closeFolder = true;
				}
				
				if (!input || closeFolder)
				{
					//Close Popup Definition, give Settings and Load
					if (addsettingPopup)
					{
						addsettingPopup->load(hInstance);
						addsettingPopup->popupcontentList = sendToPopup;
						addsettingPopup->ParseStepRC();

						ENTRY* newitem = new ENTRY;

						//This is the parent for the Created PopupSubMenu
						addsettingPopup->pParentPopup = this;
						addsettingPopup->pParentEntry = newitem;

						newitem->entryType = -1;

						newitem->x = nextXPosition;
						newitem->y = nextYPosition;
						newitem->height = folderHeight;

						if (nextYPosition+newitem->height+bottomBorder > maxHeight)
						{
							if (autoMenuBreak)
							{
								if (adaptedmaxHeight < nextYPosition-ySpacing+bottomBorder)
									adaptedmaxHeight = nextYPosition-ySpacing+bottomBorder;

								nextXPosition = nextXPosition+maxNeededWidth+xSpacing;
								nextYPosition = topBorder;

								newitem->x = nextXPosition;
								newitem->y = nextYPosition;

								menuBreaks++;

								nextYPosition = nextYPosition+newitem->height+ySpacing;
							}
							else
							{
								if (adaptedmaxHeight < nextYPosition-ySpacing+bottomBorder)
									adaptedmaxHeight = nextYPosition-ySpacing+bottomBorder;
								
								nextYPosition = topBorder;

								newitem->y = nextYPosition;

								hiddenPart++;
								
								nextYPosition = nextYPosition+newitem->height+ySpacing;
							}
						}
						else
							nextYPosition = nextYPosition+newitem->height+ySpacing;
						
						newitem->hiddenPart = hiddenPart;
						newitem->multiPart = menuBreaks;
						
						easyHelper = addsettingPopup->subname.c_str();
						easyHelper.Replace("sub_", "");

						newitem->caption = easyHelper;

						ScanKeyCodes(newitem, (string) easyHelper);

						if ( DefaultFolderIcon.empty() )
						{
							if ( stricmp(addsettingCustomIcon.c_str(), ".noexist") == NULL )
								newitem->iconPath = "*";
							else
								newitem->iconPath = addsettingCustomIcon;
						}
						else
							newitem->iconPath = DefaultFolderIcon;

						POINT tmp = buttonfont->measure(defaultDC, newitem->caption);
						if (maxNeededWidth < tmp.x)
							maxNeededWidth = tmp.x;

						entrylist.insert(entrylist.end(), newitem);
					}
					addsettingPopup = NULL;
					addsettingCustomIcon = "";
					input = false;
					sendToPopup.clear();
				}
			}
			//-----------------------------------------------------
			// SEPARATOR
			//-----------------------------------------------------
			else if ( stricmp(str_second, "!separator") == NULL || stricmp(str_second, "!seperator") == NULL )
			{
				if (input)
				{
					//Add to specific Popup Settings list
					sendToPopup.insert(sendToPopup.end(), *sit);	
				}
				else
				{
					ENTRY* newitem = new ENTRY;	
					
					newitem->entryType = 1;

					newitem->x = nextXPosition;
					newitem->y = nextYPosition;
					newitem->height = separatorHeight;
					
					if (nextYPosition+newitem->height+bottomBorder > maxHeight)
					{
						if (autoMenuBreak)
						{
							if (adaptedmaxHeight < nextYPosition-ySpacing+bottomBorder)
								adaptedmaxHeight = nextYPosition-ySpacing+bottomBorder;

							nextXPosition = nextXPosition+maxNeededWidth+xSpacing;
							nextYPosition = topBorder;

							newitem->x = nextXPosition;
							newitem->y = nextYPosition;

							menuBreaks++;

							nextYPosition = nextYPosition+newitem->height+ySpacing;
						}
						else
						{
							if (adaptedmaxHeight < nextYPosition-ySpacing+bottomBorder)
								adaptedmaxHeight = nextYPosition-ySpacing+bottomBorder;
							
							nextYPosition = topBorder;

							newitem->y = nextYPosition;

							hiddenPart++;
							
							nextYPosition = nextYPosition+newitem->height+ySpacing;
						}
					}
					else
						nextYPosition = nextYPosition+newitem->height+ySpacing;
						
					newitem->hiddenPart = hiddenPart;
					newitem->multiPart = menuBreaks;

					entrylist.insert(entrylist.end(), newitem);
				}
			}
			//-----------------------------------------------------
			// MODULE HOOK
			//-----------------------------------------------------
			else if ( quicklaunchMode == 0 && stricmp(str_second.Left(5), "!hook") == NULL )
			{
				if (input)
				{
					//Add to specific Popup Settings list
					sendToPopup.insert(sendToPopup.end(), *sit);	
				}
				else
				{
					str_second = str_second.Right(str_second.GetLength()-6);
					int hookleft = atoi( str_second.Left(str_second.Find(":")) );
					str_second = str_second.Right(str_second.GetLength()-str_second.Find(":")-1);
					int hookright = atoi(str_second.Left(str_second.Find(":")));
					str_second = str_second.Right(str_second.GetLength()-str_second.Find(":")-1);
					int hookwidth = atoi(str_second.Left(str_second.Find(":")));
					str_second = str_second.Right(str_second.GetLength()-str_second.Find(":")-1);
					int hookheight = atoi(str_second);

					string hWndbuffer = IntToString( (int) hWnd );
					string config = str_third;

					config.append( " \"$themedir$theme.rc\" " );
					config.append( hWndbuffer );

					LSExecute(NULL, config.c_str(), SW_SHOWNORMAL);

					if (hookleft == 0 && hookright == 0 && hookwidth == 0 && hookheight == 0)
						continue;

					if (nextYPosition-ySpacing+hookheight+bottomBorder > maxHeight)
					{
						if (autoMenuBreak)
						{
							if (adaptedmaxHeight < nextYPosition-ySpacing+bottomBorder)
								adaptedmaxHeight = nextYPosition-ySpacing+bottomBorder;

							nextXPosition = nextXPosition+maxNeededWidth+xSpacing;
							nextYPosition = topBorder;

							easyHelper = str_third;
							if (easyHelper.Find("%%") != -1)
							{
								easyHelper = easyHelper.Right(easyHelper.GetLength()-easyHelper.Find("%%")-2);
								easyHelper = easyHelper.Left(easyHelper.Find("%%"));
								easyHelper += " ";
								easyHelper += IntToString(hookleft).c_str();
								easyHelper += " ";
								easyHelper += IntToString(nextXPosition).c_str();
								easyHelper += " ";
								easyHelper += IntToString(nextYPosition).c_str();
							
								hookcontentList.insert(hookcontentList.end(), (string)easyHelper);
							}

							menuBreaks++;

							nextYPosition = nextYPosition+hookheight+ySpacing;
						}
						else
						{
							if (adaptedmaxHeight < nextYPosition-ySpacing+bottomBorder)
								adaptedmaxHeight = nextYPosition-ySpacing+bottomBorder;
							
							nextYPosition = topBorder;

							easyHelper = str_third;
							if (easyHelper.Find("%%") != -1)
							{
								easyHelper = easyHelper.Right(easyHelper.GetLength()-easyHelper.Find("%%")-2);
								easyHelper = easyHelper.Left(easyHelper.Find("%%"));
								easyHelper += " ";
								easyHelper += IntToString(hookleft).c_str();
								easyHelper += " ";
								easyHelper += IntToString(nextXPosition).c_str();
								easyHelper += " ";
								easyHelper += IntToString(nextYPosition).c_str();
							
								hookcontentList.insert(hookcontentList.end(), (string)easyHelper);
							}

							hiddenPart++;
							
							nextYPosition = nextYPosition+hookheight+ySpacing;
						}
					}
					else
					{
						easyHelper = str_third;
						if (easyHelper.Find("%%") != -1)
						{
							easyHelper = easyHelper.Right(easyHelper.GetLength()-easyHelper.Find("%%")-2);
							easyHelper = easyHelper.Left(easyHelper.Find("%%"));
							easyHelper += " ";
							easyHelper += IntToString(hookleft).c_str();
							easyHelper += " ";
							easyHelper += IntToString(nextXPosition).c_str();
							easyHelper += " ";
							easyHelper += IntToString(nextYPosition).c_str();
							
							hookcontentList.insert(hookcontentList.end(), (string)easyHelper);
						}

						nextYPosition = nextYPosition+hookheight+ySpacing;
					}

					if (maxNeededWidth < hookleft+hookwidth+hookright)
						maxNeededWidth = hookleft+hookwidth+hookright;

					hasHookedWindows = true;
				}
			}
			//-----------------------------------------------------
			// INFO ENTRY (xLabel(Light)) REQUIRED
			//-----------------------------------------------------
			else if ( quicklaunchMode == 0 && stricmp(str_second.Left(5), "!info") == NULL )
			{
				if (input)
				{
					//Add to specific Popup Settings list
					sendToPopup.insert(sendToPopup.end(), *sit);	
				}
				else
				{
					ENTRY* newitem = new ENTRY;

					//Needs the original "szThird", cause "str_third" hasn't Quotations anymore
					newitem->info = szThird;

					newitem->entryType = 2;

					newitem->x = nextXPosition;
					newitem->y = nextYPosition;
					newitem->height = infoHeight;
					
					if (nextYPosition+newitem->height+bottomBorder > maxHeight)
					{
						if (autoMenuBreak)
						{
							if (adaptedmaxHeight < nextYPosition-ySpacing+bottomBorder)
								adaptedmaxHeight = nextYPosition-ySpacing+bottomBorder;

							nextXPosition = nextXPosition+maxNeededWidth+xSpacing;
							nextYPosition = topBorder;

							newitem->x = nextXPosition;
							newitem->y = nextYPosition;

							menuBreaks++;

							nextYPosition = nextYPosition+newitem->height+ySpacing;
						}
						else
						{
							if (adaptedmaxHeight < nextYPosition-ySpacing+bottomBorder)
								adaptedmaxHeight = nextYPosition-ySpacing+bottomBorder;
							
							nextYPosition = topBorder;

							newitem->y = nextYPosition;

							hiddenPart++;
							
							nextYPosition = nextYPosition+newitem->height+ySpacing;
						}
					}
					else
						nextYPosition = nextYPosition+newitem->height+ySpacing;
						
					newitem->hiddenPart = hiddenPart;
					newitem->multiPart = menuBreaks;

					newitem->caption = InfoUpdate( newitem->info );

					//Measure with Type 2 only check InfoFont Settings
					POINT tmp = buttonfont->measure(defaultDC, newitem->caption, 2);
					if (maxNeededWidth < tmp.x)
						maxNeededWidth = tmp.x;

					entrylist.insert(entrylist.end(), newitem);

					SetTimer(hWnd, TIMER_UPDATE, 1000, NULL);

					hasInfoEntry = true;
				}
			}
			//-----------------------------------------------------
			// INSERT POPUPFOLDERS
			//-----------------------------------------------------
			else if ( stricmp(str_second.Left(13), "!insertfolder") == NULL || stricmp(str_second.Left(14), "!xinsertfolder") == NULL )
			{
				if (input)
				{
					//Add to specific Popup Settings list
					sendToPopup.insert(sendToPopup.end(), *sit);	
				}
				else
				{
					easyHelper = str_second;
					
					// Get FolderPath
					easyHelper = easyHelper.Right(easyHelper.GetLength()-easyHelper.Find(":")-1);
					easyHelper = TrimAndStripQuote( (string)easyHelper ).c_str();
					//---------------------------------------------------------------------------------------
					// START Scanning
					//---------------------------------------------------------------------------------------
					string searchPath = (string)easyHelper;
					string wildcardPath = "";
					string itemName = "";
					string tmpHelper = "";

					HANDLE hFindHandle;
					WIN32_FIND_DATA FindData;

					vector<string> folders;
					vector<string> files;

					bool newIteration = true;
					CString worker = searchPath.c_str();
					while (newIteration == true)
					{
						if (worker.Find("|") != -1)
						{
							searchPath = worker.Left(worker.Find("|"));
							worker = worker.Right(worker.GetLength()-worker.Find("|")-1);
						}
						else
						{
							searchPath = worker;
							newIteration = false;
						}
						
						//Check for ending BackSlash
						tmpHelper = searchPath.substr(searchPath.length()-1, searchPath.length());
						bool addBackslash = false;
						if (tmpHelper != "\\" && strstr(searchPath.c_str(), "*.") == NULL)
						{
							wildcardPath = (searchPath + "\\*.*");
							addBackslash = true;
						}
						else if (strstr(searchPath.c_str(), "*.") == NULL)
						{
							wildcardPath = (searchPath + "*.*");
						}
						else
						{
							wildcardPath = searchPath;
							searchPath = searchPath.substr(0, searchPath.find('*'));
						}

						hFindHandle = FindFirstFile(wildcardPath.c_str(), &FindData);
						
						for(;;)
						{
							if(INVALID_HANDLE_VALUE == hFindHandle)
								break;

							itemName = FindData.cFileName;

							//It is a folder
							if (stricmp(itemName.c_str(), ".") != NULL && stricmp(itemName.c_str(), "..") != NULL && !(FindData.dwFileAttributes & FILE_ATTRIBUTE_HIDDEN))
							{
								if (addBackslash)
									tmpHelper = (searchPath + "\\" + itemName);
								else
									tmpHelper = (searchPath + itemName);
								
								if (FindData.dwFileAttributes &  FILE_ATTRIBUTE_DIRECTORY)
								{
									//Check for double folders and unite
									CString helper = "";
									bool found = false;
									for (StringListIterator sit = folders.begin(); sit != folders.end(); ++sit)
									{
										easyHelper = (*sit).c_str();
										
										if (stricmp( itemName.c_str(), easyHelper.Right( itemName.length() ) ) == NULL)
										{
											easyHelper += "|";
											easyHelper += tmpHelper.c_str();
											(*sit) = (string)easyHelper;

											found = true;
											break;
										}
									}
									
									if ( !found )
										folders.insert(folders.end(), tmpHelper);
								}
								else
									files.insert(files.end(), tmpHelper);
							}

							if(!FindNextFile(hFindHandle, &FindData))
								break;
						}
					}

					FindClose(hFindHandle);
					//---------------------------------------------------------------------------------------
					// END Scanning
					//---------------------------------------------------------------------------------------

					//Sorting
					Sort(folders);
					Sort(files);

					StringListIterator sit;

					for (sit = folders.begin(); sit != folders.end(); ++sit)
					{
						ENTRY* newitem = new ENTRY;
						
						easyHelper = (*sit).c_str();
						easyHelper = easyHelper.Right(easyHelper.GetLength()-easyHelper.ReverseFind('\\')-1);

						//Remove Custom Sort HelperStrings
						if ( easyHelper.Find("[") == 0 && easyHelper.Find("]") == 3 )
							easyHelper = easyHelper.Right(easyHelper.GetLength()-4);
						//--------------------------------
						newitem->caption = easyHelper;

						ScanKeyCodes(newitem, (string) easyHelper);
									
						newitem->entryType = -2;

						newitem->x = nextXPosition;
						newitem->y = nextYPosition;
						newitem->height = folderHeight;
									
						if (nextYPosition+newitem->height+bottomBorder > maxHeight)
						{
							if (autoMenuBreak)
							{
								if (adaptedmaxHeight < nextYPosition-ySpacing+bottomBorder)
									adaptedmaxHeight = nextYPosition-ySpacing+bottomBorder;

								nextXPosition = nextXPosition+maxNeededWidth+xSpacing;
								nextYPosition = topBorder;

								newitem->x = nextXPosition;
								newitem->y = nextYPosition;

								menuBreaks++;

								nextYPosition = nextYPosition+newitem->height+ySpacing;
							}
							else
							{
								if (adaptedmaxHeight < nextYPosition-ySpacing+bottomBorder)
									adaptedmaxHeight = nextYPosition-ySpacing+bottomBorder;
									
								nextYPosition = topBorder;

								newitem->y = nextYPosition;

								hiddenPart++;
									
								nextYPosition = nextYPosition+newitem->height+ySpacing;
							}
						}
						else
							nextYPosition = nextYPosition+newitem->height+ySpacing;
								
						newitem->hiddenPart = hiddenPart;
						newitem->multiPart = menuBreaks;

						//Folders MUSTN'T have Quotes
						//Scan and process Piped Folders
						bool newIteration = true;
						CString worker = (*sit).c_str();
						while (newIteration == true)
						{
							if (worker.Find("|") != -1)
							{
								newitem->pipedList.insert(newitem->pipedList.end(), (string)worker.Left(worker.Find("|")));
								worker = worker.Right(worker.GetLength()-worker.Find("|")-1);
							}
							else
							{
								newitem->pipedList.insert(newitem->pipedList.end(), (string)worker);
								newIteration = false;
							}
						}

						newitem->action = (*sit);
					
						if ( DefaultFolderIcon.empty() )
							newitem->iconPath = "*";
						else
							newitem->iconPath = DefaultFolderIcon;

						POINT tmp = buttonfont->measure(defaultDC, newitem->caption);
						if (maxNeededWidth < tmp.x)
							maxNeededWidth = tmp.x;

						entrylist.insert(entrylist.end(), newitem);
					}

					for (sit = files.begin(); sit != files.end(); ++sit)
					{
						ENTRY* newitem = new ENTRY;
						
						easyHelper = (*sit).c_str();
						easyHelper = easyHelper.Right(easyHelper.GetLength()-easyHelper.ReverseFind('\\')-1);
						if (easyHelper.ReverseFind('.') != -1 && !showExtensions)
						{
							easyHelper = easyHelper.Left(easyHelper.ReverseFind('.'));
						}
						else if (easyHelper.ReverseFind('.') != -1 && noshowExtensions.size() != 0)
						{
							CString help1;
							CString help2;

							for (StringListIterator isit = noshowExtensions.begin(); isit != noshowExtensions.end(); ++isit)
							{
								help1 = easyHelper;
								help2 = (*isit).c_str();
								help1.MakeLower();
								help2.MakeLower();

								if ( help1.Find(help2) >= help1.GetLength()-5 )
								{
									easyHelper = easyHelper.Left(easyHelper.ReverseFind('.'));
									break;
								}
							}
						}

						//Remove Custom Sort HelperStrings
						if ( easyHelper.Find("[") == 0 && easyHelper.Find("]") == 3 )
							easyHelper = easyHelper.Right(easyHelper.GetLength()-4);
						//--------------------------------
						newitem->caption = easyHelper;

						ScanKeyCodes(newitem, (string) easyHelper);
					
						newitem->x = nextXPosition;
						newitem->y = nextYPosition;
						newitem->height = entryHeight;
									
						if (nextYPosition+newitem->height+bottomBorder > maxHeight)
						{
							if (autoMenuBreak)
							{
								if (adaptedmaxHeight < nextYPosition-ySpacing+bottomBorder)
									adaptedmaxHeight = nextYPosition-ySpacing+bottomBorder;

								nextXPosition = nextXPosition+maxNeededWidth+xSpacing;
								nextYPosition = topBorder;

								newitem->x = nextXPosition;
								newitem->y = nextYPosition;

								menuBreaks++;

								nextYPosition = nextYPosition+newitem->height+ySpacing;
							}
							else
							{
								if (adaptedmaxHeight < nextYPosition-ySpacing+bottomBorder)
									adaptedmaxHeight = nextYPosition-ySpacing+bottomBorder;
										
								nextYPosition = topBorder;

								newitem->y = nextYPosition;

								hiddenPart++;
									
								nextYPosition = nextYPosition+newitem->height+ySpacing;
							}
						}
						else
							nextYPosition = nextYPosition+newitem->height+ySpacing;
										
						newitem->hiddenPart = hiddenPart;
						newitem->multiPart = menuBreaks;

						//Files MUST have Quotes
						newitem->action = ("\"" + (*sit) + "\"");

						newitem->iconPath = (*sit);

						POINT tmp = buttonfont->measure(defaultDC, newitem->caption);
						if (maxNeededWidth < tmp.x)
							maxNeededWidth = tmp.x;

						entrylist.insert(entrylist.end(), newitem);
					}
				}
			}
			//-----------------------------------------------------
			// DYNAMIC POPUPFOLDERS
			//-----------------------------------------------------
			else if ( stricmp(str_third.Left(19), "!popupdynamicfolder") == NULL || stricmp(str_third.Left(14), "!dynamicfolder") == NULL || stricmp(str_third.Left(12), "!popupfolder") == NULL 
						|| stricmp(str_third.Left(20), "!xpopupdynamicfolder") == NULL || stricmp(str_third.Left(13), "!xpopupfolder") == NULL )
			{
				if (input)
				{
					//Add to specific Popup Settings list
					sendToPopup.insert(sendToPopup.end(), *sit);	
				}
				else
				{
					ENTRY* newitem = new ENTRY;

					newitem->entryType = -1;
					if ( stricmp(str_third.Left(19), "!popupdynamicfolder") == NULL || stricmp(str_third.Left(14), "!dynamicfolder") == NULL || stricmp(str_third.Left(20), "!xpopupdynamicfolder") == NULL)
						newitem->entryType = -2;

					newitem->x = nextXPosition;
					newitem->y = nextYPosition;
					newitem->height = folderHeight;
				
					if (nextYPosition+newitem->height+bottomBorder > maxHeight)
					{
						if (autoMenuBreak)
						{
							if (adaptedmaxHeight < nextYPosition-ySpacing+bottomBorder)
								adaptedmaxHeight = nextYPosition-ySpacing+bottomBorder;

							nextXPosition = nextXPosition+maxNeededWidth+xSpacing;
							nextYPosition = topBorder;

							newitem->x = nextXPosition;
							newitem->y = nextYPosition;

							menuBreaks++;

							nextYPosition = nextYPosition+newitem->height+ySpacing;
						}
						else
						{
							if (adaptedmaxHeight < nextYPosition-ySpacing+bottomBorder)
								adaptedmaxHeight = nextYPosition-ySpacing+bottomBorder;
							
							nextYPosition = topBorder;

							newitem->y = nextYPosition;

							hiddenPart++;
							
							nextYPosition = nextYPosition+newitem->height+ySpacing;
						}
					}
					else
						nextYPosition = nextYPosition+newitem->height+ySpacing;
						
					newitem->hiddenPart = hiddenPart;
					newitem->multiPart = menuBreaks;

					newitem->caption = str_second;

					ScanKeyCodes(newitem, (string) str_second);
									
					easyHelper = str_third;
					
					// Get FolderPath
					easyHelper = easyHelper.Right(easyHelper.GetLength()-easyHelper.Find(":")-1);
					easyHelper = TrimAndStripQuote( (string)easyHelper ).c_str();

					//Scan and process Piped Folders
					bool newIteration = true;
					CString worker = easyHelper;
					while (newIteration == true)
					{
						if (worker.Find("|") != -1)
						{
							newitem->pipedList.insert(newitem->pipedList.end(), (string)worker.Left(worker.Find("|")));
							worker = worker.Right(worker.GetLength()-worker.Find("|")-1);
						}
						else
						{
							newitem->pipedList.insert(newitem->pipedList.end(), (string)worker);
							newIteration = false;
						}
					}

					newitem->action = easyHelper;

					if ( DefaultFolderIcon.empty() )
					{
						if (customIcon)
							newitem->iconPath = str_first;
						else
							newitem->iconPath = "*";
					}
					else
						newitem->iconPath = DefaultFolderIcon;


					POINT tmp = buttonfont->measure(defaultDC, newitem->caption);
					if (maxNeededWidth < tmp.x)
						maxNeededWidth = tmp.x;

					entrylist.insert(entrylist.end(), newitem);
				}
			}
			//-----------------------------------------------------
			// POPUPTASKS
			//-----------------------------------------------------
			else if ( quicklaunchMode == 0 && (stricmp(str_third, "!popuptasks") == NULL || stricmp(str_third, "!xpopuptasks") == NULL) )
			{
				if (input)
				{
					//Add to specific Popup Settings list
					sendToPopup.insert(sendToPopup.end(), *sit);	
				}
				else
				{
					ENTRY* newitem = new ENTRY;

					newitem->hWnd = hWnd;

					newitem->entryType = -1;

					newitem->x = nextXPosition;
					newitem->y = nextYPosition;
					newitem->height = folderHeight;
					
					if (nextYPosition+newitem->height+bottomBorder > maxHeight)
					{
						if (autoMenuBreak)
						{
							if (adaptedmaxHeight < nextYPosition-ySpacing+bottomBorder)
								adaptedmaxHeight = nextYPosition-ySpacing+bottomBorder;

							nextXPosition = nextXPosition+maxNeededWidth+xSpacing;
							nextYPosition = topBorder;

							newitem->x = nextXPosition;
							newitem->y = nextYPosition;

							menuBreaks++;

							nextYPosition = nextYPosition+newitem->height+ySpacing;
						}
						else
						{
							if (adaptedmaxHeight < nextYPosition-ySpacing+bottomBorder)
								adaptedmaxHeight = nextYPosition-ySpacing+bottomBorder;
							
							nextYPosition = topBorder;

							newitem->y = nextYPosition;

							hiddenPart++;
							
							nextYPosition = nextYPosition+newitem->height+ySpacing;
						}
					}
					else
						nextYPosition = nextYPosition+newitem->height+ySpacing;
						
					newitem->hiddenPart = hiddenPart;
					newitem->multiPart = menuBreaks;

					newitem->caption = str_second;

					ScanKeyCodes(newitem, (string) str_second);
			
					if (customIcon)
						easyHelper = str_first;
					else
						easyHelper = DefaultFolderIcon.c_str();

					newitem->iconPath = easyHelper;

					POINT tmp = buttonfont->measure(defaultDC, newitem->caption);
					if (maxNeededWidth < tmp.x)
						maxNeededWidth = tmp.x;

					entrylist.insert(entrylist.end(), newitem);
				}
			}
			//-----------------------------------------------------
			// POPUP SHELLFOLDERS
			//-----------------------------------------------------
			else if ( quicklaunchMode == 0 && (stricmp(str_third, "!popupcontrolpanel") == NULL || stricmp(str_third, "!popupprinters") == NULL 
						|| stricmp(str_third, "!popuprecyclebin") == NULL || stricmp(str_third, "!popuprecentdocuments") == NULL
						|| stricmp(str_third, "!popupnetwork") == NULL || stricmp(str_third, "!popupmycomputer") == NULL 
						|| stricmp(str_third, "!xpopupcontrolpanel") == NULL || stricmp(str_third, "!xpopupprinters") == NULL 
						|| stricmp(str_third, "!xpopuprecyclebin") == NULL || stricmp(str_third, "!xpopuprecentdocuments") == NULL
						|| stricmp(str_third, "!xpopupnetwork") == NULL || stricmp(str_third, "!xpopupmycomputer") == NULL) )
			{
				if (input)
				{
					//Add to specific Popup Settings list
					sendToPopup.insert(sendToPopup.end(), *sit);	
				}
				else
				{	
					ENTRY* newitem = new ENTRY;

					if ( stricmp(str_third, "!popupcontrolpanel") == NULL || stricmp(str_third, "!xpopupcontrolpanel") == NULL )
						newitem->clsid = CSIDL_CONTROLS;
					else if (stricmp(str_third, "!popupmycomputer") == NULL || stricmp(str_third, "!xpopupmycomputer") == NULL )
						newitem->clsid = CSIDL_DRIVES;
					else if (stricmp(str_third, "!popupnetwork") == NULL || stricmp(str_third, "!xpopupnetwork") == NULL )
						newitem->clsid = CSIDL_NETWORK;
					else if (stricmp(str_third, "!popupprinters") == NULL || stricmp(str_third, "!xpopupprinters") == NULL )
						newitem->clsid = CSIDL_PRINTERS;
					else if (stricmp(str_third, "!popuprecentdocuments") == NULL || stricmp(str_third, "!xpopuprecentdocuments") == NULL )
						newitem->clsid = CSIDL_RECENT;
					else if (stricmp(str_third, "!popuprecyclebin") == NULL || stricmp(str_third, "!xpopuprecyclebin") == NULL )
						newitem->clsid = CSIDL_BITBUCKET;
					else
						newitem->clsid = 0;

					newitem->entryType = -1;

					newitem->x = nextXPosition;
					newitem->y = nextYPosition;
					newitem->height = entryHeight;

					LPITEMIDLIST m_pidlFull = NULL;
					HRESULT hres = SHGetSpecialFolderLocation(NULL, newitem->clsid, &m_pidlFull);

					if (m_pidlFull)
						newitem->pidl = m_pidlFull;
					
					if (nextYPosition+newitem->height+bottomBorder > maxHeight)
					{
						if (autoMenuBreak)
						{
							if (adaptedmaxHeight < nextYPosition-ySpacing+bottomBorder)
								adaptedmaxHeight = nextYPosition-ySpacing+bottomBorder;

							nextXPosition = nextXPosition+maxNeededWidth+xSpacing;
							nextYPosition = topBorder;

							newitem->x = nextXPosition;
							newitem->y = nextYPosition;

							menuBreaks++;

							nextYPosition = nextYPosition+newitem->height+ySpacing;
						}
						else
						{
							if (adaptedmaxHeight < nextYPosition-ySpacing+bottomBorder)
								adaptedmaxHeight = nextYPosition-ySpacing+bottomBorder;
							
							nextYPosition = topBorder;

							newitem->y = nextYPosition;

							hiddenPart++;
							
							nextYPosition = nextYPosition+newitem->height+ySpacing;
						}
					}
					else
						nextYPosition = nextYPosition+newitem->height+ySpacing;
						
					newitem->hiddenPart = hiddenPart;
					newitem->multiPart = menuBreaks;

					newitem->caption = str_second;

					ScanKeyCodes(newitem, (string) str_second);
			
					if (customIcon)
						newitem->iconPath = str_first;

					POINT tmp = buttonfont->measure(defaultDC, newitem->caption);
					if (maxNeededWidth < tmp.x)
						maxNeededWidth = tmp.x;

					entrylist.insert(entrylist.end(), newitem);
				}
			}
			//-----------------------------------------------------
			// NORMAL ENTRYS
			//-----------------------------------------------------
			else if ( stricmp(str_second, "!popupimage") != NULL && stricmp(str_third, "!popuprun") != NULL
					 && stricmp(str_third, "!datetime") != NULL && stricmp(str_second, "!remoteamp") != NULL 
					 && stricmp(str_third, "!popupdesktops") != NULL )
			{
				if (input)
				{
					//Add to specific Popup Settings list
					sendToPopup.insert(sendToPopup.end(), *sit);	
				}
				else
				{
					ENTRY* newitem = new ENTRY;

					newitem->x = nextXPosition;
					newitem->y = nextYPosition;
					newitem->height = entryHeight;
					
					if (nextYPosition+newitem->height+bottomBorder > maxHeight)
					{
						if (autoMenuBreak)
						{
							if (adaptedmaxHeight < nextYPosition-ySpacing+bottomBorder)
								adaptedmaxHeight = nextYPosition-ySpacing+bottomBorder;

							nextXPosition = nextXPosition+maxNeededWidth+xSpacing;
							nextYPosition = topBorder;

							newitem->x = nextXPosition;
							newitem->y = nextYPosition;

							menuBreaks++;

							nextYPosition = nextYPosition+newitem->height+ySpacing;
						}
						else
						{
							if (adaptedmaxHeight < nextYPosition-ySpacing+bottomBorder)
								adaptedmaxHeight = nextYPosition-ySpacing+bottomBorder;
							
							nextYPosition = topBorder;

							newitem->y = nextYPosition;

							hiddenPart++;
							
							nextYPosition = nextYPosition+newitem->height+ySpacing;
						}
					}
					else
						nextYPosition = nextYPosition+newitem->height+ySpacing;
						
					newitem->hiddenPart = hiddenPart;
					newitem->multiPart = menuBreaks;

					newitem->caption = str_second;

					ScanKeyCodes(newitem, (string) str_second);
		
					if ( PathFileExists(str_third) )
						newitem->action = ("\"" + str_third + "\"");
					else
						newitem->action = str_third;

					if (customIcon)
						easyHelper = str_first;
					else
						easyHelper = TrimAndStripQuote( newitem->action ).c_str();

					if ( customIcon )
						newitem->iconPath = easyHelper;
					else if ( easyHelper.Left(1) != "!" )
						newitem->iconPath = easyHelper;
					else 
						newitem->iconPath = DefaultIcon;

					POINT tmp = buttonfont->measure(defaultDC, newitem->caption);
					if (maxNeededWidth < tmp.x)
						maxNeededWidth = tmp.x;

					entrylist.insert(entrylist.end(), newitem);
				}
			}
		}

		//Check Max/Min EntryWidth
		maxNeededWidth = max(min(maxNeededWidth, maxEntryWidth), minEntryWidth);

		// Check Last MenuColumn for Height
		if (adaptedmaxHeight != 0 && adaptedmaxHeight < nextYPosition-ySpacing+bottomBorder)
			adaptedmaxHeight = nextYPosition-ySpacing+bottomBorder;

		//Measure with Type 1 only check TitleFont Settings for maxNeededWidth
		POINT tmp = buttonfont->measure(defaultDC, captionname, 1);
		int neededTitleWidth = leftBorder+tmp.x+rightBorder;

		//Adapt EntryWidth based on Max/Min Width
		if ( leftBorder+(menuBreaks+1)*maxNeededWidth+menuBreaks*xSpacing+rightBorder >= maxWidth )
		{
			actualwidth = maxWidth;
			maxNeededWidth = (actualwidth-leftBorder-rightBorder-menuBreaks*xSpacing)/(menuBreaks+1);
		}
		else if (leftBorder+(menuBreaks+1)*maxNeededWidth+menuBreaks*xSpacing+rightBorder <= minWidth )
		{
			actualwidth = minWidth;
			maxNeededWidth = (actualwidth-leftBorder-rightBorder-menuBreaks*xSpacing)/(menuBreaks+1);
		}
		else
		{
			actualwidth = leftBorder+(menuBreaks+1)*maxNeededWidth+menuBreaks*xSpacing+rightBorder;
			actualwidth = min(max(neededTitleWidth, actualwidth), maxWidth);
		}

		if (hiddenPart != 0 || menuBreaks != 0)
			actualheight = adaptedmaxHeight;
		else
			actualheight = min(nextYPosition-ySpacing+bottomBorder, maxHeight);
		actualheight = max(actualheight, minHeight);

		//Hook Positioning
		//----------------------------------------------
		string x = "";
		string y = "";
		string space = "";
	
		for (sit = hookcontentList.begin(); sit != hookcontentList.end(); ++sit)
		{
			x = "0";
			y = "0";
			space = "0";
			easyHelper = (*sit).c_str();
			y = easyHelper.Right(easyHelper.GetLength()-easyHelper.ReverseFind(' '));
			easyHelper = easyHelper.Left(easyHelper.ReverseFind(' '));
			x = easyHelper.Right(easyHelper.GetLength()-easyHelper.ReverseFind(' '));
			easyHelper = easyHelper.Left(easyHelper.ReverseFind(' '));
			space = easyHelper.Right(easyHelper.GetLength()-easyHelper.ReverseFind(' '));
			easyHelper = easyHelper.Left(easyHelper.ReverseFind(' '));
			
			int iterate = leftBorder;
			while ( atoi(x.c_str()) > iterate )
				iterate = iterate+maxNeededWidth+xSpacing;

			x = IntToString(iterate+atoi(space.c_str()));

			easyHelper += " ";
			easyHelper += x.c_str();
			easyHelper += " ";
			easyHelper += y.c_str();

			LSExecute(NULL, easyHelper, SW_SHOWNORMAL);
		}

		hookcontentList.clear();
		//----------------------------------------------

		for (EntryListIterator eit = entrylist.begin(); eit != entrylist.end(); ++eit)
		{
			if (startentry == NULL && (*eit)->entryType <= 0 )
				startentry = *eit;

			if (menuBreaks == 0)
				(*eit)->width = max(maxNeededWidth, actualwidth-leftBorder-rightBorder);
			else
			{
				if (minWidth > leftBorder+maxNeededWidth+rightBorder)
					maxNeededWidth = (minWidth-(menuBreaks*xSpacing)-leftBorder-rightBorder)/(menuBreaks+1);

				(*eit)->width = maxNeededWidth;
				
				(*eit)->x = leftBorder + (*eit)->multiPart*(maxNeededWidth + xSpacing);
			}
			
			if (cacheAllPopupsAtStartup)
			{
				if ( (*eit)->entryType < 0 && (*eit)->clsid == 0 && (*eit)->hWnd == NULL )
					CreateSubMenu( ("sub_"+(*eit)->caption), (*eit)->action, (*eit));
			}
		}

		sendToPopup.clear();

		maxPart = hiddenPart;
		maxMenuBreak = menuBreaks;
	}

	reposition(2, 0, 0, actualwidth, actualheight, (externCall ? 2 : 0));

	settingsList.clear();
}

void Popup::ScanSubMenu(string scanpath, int dynamic)
{
	CString easyHelper = "";

	//---------------------------------------------------------------------------------------
	// START Scanning
	//---------------------------------------------------------------------------------------
	string searchPath = "";
	string wildcardPath = "";
	string itemName = "";
	string tmpHelper = "";

	HANDLE hFindHandle;
	WIN32_FIND_DATA FindData;

	vector<string> folders;
	vector<string> files;

	bool newIteration = true;
	CString worker = scanpath.c_str();
	while (newIteration == true)
	{
		if (worker.Find("|") != -1)
		{
			searchPath = worker.Left(worker.Find("|"));
			worker = worker.Right(worker.GetLength()-worker.Find("|")-1);
		}
		else
		{
			searchPath = worker;
			newIteration = false;
		}
		
		//Check for ending BackSlash
		tmpHelper = searchPath.substr(searchPath.length()-1, searchPath.length());
		bool addBackslash = false;
		if (tmpHelper != "\\" && strstr(searchPath.c_str(), "*.") == NULL)
		{
			wildcardPath = (searchPath + "\\*.*");
			addBackslash = true;
		}
		else if (strstr(searchPath.c_str(), "*.") == NULL)
		{
			wildcardPath = (searchPath + "*.*");
		}
		else
		{
			wildcardPath = searchPath;
			searchPath = searchPath.substr(0, searchPath.find('*'));
		}

		hFindHandle = FindFirstFile(wildcardPath.c_str(), &FindData);
		
		for(;;)
		{
			if(INVALID_HANDLE_VALUE == hFindHandle)
				break;

			itemName = FindData.cFileName;

			//It is a folder
			if (stricmp(itemName.c_str(), ".") != NULL && stricmp(itemName.c_str(), "..") != NULL && !(FindData.dwFileAttributes & FILE_ATTRIBUTE_HIDDEN))
			{
				if (addBackslash)
					tmpHelper = (searchPath + "\\" + itemName);
				else
					tmpHelper = (searchPath + itemName);
				
				if (FindData.dwFileAttributes &  FILE_ATTRIBUTE_DIRECTORY)
				{
					//Check for double folders and unite
					CString helper = "";
					bool found = false;
					for (StringListIterator sit = folders.begin(); sit != folders.end(); ++sit)
					{
						easyHelper = (*sit).c_str();
						
						if (stricmp( itemName.c_str(), easyHelper.Right( itemName.length() ) ) == NULL)
						{
							easyHelper += "|";
							easyHelper += tmpHelper.c_str();
							(*sit) = (string)easyHelper;

							found = true;
							break;
						}
					}
					
					if ( !found )
						folders.insert(folders.end(), tmpHelper);
				}
				else
					files.insert(files.end(), tmpHelper);
			}

			if(!FindNextFile(hFindHandle, &FindData))
				break;
		}
	}

	FindClose(hFindHandle);
	//---------------------------------------------------------------------------------------
	// END Scanning
	//---------------------------------------------------------------------------------------

	//Check if the entryCount has changed (only reason for real updateing)
	if ( entryCount != folders.size()+files.size() )
	{
		for (EntryListIterator eit = entrylist.begin(); eit != entrylist.end(); ++eit)
			delete (*eit);

		entrylist.clear();

		for(PopupListIterator plit = subpopupList.begin(); plit != subpopupList.end(); plit++)
		{
			(*plit)->hide();
			delete *plit;
		}

		subpopupList.clear();

		startentry = NULL;

		entryCount = folders.size()+files.size();
	}
	else
	{
		return;
	}

	//Sorting
	Sort(folders);
	Sort(files);

	//Initializing of Values
	int itemCounter = 0;
	int nextXPosition = leftBorder;
	int nextYPosition = topBorder;
	int hiddenPart = 0;
	int menuBreaks = 0;
	int maxNeededWidth = 0;
	int adaptedmaxHeight = 0;

	StringListIterator sit;

	for (sit = folders.begin(); sit != folders.end(); ++sit)
	{
		itemCounter++;

		ENTRY* newitem = new ENTRY;
		
		easyHelper = (*sit).c_str();
		easyHelper = easyHelper.Right(easyHelper.GetLength()-easyHelper.ReverseFind('\\')-1);

		//Remove Custom Sort HelperStrings
		if ( easyHelper.Find("[") == 0 && easyHelper.Find("]") == 3 )
			easyHelper = easyHelper.Right(easyHelper.GetLength()-4);
		//--------------------------------
		newitem->caption = easyHelper;

		ScanKeyCodes(newitem, (string) easyHelper);
			
		newitem->entryType = -1;
		if (dynamic == -2)
			newitem->entryType = -2;

		if (quicklaunchMode != 0)
		{
			newitem->width = quicklaunchItemSize;
			newitem->height = quicklaunchItemSize;
			newitem->x = nextXPosition;
			newitem->y = nextYPosition;

			if (quicklaunchMode == 1)
			{
				if (itemCounter%quicklaunchWrapCount != 0)
				{
					nextXPosition = nextXPosition+newitem->width+xSpacing;
					nextYPosition = nextYPosition;
				}
				else
				{
					nextXPosition = leftBorder;
					nextYPosition = nextYPosition+newitem->height+ySpacing;
				}
			}
			else if (quicklaunchMode == 2)
			{
				if (itemCounter%quicklaunchWrapCount != 0)
				{
					nextXPosition = nextXPosition;
					nextYPosition = nextYPosition+newitem->height+ySpacing;
				}
				else
				{
					nextXPosition = nextXPosition+newitem->width+xSpacing;
					nextYPosition = topBorder;
				}
			}
		}
		else
		{
			newitem->x = nextXPosition;
			newitem->y = nextYPosition;
			newitem->height = folderHeight;
					
			if (nextYPosition+newitem->height+bottomBorder > maxHeight)
			{
				if (autoMenuBreak)
				{
					if (adaptedmaxHeight < nextYPosition-ySpacing+bottomBorder)
						adaptedmaxHeight = nextYPosition-ySpacing+bottomBorder;

					nextXPosition = nextXPosition+maxNeededWidth+xSpacing;
					nextYPosition = topBorder;

					newitem->x = nextXPosition;
					newitem->y = nextYPosition;

					menuBreaks++;

					nextYPosition = nextYPosition+newitem->height+ySpacing;
				}
				else
				{
					if (adaptedmaxHeight < nextYPosition-ySpacing+bottomBorder)
						adaptedmaxHeight = nextYPosition-ySpacing+bottomBorder;
					
					nextYPosition = topBorder;

					newitem->y = nextYPosition;

					hiddenPart++;
					
					nextYPosition = nextYPosition+newitem->height+ySpacing;
				}
			}
			else
				nextYPosition = nextYPosition+newitem->height+ySpacing;
		}
				
		newitem->hiddenPart = hiddenPart;
		newitem->multiPart = menuBreaks;

		//Folders MUSTN'T have Quotes
		//Scan and process Piped Folders
		bool newIteration = true;
		CString worker = (*sit).c_str();
		while (newIteration == true)
		{
			if (worker.Find("|") != -1)
			{
				newitem->pipedList.insert(newitem->pipedList.end(), (string)worker.Left(worker.Find("|")));
				worker = worker.Right(worker.GetLength()-worker.Find("|")-1);
			}
			else
			{
				newitem->pipedList.insert(newitem->pipedList.end(), (string)worker);
				newIteration = false;
			}
		}

		newitem->action = (*sit);
	
		if ( DefaultFolderIcon.empty() )
			newitem->iconPath = "*";
		else
			newitem->iconPath = DefaultFolderIcon;


		POINT tmp = buttonfont->measure(defaultDC, newitem->caption);
		if (maxNeededWidth < tmp.x)
			maxNeededWidth = tmp.x;

		entrylist.insert(entrylist.end(), newitem);
	}

	//Insert AutoSeparator if wished between Folder and Files
	if (autoSeparator && folders.size() >= 1 && files.size() >= 1 && quicklaunchMode == 0)
	{
		ENTRY* newitem = new ENTRY;	
					
		newitem->entryType = 1;

		newitem->x = nextXPosition;
		newitem->y = nextYPosition;
		newitem->height = separatorHeight;
					
		if (nextYPosition+newitem->height+bottomBorder > maxHeight)
		{
			if (autoMenuBreak)
			{
				if (adaptedmaxHeight < nextYPosition-ySpacing+bottomBorder)
					adaptedmaxHeight = nextYPosition-ySpacing+bottomBorder;

				nextXPosition = nextXPosition+maxNeededWidth+xSpacing;
				nextYPosition = topBorder;

				newitem->x = nextXPosition;
				newitem->y = nextYPosition;

				menuBreaks++;

				nextYPosition = nextYPosition+newitem->height+ySpacing;
			}
			else
			{
				if (adaptedmaxHeight < nextYPosition-ySpacing+bottomBorder)
					adaptedmaxHeight = nextYPosition-ySpacing+bottomBorder;
					
				nextYPosition = topBorder;

				newitem->y = nextYPosition;

				hiddenPart++;
				
				nextYPosition = nextYPosition+newitem->height+ySpacing;
			}
		}
		else
			nextYPosition = nextYPosition+newitem->height+ySpacing;
						
		newitem->hiddenPart = hiddenPart;
		newitem->multiPart = menuBreaks;

		entrylist.insert(entrylist.end(), newitem);
	}

	for (sit = files.begin(); sit != files.end(); ++sit)
	{
		itemCounter++;

		ENTRY* newitem = new ENTRY;
		
		easyHelper = (*sit).c_str();
		easyHelper = easyHelper.Right(easyHelper.GetLength()-easyHelper.ReverseFind('\\')-1);
		if (easyHelper.ReverseFind('.') != -1 && !showExtensions)
		{
			easyHelper = easyHelper.Left(easyHelper.ReverseFind('.'));
		}
		else if (easyHelper.ReverseFind('.') != -1 && noshowExtensions.size() != 0)
		{
			CString help1;
			CString help2;

			for (StringListIterator isit = noshowExtensions.begin(); isit != noshowExtensions.end(); ++isit)
			{
				help1 = easyHelper;
				help2 = (*isit).c_str();
				help1.MakeLower();
				help2.MakeLower();

				if ( help1.Find(help2) >= help1.GetLength()-5 )
				{
					easyHelper = easyHelper.Left(easyHelper.ReverseFind('.'));
					break;
				}
			}
		}

		//Remove Custom Sort HelperStrings
		if ( easyHelper.Find("[") == 0 && easyHelper.Find("]") == 3 )
			easyHelper = easyHelper.Right(easyHelper.GetLength()-4);
		//--------------------------------
		newitem->caption = easyHelper;

		ScanKeyCodes(newitem, (string) easyHelper);

		if (quicklaunchMode != 0)
		{
			newitem->width = quicklaunchItemSize;
			newitem->height = quicklaunchItemSize;
			newitem->x = nextXPosition;
			newitem->y = nextYPosition;

			if (quicklaunchMode == 1)
			{
				if (itemCounter%quicklaunchWrapCount != 0)
				{
					nextXPosition = nextXPosition+newitem->width+xSpacing;
					nextYPosition = nextYPosition;
				}
				else
				{
					nextXPosition = leftBorder;
					nextYPosition = nextYPosition+newitem->height+ySpacing;
				}
			}
			else if (quicklaunchMode == 2)
			{
				if (itemCounter%quicklaunchWrapCount != 0)
				{
					nextXPosition = nextXPosition;
					nextYPosition = nextYPosition+newitem->height+ySpacing;
				}
				else
				{
					nextXPosition = nextXPosition+newitem->width+xSpacing;
					nextYPosition = topBorder;
				}
			}
		}
		else
		{
			newitem->x = nextXPosition;
			newitem->y = nextYPosition;
			newitem->height = entryHeight;
					
			if (nextYPosition+newitem->height+bottomBorder > maxHeight)
			{
				if (autoMenuBreak)
				{
					if (adaptedmaxHeight < nextYPosition-ySpacing+bottomBorder)
						adaptedmaxHeight = nextYPosition-ySpacing+bottomBorder;

					nextXPosition = nextXPosition+maxNeededWidth+xSpacing;
					nextYPosition = topBorder;

					newitem->x = nextXPosition;
					newitem->y = nextYPosition;

					menuBreaks++;

					nextYPosition = nextYPosition+newitem->height+ySpacing;
				}
				else
				{
					if (adaptedmaxHeight < nextYPosition-ySpacing+bottomBorder)
						adaptedmaxHeight = nextYPosition-ySpacing+bottomBorder;
						
					nextYPosition = topBorder;

					newitem->y = nextYPosition;

					hiddenPart++;
					
					nextYPosition = nextYPosition+newitem->height+ySpacing;
				}
			}
			else
				nextYPosition = nextYPosition+newitem->height+ySpacing;
		}
						
		newitem->hiddenPart = hiddenPart;
		newitem->multiPart = menuBreaks;

		//Files MUST have Quotes
		newitem->action = ("\"" + (*sit) + "\"");

		newitem->iconPath = (*sit);

		POINT tmp = buttonfont->measure(defaultDC, newitem->caption);
		if (maxNeededWidth < tmp.x)
			maxNeededWidth = tmp.x;

		entrylist.insert(entrylist.end(), newitem);
	}

	if (quicklaunchMode != 0)
	{
		int tmpCount;
		if (itemCounter < quicklaunchWrapCount)
			tmpCount = itemCounter;
		else
			tmpCount = quicklaunchWrapCount;

		if (quicklaunchMode == 1)
		{
			actualwidth = leftBorder+tmpCount*quicklaunchItemSize+(tmpCount-1)*xSpacing+rightBorder;
			actualheight = nextYPosition+((itemCounter%quicklaunchWrapCount != 0) ? quicklaunchItemSize : 0)+bottomBorder;
		}
		else if (quicklaunchMode == 2)
		{
			actualwidth = nextXPosition+((itemCounter%quicklaunchWrapCount != 0) ? quicklaunchItemSize : 0)+rightBorder;
			actualheight = topBorder+tmpCount*quicklaunchItemSize+(tmpCount-1)*ySpacing+bottomBorder;
		}
	}
	else
	{
		//Measure with Type 1 only check TitleFont Settings for maxNeededWidth
		POINT tmp = buttonfont->measure(defaultDC, captionname, 1);
		int neededTitleWidth = leftBorder+tmp.x+rightBorder;

		//Check Max/Min EntryWidth
		maxNeededWidth = max(min(maxNeededWidth, maxEntryWidth), minEntryWidth);

		//Adapt EntryWidth based on Max/Min Width
		if ( leftBorder+(menuBreaks+1)*maxNeededWidth+menuBreaks*xSpacing+rightBorder >= maxWidth )
		{
			actualwidth = maxWidth;
			maxNeededWidth = (actualwidth-leftBorder-rightBorder-menuBreaks*xSpacing)/(menuBreaks+1);
		}
		else if (leftBorder+(menuBreaks+1)*maxNeededWidth+menuBreaks*xSpacing+rightBorder <= minWidth )
		{
			actualwidth = minWidth;
			maxNeededWidth = (actualwidth-leftBorder-rightBorder-menuBreaks*xSpacing)/(menuBreaks+1);
		}
		else
		{
			actualwidth = leftBorder+(menuBreaks+1)*maxNeededWidth+menuBreaks*xSpacing+rightBorder;
			actualwidth = min(max(neededTitleWidth, actualwidth), maxWidth);
		}

		if (hiddenPart != 0 || menuBreaks != 0)
			actualheight = adaptedmaxHeight;
		else
			actualheight = min(nextYPosition-ySpacing+bottomBorder, maxHeight);
		actualheight = max(actualheight, minHeight);
	}

	for (EntryListIterator eit = entrylist.begin(); eit != entrylist.end(); ++eit)
	{
		if (startentry == NULL && (*eit)->entryType <= 0)
			startentry = *eit;

		if (quicklaunchMode == 0)
		{
			if (menuBreaks == 0)
				(*eit)->width = max(maxNeededWidth, actualwidth-leftBorder-rightBorder);
			else
			{
				if (minWidth > leftBorder+maxNeededWidth+rightBorder)
					maxNeededWidth = (minWidth-(menuBreaks*xSpacing)-leftBorder-rightBorder)/(menuBreaks+1);

				(*eit)->width = maxNeededWidth;
				
				(*eit)->x = leftBorder + (*eit)->multiPart*(maxNeededWidth + xSpacing);	
			}
		}

		if (cacheAllPopupsAtStartup)
		{
			if ( (*eit)->entryType < 0 && (*eit)->clsid == 0 && (*eit)->hWnd == NULL)
				CreateSubMenu( ("sub_"+(*eit)->caption), (*eit)->action, (*eit));
		}
	}

	maxPart = hiddenPart;
	maxMenuBreak = menuBreaks;

	reposition(3, 0, 0, actualwidth, actualheight);
}

void Popup::ScanShellFolder(int clsid)
{
	EntryListIterator eit;
	for (eit = entrylist.begin(); eit != entrylist.end(); ++eit)
		delete (*eit);

	entrylist.clear();

	//SHELL PART 1
	LPITEMIDLIST m_pidlFull = NULL;
	IShellFolder *m_pshf = NULL;
	SHGetSpecialFolderLocation(NULL, clsid, &m_pidlFull);

	if (m_pidlFull)
	{
		SHFILEINFO shfi;

		SHGetFileInfo((LPSTR) m_pidlFull, 0, &shfi, sizeof(SHFILEINFO), SHGFI_ATTRIBUTES | SHGFI_DISPLAYNAME |
		                                        SHGFI_PIDL | SHGFI_SYSICONINDEX | SHGFI_SMALLICON);

		if (shfi.dwAttributes & SFGAO_FOLDER && !(shfi.dwAttributes & SFGAO_REMOVABLE))
		{
			IShellFolder *pshfDesktop;
			SHGetDesktopFolder(&pshfDesktop);

			pshfDesktop->BindToObject(m_pidlFull, NULL, IID_IShellFolder,
			                          (void **) &m_pshf);

			pshfDesktop->Release();
		}
	}

	//---------------------------------------------------------------------------------------
	// START Scanning
	//---------------------------------------------------------------------------------------
	IEnumIDList *peidl;
	vector<LPITEMIDLIST> PIDLList;
	LPITEMIDLIST pidl;

	HRESULT hr = m_pshf->EnumObjects(NULL, SHCONTF_FOLDERS |
	                         SHCONTF_NONFOLDERS, &peidl);

	if ( SUCCEEDED(hr) )
	{
		while (peidl->Next(1, &pidl, NULL) == S_OK)
		{
			LPITEMIDLIST pidlFull = JoinIDLists(m_pidlFull, pidl);

			PIDLList.insert(PIDLList.end(), pidlFull);
		}

		peidl->Release();
	}
	//---------------------------------------------------------------------------------------
	// END Scanning
	//---------------------------------------------------------------------------------------

	//Sorting
	if (clsid != CSIDL_DRIVES && clsid != CSIDL_RECENT)
		SortShellFolder(PIDLList);

	//Initializing of Values
	int nextXPosition = leftBorder;
	int nextYPosition = topBorder;
	int hiddenPart = 0;
	int menuBreaks = 0;
	int maxNeededWidth = 0;
	int adaptedmaxHeight = 0;

	for (vector<LPITEMIDLIST>::iterator pit = PIDLList.begin(); pit != PIDLList.end(); ++pit)
	{
		SHFILEINFO shfi;

		SHGetFileInfo((LPCSTR) (*pit), 0, &shfi, sizeof(SHFILEINFO),
										  SHGFI_ATTRIBUTES | SHGFI_DISPLAYNAME | SHGFI_PIDL );

		if (shfi.dwAttributes & SFGAO_FOLDER)
		{
			//Entry
			ENTRY* newitem = new ENTRY;

			newitem->pidl = (*pit);

			newitem->x = nextXPosition;
			newitem->y = nextYPosition;

			newitem->caption = shfi.szDisplayName;

			newitem->height = entryHeight;
						
			if (nextYPosition+newitem->height+bottomBorder > maxHeight)
			{
				if (autoMenuBreak)
				{
					if (adaptedmaxHeight < nextYPosition-ySpacing+bottomBorder)
						adaptedmaxHeight = nextYPosition-ySpacing+bottomBorder;

					nextXPosition = nextXPosition+maxNeededWidth+xSpacing;
					nextYPosition = topBorder;

					newitem->x = nextXPosition;
					newitem->y = nextYPosition;

					menuBreaks++;

					nextYPosition = nextYPosition+newitem->height+ySpacing;
				}
				else
				{
					if (adaptedmaxHeight < nextYPosition-ySpacing+bottomBorder)
						adaptedmaxHeight = nextYPosition-ySpacing+bottomBorder;
							
					nextYPosition = topBorder;

					newitem->y = nextYPosition;

					hiddenPart++;
						
					nextYPosition = nextYPosition+newitem->height+ySpacing;
				}
			}
			else
				nextYPosition = nextYPosition+newitem->height+ySpacing;
								
			newitem->hiddenPart = hiddenPart;
			newitem->multiPart = menuBreaks;

			POINT tmp = buttonfont->measure(defaultDC, newitem->caption);
			if (maxNeededWidth < tmp.x)
				maxNeededWidth = tmp.x;

			entrylist.insert(entrylist.end(), newitem);
		}
		else
		{
			//Entry
			ENTRY* newitem = new ENTRY;

			newitem->pidl = (*pit);

			newitem->x = nextXPosition;
			newitem->y = nextYPosition;

			newitem->caption = shfi.szDisplayName;
			
			newitem->height = entryHeight;
						
			if (nextYPosition+newitem->height+bottomBorder > maxHeight)
			{
				if (autoMenuBreak)
				{
					if (adaptedmaxHeight < nextYPosition-ySpacing+bottomBorder)
						adaptedmaxHeight = nextYPosition-ySpacing+bottomBorder;

					nextXPosition = nextXPosition+maxNeededWidth+xSpacing;
					nextYPosition = topBorder;

					newitem->x = nextXPosition;
					newitem->y = nextYPosition;

					menuBreaks++;

					nextYPosition = nextYPosition+newitem->height+ySpacing;
				}
				else
				{
					if (adaptedmaxHeight < nextYPosition-ySpacing+bottomBorder)
						adaptedmaxHeight = nextYPosition-ySpacing+bottomBorder;
							
					nextYPosition = topBorder;

					newitem->y = nextYPosition;

					hiddenPart++;
						
					nextYPosition = nextYPosition+newitem->height+ySpacing;
				}
			}
			else
				nextYPosition = nextYPosition+newitem->height+ySpacing;
								
			newitem->hiddenPart = hiddenPart;
			newitem->multiPart = menuBreaks;

			POINT tmp = buttonfont->measure(defaultDC, newitem->caption);
			if (maxNeededWidth < tmp.x)
				maxNeededWidth = tmp.x;

			entrylist.insert(entrylist.end(), newitem);
		}
	}

	//Check Max/Min EntryWidth
	maxNeededWidth = max(min(maxNeededWidth, maxEntryWidth), minEntryWidth);
		
	//Measure with Type 1 only check TitleFont Settings for maxNeededWidth
	POINT tmp = buttonfont->measure(defaultDC, captionname, 1);
	int neededTitleWidth = leftBorder+tmp.x+rightBorder;

	//Adapt EntryWidth based on Max/Min Width
	if ( leftBorder+(menuBreaks+1)*maxNeededWidth+menuBreaks*xSpacing+rightBorder >= maxWidth )
	{
		actualwidth = maxWidth;
		maxNeededWidth = (actualwidth-leftBorder-rightBorder-menuBreaks*xSpacing)/(menuBreaks+1);
	}
	else if (leftBorder+(menuBreaks+1)*maxNeededWidth+menuBreaks*xSpacing+rightBorder <= minWidth )
	{
		actualwidth = minWidth;
		maxNeededWidth = (actualwidth-leftBorder-rightBorder-menuBreaks*xSpacing)/(menuBreaks+1);
	}
	else
	{
		actualwidth = leftBorder+(menuBreaks+1)*maxNeededWidth+menuBreaks*xSpacing+rightBorder;
		actualwidth = min(max(neededTitleWidth, actualwidth), maxWidth);
	}

	if (hiddenPart != 0 || menuBreaks != 0)
		actualheight = adaptedmaxHeight;
	else
		actualheight = min(nextYPosition-ySpacing+bottomBorder, maxHeight);
	actualheight = max(actualheight, minHeight);

	for (eit = entrylist.begin(); eit != entrylist.end(); ++eit)
	{
		if (startentry == NULL && (*eit)->entryType <= 0)
			startentry = *eit;

		if (menuBreaks == 0)
			(*eit)->width = max(maxNeededWidth, actualwidth-leftBorder-rightBorder);
		else
		{
			if (minWidth > leftBorder+maxNeededWidth+rightBorder)
				maxNeededWidth = (minWidth-(menuBreaks*xSpacing)-leftBorder-rightBorder)/(menuBreaks+1);

			(*eit)->width = maxNeededWidth;
				
			(*eit)->x = leftBorder + (*eit)->multiPart*(maxNeededWidth + xSpacing);
		}
	}

	maxPart = hiddenPart;
	maxMenuBreak = menuBreaks;

	reposition(2, 0, 0, actualwidth, actualheight);
}

void Popup::ScanTasks()
{
	SetTimer(hWnd, TIMER_UPDATE, tasksUpdate, NULL);

	isTaskPopup = true;

	EntryListIterator eit;
	for (eit = entrylist.begin(); eit != entrylist.end(); ++eit)
		delete (*eit);

	entrylist.clear();

	//---------------------------------------------------------------------------------------
	// START Scanning
	//---------------------------------------------------------------------------------------
	tasklist.clear();

	EnumWindows(Popup::EnumWindowsProc, (LPARAM) this);
	//---------------------------------------------------------------------------------------
	// END Scanning
	//---------------------------------------------------------------------------------------

	//Sorting
	SortTasks(tasklist);

	//Initializing of Values
	int nextXPosition = leftBorder;
	int nextYPosition = topBorder;
	int hiddenPart = 0;
	int menuBreaks = 0;
	int maxNeededWidth = 0;
	int adaptedmaxHeight = 0;

	for (hWndTaskListIterator tit = tasklist.begin(); tit != tasklist.end(); ++tit)
	{
		ENTRY* newitem = new ENTRY;

		newitem->x = nextXPosition;
		
		char windowCaption[MAX_PATH_LENGTH] = {0};
		GetWindowText( (*tit), windowCaption, MAX_PATH_LENGTH );
		newitem->caption = windowCaption;

		newitem->y = nextYPosition;
		newitem->height = entryHeight;
				
		if (nextYPosition+newitem->height+bottomBorder > maxHeight)
		{
			if (autoMenuBreak)
			{
				if (adaptedmaxHeight < nextYPosition-ySpacing+bottomBorder)
					adaptedmaxHeight = nextYPosition-ySpacing+bottomBorder;

				nextXPosition = nextXPosition+maxNeededWidth+xSpacing;
				nextYPosition = topBorder;

				newitem->x = nextXPosition;
				newitem->y = nextYPosition;

				menuBreaks++;

				nextYPosition = nextYPosition+newitem->height+ySpacing;
			}
			else
			{
				if (adaptedmaxHeight < nextYPosition-ySpacing+bottomBorder)
					adaptedmaxHeight = nextYPosition-ySpacing+bottomBorder;
					
				nextYPosition = topBorder;

				newitem->y = nextYPosition;

				hiddenPart++;
				
				nextYPosition = nextYPosition+newitem->height+ySpacing;
			}
		}
		else
			nextYPosition = nextYPosition+newitem->height+ySpacing;
						
		newitem->hiddenPart = hiddenPart;
		newitem->multiPart = menuBreaks;

		newitem->hWnd = (*tit);

		POINT tmp = buttonfont->measure(defaultDC, newitem->caption);
		if (maxNeededWidth < tmp.x)
			maxNeededWidth = tmp.x;

		entrylist.insert(entrylist.end(), newitem);
	}

	//Check Max/Min EntryWidth
	maxNeededWidth = max(min(maxNeededWidth, maxEntryWidth), minEntryWidth);

	//Measure with Type 1 only check TitleFont Settings for maxNeededWidth
	POINT tmp = buttonfont->measure(defaultDC, captionname, 1);
	int neededTitleWidth = leftBorder+tmp.x+rightBorder;

	//Adapt EntryWidth based on Max/Min Width
	if ( leftBorder+(menuBreaks+1)*maxNeededWidth+menuBreaks*xSpacing+rightBorder >= maxWidth )
	{
		actualwidth = maxWidth;
		maxNeededWidth = (actualwidth-leftBorder-rightBorder-menuBreaks*xSpacing)/(menuBreaks+1);
	}
	else if (leftBorder+(menuBreaks+1)*maxNeededWidth+menuBreaks*xSpacing+rightBorder <= minWidth )
	{
		actualwidth = minWidth;
		maxNeededWidth = (actualwidth-leftBorder-rightBorder-menuBreaks*xSpacing)/(menuBreaks+1);
	}
	else
	{
		actualwidth = leftBorder+(menuBreaks+1)*maxNeededWidth+menuBreaks*xSpacing+rightBorder;
		actualwidth = min(max(neededTitleWidth, actualwidth), maxWidth);
	}

	if (hiddenPart != 0 || menuBreaks != 0)
		actualheight = adaptedmaxHeight;
	else
		actualheight = min(nextYPosition-ySpacing+bottomBorder, maxHeight);
	actualheight = max(actualheight, minHeight);

	for (eit = entrylist.begin(); eit != entrylist.end(); ++eit)
	{
		if (startentry == NULL)
			startentry = *eit;

		if (menuBreaks == 0)
			(*eit)->width = max(maxNeededWidth, actualwidth-leftBorder-rightBorder);
		else
		{
			if (minWidth > leftBorder+maxNeededWidth+rightBorder)
				maxNeededWidth = (minWidth-(menuBreaks*xSpacing)-leftBorder-rightBorder)/(menuBreaks+1);

			(*eit)->width = maxNeededWidth;
			
			(*eit)->x = leftBorder + (*eit)->multiPart*(maxNeededWidth + xSpacing);
		}
	}

	maxPart = hiddenPart;
	maxMenuBreak = menuBreaks;

	reposition(2, 0, 0, actualwidth, actualheight);
}

BOOL WINAPI Popup::EnumWindowsProc(HWND hWnd, LPARAM lParam)
{
	Popup *popup = (Popup *) lParam;
	if (popup && IsAppWindow(hWnd))
	{
		popup->tasklist.insert(popup->tasklist.end(), hWnd);
	}
	return TRUE;
}

void Popup::CreateSubMenu( string newsubname, string newrootpath, ENTRY* pNewParentEntry, bool jumpTo)
{
	Popup *popup = new Popup(name, newsubname, popuptexture, buttontexture, buttonicon, buttonfont);
			
	subpopupList.insert(subpopupList.end(), popup);
					
	popup->load(hInstance);

	if (!newrootpath.empty())
	{
		popup->ScanSubMenu(newrootpath, pNewParentEntry->entryType);
		popup->rootPath = newrootpath;
	}

	popup->pParentPopup = this;
	popup->pParentEntry = pNewParentEntry;

	if ( pNewParentEntry->entryType == -2 )
		popup->dynamicPopup = true;

	popup->show();

	if (jumpTo)
		SetCursorPos(popup->actualx+popup->startentry->x+2, popup->actualy+popup->startentry->y+2);
}

void Popup::CreateShellSubMenu( string newsubname, ENTRY* pNewParentEntry, int clsid, bool jumpTo)
{
	Popup *popup = new Popup(name, newsubname, popuptexture, buttontexture, buttonicon, buttonfont);
				
	subpopupList.insert(subpopupList.end(), popup);
										
	popup->load(hInstance);

	popup->ScanShellFolder(clsid);

	popup->pParentPopup = this;
	popup->pParentEntry = pNewParentEntry;

	popup->show();
	
	if (jumpTo)
		SetCursorPos(popup->actualx+popup->startentry->x+2, popup->actualy+popup->startentry->y+2);
}

void Popup::CreateTaskSubMenu( string newsubname, ENTRY* pNewParentEntry, bool jumpTo)
{
	Popup *popup = new Popup(name, newsubname, popuptexture, buttontexture, buttonicon, buttonfont);
				
	subpopupList.insert(subpopupList.end(), popup);
										
	popup->load(hInstance);

	popup->pParentPopup = this;
	popup->pParentEntry = pNewParentEntry;

	popup->show();	
	
	if (jumpTo)
		SetCursorPos(popup->actualx+popup->startentry->x+2, popup->actualy+popup->startentry->y+2);
}

//////////////////////////////////////////////////////////////////////
// Window Related: Show/Hide/Reposition
//////////////////////////////////////////////////////////////////////

void Popup::reposition(int type, int newx, int newy, int newwidth, int newheight, int externCall)
{
	if ( externCall == 1 )
	{
		minWidth = newwidth;
		minHeight = newheight;
		maxWidth = newwidth;
		maxHeight = newheight;

		for(PopupListIterator plit = subpopupList.begin(); plit != subpopupList.end(); plit++)
		{
			(*plit)->hide();
			delete *plit;
		}

		subpopupList.clear();

		for (EntryListIterator eit = entrylist.begin(); eit != entrylist.end(); ++eit)
			delete (*eit);

		entrylist.clear();

		ParseStepRC(true);
	}
	else
	{
		if ( type == 1 )
		{
			SetWindowPos(hWnd, 0, newx, newy, 0, 0,
					SWP_NOACTIVATE | SWP_NOSIZE | SWP_NOZORDER);
		}
		else if ( type == 2 )
		{
			SetWindowPos(hWnd, 0, 0, 0, newwidth, newheight,
					SWP_NOACTIVATE | SWP_NOMOVE | SWP_NOZORDER);
		}
		else if ( type == 3 )
		{
			SetWindowPos(hWnd, 0, newx, newy, newwidth, newheight,
					SWP_NOACTIVATE | SWP_NOZORDER);
		}
		
		//Set new Rect for ToolTip Tracker
		if ( type != 1 && quicklaunchMode != 0 && ToolTips )
			updateHint(hWnd, "", true);
	}
}

void Popup::show(bool setcursor)
{
	bool showit = false;

	//UnPin "SubPopupMenus", if they are called by Parent
	if ( PinnedMode && pParentPopup != NULL && pParentEntry != NULL )
	{
		PinnedMode = false;
		setAlwaysOnTop(true);

		showit = true;
	}

	//Check if Main Popup is visible
	Popup* tmpPointer = pParentPopup;
	if (tmpPointer != NULL && !showit)
	{
		while (tmpPointer->pParentPopup != NULL)
		{
			if ( tmpPointer->isVisible() && tmpPointer->PinnedMode )
			{
				showit = true;
				break;
			}
			tmpPointer = tmpPointer->pParentPopup;
		}
			
		if ( !tmpPointer->isVisible() )
			return;
		else
			showit = true;
	}
	else
		showit = true;

	if (showit)
	{
		//Reposition "SubPopupMenus"
		if (pParentPopup != NULL && pParentEntry != NULL)
		{
			if (dynamicPopup)
				ScanSubMenu(rootPath, true);
			else if (pParentPopup->hWnd == pParentEntry->hWnd)
				ScanTasks();

			int x = 0;
			int y = 0;

			//QuickLaunch Mode
			if (quicklaunchMode != 0)
			{
				if (quicklaunchMode == 1)
				{
					x = pParentPopup->actualx + pParentEntry->x + overlapX;
					
					if (quicklaunchOpenTo == 1)
						y = pParentPopup->actualy - actualheight + overlapY;
					else if (quicklaunchOpenTo == 2)
						y = pParentPopup->actualy + pParentPopup->actualheight + overlapY;
				}
				else if (quicklaunchMode == 2)
				{
					if (quicklaunchOpenTo == 1)
						x = pParentPopup->actualx + pParentPopup->actualwidth + overlapX;
					else if (quicklaunchOpenTo == 2)
						x = pParentPopup->actualx - actualwidth + overlapX;

					y = pParentPopup->actualy + pParentEntry->y + overlapY;
				}

				//InScreen Check
				if (x+actualwidth > screenX)
					x = pParentPopup->actualx-actualwidth-overlapX;

				if (y+actualheight > screenY)
					y = screenY-actualheight;

				//Minimum x, y is 0
				if ( x < 0 )
					x = 0;
				if ( y < 0 )
					y = 0;

				reposition(1, x, y, 0, 0);
			}
			//Normal PopupMenus
			else
			{
				if ( openToLeft )
				{
					if ( pParentEntry->multiPart == 0 )
						x = pParentPopup->actualx - actualwidth - overlapX;
					else
						x = pParentPopup->actualx + pParentEntry->x - actualwidth - pParentPopup->xSpacing/2;
							
					y = pParentPopup->actualy + pParentEntry->y + overlapY;

					//InScreen Check
					if ( x < 0 && pParentEntry->multiPart == 0 && pParentPopup->maxMenuBreak != 0 )
					{
						x = pParentPopup->actualx + pParentEntry->x + pParentEntry->width + pParentPopup->xSpacing/2;
					}
					else if ( x < 0 )
					{
						x = pParentPopup->actualx + pParentPopup->actualwidth + overlapX;
					}

					if (y+actualheight > screenY)
						y = screenY-actualheight;

					//Maximum x is screenX, Minimum y is 0
					if ( x > screenX )
						x = screenX-actualwidth;
					if ( y < 0 )
						y = 0;
				}
				else
				{
					if ( pParentEntry->multiPart != pParentPopup->maxMenuBreak && pParentPopup->maxMenuBreak != 0 )
						x = pParentPopup->actualx + pParentEntry->x + pParentEntry->width + pParentPopup->xSpacing/2;
					else
						x = pParentPopup->actualx + pParentPopup->actualwidth + overlapX;
			
					y = pParentPopup->actualy + pParentEntry->y + overlapY;

					//InScreen Check
					if ( x+actualwidth > screenX && pParentEntry->multiPart == 0 )
					{
						x = pParentPopup->actualx - actualwidth - overlapX;
					}
					else if ( x+actualwidth > screenX )
					{
						x = pParentPopup->actualx + pParentEntry->x - actualwidth - pParentPopup->xSpacing/2;
					}

					if (y+actualheight > screenY)
						y = screenY-actualheight;

					//Minimum x, y is 0
					if ( x < 0 )
						x = 0;
					if ( y < 0 )
						y = 0;
				}

				reposition(1, x, y, 0, 0);
			}
		}

		if ( hasInfoEntry )
		{
			for (EntryListIterator eit = entrylist.begin(); eit != entrylist.end(); ++eit)
			{
				if ( !(*eit)->info.empty() )
					(*eit)->caption = InfoUpdate( (*eit)->info );
			}

			repaint(false, true);
		}
		else if (alphaMap)
			repaint();

		if (alphaFade)
		{
			UpdateAlpha = 0;

			UpdateWindowTransparent(hWnd, UpdateAlpha);

			ShowWindow(hWnd, SW_SHOW);
				
			KillTimer(hWnd, TIMER_ALPHAFADEOUT);
			SetTimer(hWnd, TIMER_ALPHAFADEIN, alphaFadeDelay, NULL);
		}
		else
		{
			ShowWindow(hWnd, SW_SHOW);
		}

		//16Bit Compatibility
		if ( !alphaMap )
			repaint(true);

		//Only Main Popups
		if (pParentPopup == NULL)
			SetTimer(hWnd, TIMER_ACTIVE, hideDelay, NULL);

		visible = true;

		visiblepopupList.insert(visiblepopupList.end(), this);

		if ( setcursor )
			SetCursorPos(actualx+startentry->x+2, actualy+startentry->y+2);

		if ( !BangToCall.empty() )
		{
			if ( !openSound.empty() )
				ParseBangCommand(hWnd, "!playsound", openSound.c_str());

			if ( !openCommand.empty() )
			{
				CString result = openCommand.c_str();

				CString worker = BangToCall.c_str();
				worker.Replace("!", "");
				
				result.Replace("%[popupname]%", worker);
				result.Replace("%{popupname}%", worker);

				LSExecute(hWnd, result, SW_SHOWNORMAL);
			}

			SetForegroundWindow(hWnd);
			SetActiveWindow(hWnd);
			SetFocus(hWnd);
		}
	}
}

void Popup::hide(bool noFade)
{
	bool wasVisible = visible;

	if ( !PinnedMode && visible )
	{
		if (alphaFade && !noFade)
		{
			UpdateAlpha = alphaTransparency;

			KillTimer(hWnd, TIMER_ALPHAFADEIN);
			SetTimer(hWnd, TIMER_ALPHAFADEOUT, alphaFadeDelay, NULL);
		}
		else if ( !alphaFade || noFade )
		{
			KillTimer(hWnd, TIMER_ALPHAFADEIN);
			KillTimer(hWnd, TIMER_ALPHAFADEOUT);
			ShowWindow(hWnd, SW_HIDE);
		
			lastactiveentry = NULL;
			hoverentry = NULL;
			lastkeyentry = -1;

			currentPart = 0;
			
			visible = false;
		}

		for( PopupListIterator plit = visiblepopupList.begin(); plit != visiblepopupList.end(); plit++)
		{
			if ( (*plit) == this )
			{
				visiblepopupList.erase( plit );
				break;
			}
		}
		
		KillTimer(hWnd, TIMER_ACTIVE);
	
		for( plit = subpopupList.begin(); plit != subpopupList.end(); plit++)
		{
			if ( !(*plit)->PinnedMode && (*plit)->isVisible() )
				(*plit)->hide(noFade);
		}
		
		if ( !BangToCall.empty() )
		{	
			if ( !closeSound.empty() && !execSoundPlayed )
				ParseBangCommand(hWnd, "!playsound", closeSound.c_str());

			execSoundPlayed = false;

			if ( !closeCommand.empty() && wasVisible )
			{
				CString result = closeCommand.c_str();

				CString worker = BangToCall.c_str();
				worker.Replace("!", "");
				
				result.Replace("%[popupname]%", worker);
				result.Replace("%{popupname}%", worker);

				LSExecute(hWnd, result, SW_SHOWNORMAL);
			}
		}
	}
}


void Popup::ToggleShade()
{
	if (shaded != -1)
	{
		if ( !BangToCall.empty() && !unshadeCommand.empty() )
		{
			CString result = unshadeCommand.c_str();

			CString worker = BangToCall.c_str();
			worker.Replace("!", "");
				
			result.Replace("%[popupname]%", worker);
			result.Replace("%{popupname}%", worker);

			LSExecute(hWnd, result, SW_SHOWNORMAL);
		}

		int tmpHeight = shaded;
		shaded = -1;
		reposition(2, 0, 0, actualwidth, tmpHeight);
	}
	else
	{
		if ( !BangToCall.empty() && !shadeCommand.empty() )
		{
			CString result = shadeCommand.c_str();

			CString worker = BangToCall.c_str();
			worker.Replace("!", "");
				
			result.Replace("%[popupname]%", worker);
			result.Replace("%{popupname}%", worker);

			LSExecute(hWnd, result, SW_SHOWNORMAL);
		}

		shaded = actualheight;
		reposition(2, 0, 0, actualwidth, max(max(topBorder+bottomBorder, popuptexture->getMinNeededHeight()), orgminHeight));
	}
}

//------------------------------------------------------------
//Mouse Related Actions (Clicks, Enter, Leave, Move, ...)
//------------------------------------------------------------

void Popup::ontemplateButtonDown(const int &x, const int &y, int actionType)
{
	if ( actionType == 6 )
	{
		if ( ((showCaption == 1 || showCaption == 0) && MoveAllowed && y >= 0 && y <= topBorder)
			|| ((showCaption == 2 || showCaption == 0) && MoveAllowed && y >= actualheight-bottomBorder && y <= actualheight) )
		{
			mousePressed = true;
			GetCursorPos(&savedpt);
			ScreenToClient(hWnd, &savedpt);
			moveButtonDown = true;
			SetCapture(hWnd);
		}
	}
	else
		SetCapture(hWnd);
}

void Popup::ontemplateButtonUp(const int &x, const int &y, int actionType, int mousebutton)
{
	bool normalactions = true;
	bool somethingdone = false;
	if ( actionType == 1 || actionType == 2 || actionType == 7 )
	{
		if ( ((showCaption == 1 || showCaption == 0) && y >= 0 && y <= topBorder)
			|| ((showCaption == 2 || showCaption == 0) && y >= actualheight-bottomBorder && y <= actualheight) )
		{
			normalactions = false;
			somethingdone = true;
			if ( PinnedMode && (actionType == 2 || actionType == 7) )
			{
				PinnedMode = false;
				setAlwaysOnTop(true);
			}
			if (!PinnedMode && actionType == 1)
			{
				PinnedMode = true;
				if ( pinnedPopupNotOnTop )
					setAlwaysOnTop(false);
				else
					setAlwaysOnTop(true);
			}
			if ( actionType == 2 || actionType == 7 )
				hide();
		}
	}
	else if ( actionType == 3 )
	{
		if ( ((showCaption == 1 || showCaption == 0) && y >= 0 && y <= topBorder)
			|| ((showCaption == 2 || showCaption == 0) && y >= actualheight-bottomBorder && y <= actualheight) )
		{
			normalactions = false;
			somethingdone = true;
			if ( PinnedMode )
			{
				PinnedMode = false;
				setAlwaysOnTop(true);
				hide();
			}
			else if ( !PinnedMode )
			{
				PinnedMode = true;
				if ( pinnedPopupNotOnTop )
					setAlwaysOnTop(false);
				else
					setAlwaysOnTop(true);
			}
		}
	}
	else if ( actionType == 4 )
	{
		if ( ((showCaption == 1 || showCaption == 0) && y >= 0 && y <= topBorder)
			|| ((showCaption == 2 || showCaption == 0) && y >= actualheight-bottomBorder && y <= actualheight) )
		{
			normalactions = false;
			if ( ShadeAllowed && !(0x8000 & GetKeyState(VK_CONTROL)) && !(0x8000 & GetKeyState(VK_SHIFT)) )
			{
				ToggleShade();
				somethingdone = true;
			}
		}
	}
	else if ( actionType == 5 )
	{
		if ( ((showCaption == 1 || showCaption == 0) && y >= 0 && y <= topBorder)
			|| ((showCaption == 2 || showCaption == 0) && y >= actualheight-bottomBorder && y <= actualheight) )
		{
			normalactions = false;
			if ( PinnedMode )
			{
				somethingdone = true;
				setAlwaysOnTop(!alwaysOnTop);
			}
		}
	}
	else if ( actionType == 6 )
	{
		if ( moveButtonDown )
		{
			somethingdone = true;
			normalactions = false;
			if ( MoveAllowed && mousePressed && mouseMoved && pParentPopup != NULL )
			{
				if ( AutoPin && !PinnedMode )
				{
					PinnedMode = true;
					if (pinnedPopupNotOnTop)
						setAlwaysOnTop(false);
					else
						setAlwaysOnTop(true);
					SetTimer(hWnd, TIMER_ACTIVE, hideDelay, NULL);
				}	
			}
			mouseMoved = false;

			//To be on the save side
			mousePressed = false;

			moveButtonDown = false;
		}
	}

	if ( normalactions && shaded == -1 )
	{
		if ( mousebutton == 1 )
			//ActionAtPoint: First Bool is Normal Action, Second is ContextMenu
			ActionAtPoint(x, y, true);
		else if ( mousebutton == 2 )
			//ActionAtPoint: First Bool is Normal Action, Second is ContextMenu
			ActionAtPoint(x, y, false, true);
	}
	if ( !somethingdone && shaded == -1 && mousebutton == 3 && (x >= 4 || y >= 4) )
	{
		//Bool Value: True = Upwards, False = downwards
		repaint(SelectNextEntry(false));
	}

	ReleaseCapture();
}

void Popup::onLButtonDblClk(int x, int y)
{
	if ( ((showCaption == 1 || showCaption == 0) && HideAllowed && y >= 0 && y <= topBorder)
		|| ((showCaption == 2 || showCaption == 0) && HideAllowed && y >= actualheight-bottomBorder && y <= actualheight) )
	{
		if (PinnedMode)
		{
			PinnedMode = false;
			setAlwaysOnTop(true);
			hide();
		}
		else
			hide();
	}
	else 
	{
		for (EntryListIterator eit = entrylist.begin(); eit != entrylist.end(); ++eit)
		{
			if (currentPart != (*eit)->hiddenPart || (*eit)->entryType >= 0)
				continue;

			if ( x < ((*eit)->x + (*eit)->width) && x >= (*eit)->x &&
				 y < ((*eit)->y + (*eit)->height) && y >= (*eit)->y )
			{
				for(PopupListIterator plit = subpopupList.begin(); plit != subpopupList.end(); plit++)
				{
					if (stricmp( (*eit)->caption.c_str(), (*plit)->captionname.c_str()) == NULL)
					{
						if ( !(*eit)->action.empty() )
						{
							if ((*eit)->pipedList.size() == 1)
							{
								LSExecute(NULL, ("explorer.exe /e," + (*eit)->action).c_str(), SW_SHOWNORMAL);
								Popup *tmpPointer = pParentPopup;
								if (tmpPointer != NULL)
								{
									while (tmpPointer->pParentPopup != NULL)
										tmpPointer = tmpPointer->pParentPopup;
									tmpPointer->hide(true);
								}
								else
									hide(true);
							}
							else
							{
								HMENU GroupMenu = CreatePopupMenu();

								int counter = 1;
								for(StringListIterator sit = (*eit)->pipedList.begin(); sit != (*eit)->pipedList.end(); ++sit)
								{
									AppendMenu( GroupMenu, MF_STRING, counter, (*sit).c_str() );
									counter++;
								}

								// display the menu
								POINT pt;
								GetCursorPos( &pt );

								int command = TrackPopupMenu( GroupMenu,
															  TPM_RETURNCMD | TPM_RIGHTBUTTON,
															  pt.x, pt.y, 0,
															  this->gethWnd(),
															  NULL );

								if ( command )
								{
									StringListIterator sit = (*eit)->pipedList.begin();
									counter = 1; 
									while (counter < command)
									{
										sit++;
										counter++;
									}

									if ( counter <= (*eit)->pipedList.size() )
									{
										LSExecute(NULL, ("explorer.exe /e," + (*sit)).c_str(), SW_SHOWNORMAL);
										Popup *tmpPointer = pParentPopup;
										if (tmpPointer != NULL)
										{
											while (tmpPointer->pParentPopup != NULL)
												tmpPointer = tmpPointer->pParentPopup;
											tmpPointer->hide(true);
										}
										else
											hide(true);
									}
								}
											
								if (GroupMenu)
									DestroyMenu(GroupMenu);
							}
						}
						break;
					}
				}
			}
		}
	}
}

void Popup::onWheelDown(int x, int y)
{
	if ( (0x8000 & GetKeyState(VK_CONTROL)) )
	{
		if ( maxPart != 0)
		{
			currentPart++;
			if (currentPart > maxPart)
				currentPart = 0;

			oldmovetracker.x = 0;

			repaint(true);
		}
	}
	else
	{
		//Bool Value: True = Upwards, False = downwards
		repaint(SelectNextEntry(false));
	}
}

void Popup::onWheelUp(int x, int y)
{
	if ( (0x8000 & GetKeyState(VK_CONTROL)) )
	{
		if ( maxPart != 0)
		{
			currentPart--;
			if (currentPart < 0)
				currentPart = maxPart;

			oldmovetracker.x = 0;

			repaint(true);
		}
	}
	else
	{
		//Bool Value: True = Upwards, False = downwards
		repaint(SelectNextEntry(true));
	}
}

void Popup::onKey(int nKey)
{
	switch (nKey)
	{
		case VK_F5:
		{
			if ( !BangToCall.empty() && pParentPopup == NULL )
			{
				if ( compatibilitymode )
					ParseBangCommand(hWnd, "!popuprescan", "");
				else
					ParseBangCommand(hWnd, "!xpopuprescan", "");
			}
			else if ( pParentPopup != NULL )
			{
				for(PopupListIterator plit = pParentPopup->subpopupList.begin(); plit != pParentPopup->subpopupList.end(); plit++)
				{
					if ( (*plit) == this )
					{
						pParentPopup->subpopupList.erase( plit );
						(*plit)->hide(true);
						delete *plit;
						break;
					}
				}
			}
		}
		break;

		case VK_PRIOR:
		{
			if ( maxPart != 0)
			{
				currentPart--;
				if (currentPart < 0)
					currentPart = maxPart;

				oldmovetracker.x = 0;

				repaint(true);
			}
		}
		break;

		case VK_NEXT:
		{
			if ( maxPart != 0)
			{
				currentPart++;
				if (currentPart > maxPart)
					currentPart = 0;

				oldmovetracker.x = 0;

				repaint(true);
			}
		}
		break;

		case VK_RIGHT:
		{
			for (EntryListIterator eit = entrylist.begin(); eit != entrylist.end(); ++eit)
			{
				if ( (*eit) == hoverentry && (*eit)->entryType < 0 )
				{
					lastactiveentry = (*eit);
					bool alreadyCreated = false;
					for(PopupListIterator plit = subpopupList.begin(); plit != subpopupList.end(); plit++)
					{
						if (stricmp( (*eit)->caption.c_str(), (*plit)->captionname.c_str()) == NULL)
						{
							(*plit)->hoverentry = (*plit)->startentry;
							(*plit)->show(true);
							alreadyCreated = true;
						}
						else
						{
							if ( !(*plit)->PinnedMode && (*plit)->isVisible() )
								(*plit)->hide();
						}
					}
					if (!alreadyCreated && (*eit)->clsid == 0 && (*eit)->hWnd == NULL)
						CreateSubMenu( ("sub_"+(*eit)->caption), (*eit)->action, (*eit), true);
					else if (!alreadyCreated && (*eit)->clsid != 0)
						CreateShellSubMenu( ("sub_"+(*eit)->caption), (*eit), (*eit)->clsid, true);
					else if (!alreadyCreated && (*eit)->hWnd == hWnd)
						CreateTaskSubMenu( ("sub_"+(*eit)->caption), (*eit), true );
				
					break;
				}
			}
		}
		break;

		case VK_LEFT:
		{
			if (pParentPopup != NULL)
			{
				if (pParentPopup->lastactiveentry != NULL)
				{
					pParentPopup->hoverentry = pParentPopup->lastactiveentry;
					SetCursorPos(pParentPopup->actualx+pParentPopup->lastactiveentry->x+2, pParentPopup->actualy+pParentPopup->lastactiveentry->y+2);	
				}
				else
				{
					pParentPopup->hoverentry = pParentPopup->lastactiveentry;
					SetCursorPos(pParentPopup->actualx+pParentPopup->startentry->x+2, pParentPopup->actualy+pParentPopup->startentry->y+2);	
				}

				if ( !PinnedMode && visible )
					hide();

				pParentPopup->repaint();

				SetForegroundWindow(pParentPopup->gethWnd());
				SetFocus(pParentPopup->gethWnd());
			}
		}
		break;

		case VK_ESCAPE:
		{
			if ( HideAllowed )
			{
				//Search for Main Popup and Hide All SubPopups
				Popup* tmpPopupPointer = pParentPopup;
				if (tmpPopupPointer != NULL)
				{
					while (tmpPopupPointer->pParentPopup != NULL)
						tmpPopupPointer = tmpPopupPointer->pParentPopup;

					if ( !tmpPopupPointer->PinnedMode && tmpPopupPointer->isVisible() )
						tmpPopupPointer->hide();
				}
				else
				{
					if ( !PinnedMode && visible )
						hide();
				}
			}
		}
		break;

		case VK_UP:
		{
			for(PopupListIterator plit = subpopupList.begin(); plit != subpopupList.end(); plit++)
			{
				if ( !(*plit)->PinnedMode && (*plit)->isVisible() )
					(*plit)->hide();
			}

			//Bool Value: True = Upwards, False = downwards
			repaint(SelectNextEntry(true));
		}
		break;

		case VK_DOWN:
		{
			for(PopupListIterator plit = subpopupList.begin(); plit != subpopupList.end(); plit++)
			{
				if ( !(*plit)->PinnedMode && (*plit)->isVisible() )
					(*plit)->hide();
			}

			//Bool Value: True = Upwards, False = downwards
			repaint(SelectNextEntry(false));
		}
		break;

		case VK_RETURN:
		{
			for (EntryListIterator eit = entrylist.begin(); eit != entrylist.end(); ++eit)
			{
				//Normal Action
				if ( (*eit) == hoverentry && (*eit)->entryType >= 0 && !(*eit)->action.empty() && (*eit)->pidl == NULL )
				{
					LSExecute(NULL, (*eit)->action.c_str(), NULL);

					Popup* tmpPopupPointer = pParentPopup;
					if (tmpPopupPointer != NULL)
					{
						while (tmpPopupPointer->pParentPopup != NULL)
							tmpPopupPointer = tmpPopupPointer->pParentPopup;

						if ( !execSound.empty() )
						{
							tmpPopupPointer->execSoundPlayed = true;
							ParseBangCommand(hWnd, "!playsound", execSound.c_str());
						}
						tmpPopupPointer->hide(true);
					}
					else
					{
						if ( !execSound.empty() )
						{
							execSoundPlayed = true;
							ParseBangCommand(hWnd, "!playsound", execSound.c_str());
						}
						hide(true);
					}

					break;
				}
				//System Action
				else if ( (*eit) == hoverentry && (*eit)->entryType >= 0 && (*eit)->action.empty() && (*eit)->pidl != NULL )
				{
					Invoke( (*eit)->pidl );
					
					Popup* tmpPopupPointer = pParentPopup;
					if (tmpPopupPointer != NULL)
					{
						while (tmpPopupPointer->pParentPopup != NULL)
							tmpPopupPointer = tmpPopupPointer->pParentPopup;

						if ( !execSound.empty() )
						{
							tmpPopupPointer->execSoundPlayed = true;
							ParseBangCommand(hWnd, "!playsound", execSound.c_str());
						}
						tmpPopupPointer->hide(true);
					}
					else
					{
						if ( !execSound.empty() )
						{
							execSoundPlayed = true;
							ParseBangCommand(hWnd, "!playsound", execSound.c_str());
						}
						hide(true);
					}

					break;
				}
				//Tasks
				else if ( (*eit)->hWnd != NULL )
				{
					SwitchToThisWindow( (*eit)->hWnd, SW_RESTORE );
					
					Popup* tmpPopupPointer = pParentPopup;
					if (tmpPopupPointer != NULL)
					{
						while (tmpPopupPointer->pParentPopup != NULL)
							tmpPopupPointer = tmpPopupPointer->pParentPopup;

						if ( !execSound.empty() )
						{
							tmpPopupPointer->execSoundPlayed = true;
							ParseBangCommand(hWnd, "!playsound", execSound.c_str());
						}
						tmpPopupPointer->hide(true);
					}
					else
					{
						if ( !execSound.empty() )
						{
							execSoundPlayed = true;
							ParseBangCommand(hWnd, "!playsound", execSound.c_str());
						}
						hide(true);
					}

					break;
				}
				//Folder
				else if ( (*eit) == hoverentry && (*eit)->entryType < 0 )
				{
					lastactiveentry = (*eit);
					bool alreadyCreated = false;
					for(PopupListIterator plit = subpopupList.begin(); plit != subpopupList.end(); plit++)
					{
						if (stricmp( (*eit)->caption.c_str(), (*plit)->captionname.c_str()) == NULL)
						{
							(*plit)->hoverentry = (*plit)->startentry;
							(*plit)->show(true);
							alreadyCreated = true;
						}
						else
						{
							if ( !(*plit)->PinnedMode && (*plit)->isVisible() )
								(*plit)->hide();
						}
					}
					if (!alreadyCreated && (*eit)->clsid == 0 && (*eit)->hWnd == NULL)
						CreateSubMenu( ("sub_"+(*eit)->caption), (*eit)->action, (*eit), true);
					else if (!alreadyCreated && (*eit)->clsid != 0)
						CreateShellSubMenu( ("sub_"+(*eit)->caption), (*eit), (*eit)->clsid, true);
					else if (!alreadyCreated && (*eit)->hWnd == hWnd)
						CreateTaskSubMenu( ("sub_"+(*eit)->caption), (*eit), true );
				
					break;
				}
			}
		}
		break;

		default:
		{
			int howmuchfolders = 0;
			int howmuchfiles = 0;
			int iWhatFound = 0;
			int entrycounter = 0;
			
			for (EntryListIterator eit = entrylist.begin(); eit != entrylist.end(); ++eit)
			{
				entrycounter++;
				if ( (*eit)->keyCode == nKey && (*eit)->entryType < 0 )
					howmuchfolders++;
				else if ( (*eit)->keyCode == nKey && (*eit)->entryType >= 0 )
					howmuchfiles++;

				if ( hoverentry == (*eit) && (*eit)->keyCode == nKey )
					lastkeyentry = entrycounter;
			}

			if ( howmuchfolders == 0 && howmuchfiles == 0 )
				break;
	
			entrycounter = 0;
			for (eit = entrylist.begin(); eit != entrylist.end(); ++eit)
			{
				entrycounter++;
				if ( ((*eit)->keyCode == nKey || (inChordingTimeOut && (*eit)->keyCode1 == nKey)) && entrycounter > lastkeyentry )
				{
					if ( (*eit)->entryType < 0 )
						lastactiveentry = (*eit);
					hoverentry = (*eit);
					lastkeyentry = entrycounter;
				
					if ( (*eit)->hiddenPart != currentPart )
						currentPart = (*eit)->hiddenPart;

					SetCursorPos(actualx+(*eit)->x+2, actualy+(*eit)->y+2);

					iWhatFound = (((*eit)->entryType < 0 && !(*eit)->action.empty()) ? 2 : 1);

					break;
				}
			}
			if (lastkeyentry != 0 && iWhatFound == 0)
			{
				entrycounter = 0;
				for (EntryListIterator eit = entrylist.begin(); eit != entrylist.end(); ++eit)
				{
					entrycounter++;
					if ( (*eit)->keyCode == nKey || (inChordingTimeOut && (*eit)->keyCode1 == nKey) )
					{
						if ( (*eit)->entryType < 0 )
							lastactiveentry = (*eit);
						hoverentry = (*eit);
						lastkeyentry = entrycounter;
					
						if ( (*eit)->hiddenPart != currentPart )
							currentPart = (*eit)->hiddenPart;

						SetCursorPos(actualx+(*eit)->x+2, actualy+(*eit)->y+2);

						iWhatFound = (((*eit)->entryType < 0 && !(*eit)->action.empty()) ? 2 : 1);

						break;
					}
				}
			}

			repaint(true);

			//One Folder ->Open It
			if (iWhatFound == 2 && howmuchfolders == 1 && howmuchfiles == 0)
			{
				bool alreadyCreated = false;
				for(PopupListIterator plit = subpopupList.begin(); plit != subpopupList.end(); plit++)
				{
					if (stricmp( lastactiveentry->caption.c_str(), (*plit)->captionname.c_str()) == NULL)
					{
						(*plit)->hoverentry = (*plit)->startentry;
						(*plit)->show(true);
						alreadyCreated = true;
					}
					else
					{
						if ( !(*plit)->PinnedMode && (*plit)->isVisible() )
							(*plit)->hide();
					}
				}
				if (!alreadyCreated && lastactiveentry->clsid == 0 && lastactiveentry->hWnd == NULL)
					CreateSubMenu( ("sub_"+lastactiveentry->caption), lastactiveentry->action, lastactiveentry, true);
				else if (!alreadyCreated && lastactiveentry->clsid != 0)
					CreateShellSubMenu( ("sub_"+lastactiveentry->caption), lastactiveentry, lastactiveentry->clsid, true);
				else if (!alreadyCreated && lastactiveentry->hWnd == hWnd)
					CreateTaskSubMenu( ("sub_"+lastactiveentry->caption), lastactiveentry, true);
			}
			//One File -> Open It
			else if (iWhatFound == 1 && howmuchfolders == 0 && howmuchfiles == 1 && hoverentry != NULL)
			{
				//Normal Action
				if ( hoverentry->entryType >= 0 && !hoverentry->action.empty() && hoverentry->pidl == NULL )
				{
					LSExecute(NULL, hoverentry->action.c_str(), NULL);
					
					Popup *tmpPointer = pParentPopup;
					if (tmpPointer != NULL)
					{
						while (tmpPointer->pParentPopup != NULL)
							tmpPointer = tmpPointer->pParentPopup;

						if ( !execSound.empty() )
						{
							tmpPointer->execSoundPlayed = true;
							ParseBangCommand(hWnd, "!playsound", execSound.c_str());
						}
						tmpPointer->hide(true);
					}
					else
					{
						if ( !execSound.empty() )
						{
							execSoundPlayed = true;
							ParseBangCommand(hWnd, "!playsound", execSound.c_str());
						}
						hide(true);
					}
					
					break;
				}
				//System Action
				else if ( hoverentry->entryType >= 0 && hoverentry->action.empty() && hoverentry->pidl != NULL )
				{
					Invoke( hoverentry->pidl );
					
					Popup *tmpPointer = pParentPopup;
					if (tmpPointer != NULL)
					{
						while (tmpPointer->pParentPopup != NULL)
							tmpPointer = tmpPointer->pParentPopup;

						if ( !execSound.empty() )
						{
							tmpPointer->execSoundPlayed = true;
							ParseBangCommand(hWnd, "!playsound", execSound.c_str());
						}
						tmpPointer->hide(true);
					}
					else
					{
						if ( !execSound.empty() )
						{
							execSoundPlayed = true;
							ParseBangCommand(hWnd, "!playsound", execSound.c_str());
						}
						hide(true);
					}

					break;
				}
				//Tasks
				else if ( hoverentry->hWnd != NULL )
				{
					SwitchToThisWindow( hoverentry->hWnd, SW_RESTORE );
					
					Popup *tmpPointer = pParentPopup;
					if (tmpPointer != NULL)
					{
						while (tmpPointer->pParentPopup != NULL)
							tmpPointer = tmpPointer->pParentPopup;

						if ( !execSound.empty() )
						{
							tmpPointer->execSoundPlayed = true;
							ParseBangCommand(hWnd, "!playsound", execSound.c_str());
						}
						tmpPointer->hide(true);
					}
					else
					{
						if ( !execSound.empty() )
						{
							execSoundPlayed = true;
							ParseBangCommand(hWnd, "!playsound", execSound.c_str());
						}
						hide(true);
					}
					
					break;
				}
			}
			else
			{
				inChordingTimeOut = true;
				SetTimer(hWnd, TIMER_CHORDING, 750, 0);
			}
		}
		break;
	}
}

void Popup::onMouseMove(int x, int y)
{
	if ( mousePressed )
	{
		KillTimer(hWnd, TIMER_MOUSETRACK);

		POINT newpt, pt;
		GetCursorPos(&newpt);
		pt = newpt;
		
		newpt.x = newpt.x-savedpt.x;
		newpt.y = newpt.y-savedpt.y;

		if ( !mouseMoved && newpt.x != actualx && newpt.y != actualy )
			mouseMoved = true;

		manualMove = false;

		if ( mouseMoved )
			reposition(1, newpt.x, newpt.y, 0, 0);
	}
	else if ( mouseInside )
	{
		SetTimer(hWnd, TIMER_MOUSETRACK, trackDelay, NULL);

		manualMove = true;

		HoverActions(x, y);
	}

	if( !mouseInside )
	{
		mouseInside = true;
		SetTimer(hWnd, TIMER_MOUSETRACK, trackDelay, NULL);

		if ( visiblepopupList.size() == 1 )
		{
			SetForegroundWindow(hWnd);
			SetActiveWindow(hWnd);
			SetFocus(hWnd);
			BringWindowToTop(hWnd);
		}

		lastactiveentry = startentry;

		HoverActions(x, y);
	}
}

void Popup::repaint(bool invalidateCache, bool regionUpdate)
{
	if(invalidateCache)
	{
		if(defaultDC && defaultBitmap)
		{
			defaultBitmap = (HBITMAP) SelectObject(defaultDC, defaultBitmap);
			DeleteObject(defaultBitmap);
			defaultBitmap = 0;
		}
	}

	if ( !alphaMap )
	{
		InvalidateRect(hWnd, NULL, false);
	
		if ( invalidateCache || regionUpdate || transparencyNeededAlways )
			updateRegion();
	}

	//Attention!! Background MUST exist
	if ( alphaMap && IsOS(OS_2KXP) && popuptexture && buttontexture && buttonicon && buttonfont )
		UpdateWindowTransparent(hWnd, alphaTransparency);
}

void Popup::updateRegion()
{
	HDC tmpDC = CreateCompatibleDC(NULL);
	HBITMAP tmpBmp = CreateBitmap(actualwidth, actualheight, 1, bitsperpixel, NULL);
	HGDIOBJ tmpObj = SelectObject(tmpDC, tmpBmp);

	onPaint(tmpDC);
					
	SelectObject(tmpDC, tmpObj);

	HRGN region = BitmapToRegion(tmpBmp, RGB(255, 0, 255), 0, 0, 0);
	HRGN windowRgn = CreateRectRgn(0, 0, 0, 0);
	CombineRgn(windowRgn, region, NULL, RGN_COPY);

	SetWindowRgn(hWnd, windowRgn, true);	

	DeleteObject(windowRgn);
	DeleteObject(region);
	DeleteObject(tmpBmp);
	DeleteObject(tmpObj);
	DeleteDC(tmpDC);
}

void Popup::onPaint(HDC hDC)
{
	int width = actualwidth;
	int height = actualheight;

	// make a rendition of the background
	HDC	backgroundDC = CreateCompatibleDC(hDC);

	//HBITMAP backgroundBitmap = CreateCompatibleBitmap(hDC, width, height);
	HBITMAP backgroundBitmap = CreateCompatibleBitmap(hDC, width, height);
	SelectObject(backgroundDC, backgroundBitmap);
		
	//Normal Background Painting
	popuptexture->apply(backgroundDC, 0, 0, width, height);

	// create & keep a cached rendition of the default popupmenu
	if(!defaultBitmap)
	{
		if(!defaultDC)
			defaultDC = CreateCompatibleDC(hDC);

		defaultBitmap = CreateCompatibleBitmap(hDC, width, height);
		SelectObject(defaultDC, defaultBitmap);
		
		// blt background into double buffer
		BitBlt(defaultDC, 0, 0, width, height, backgroundDC, 0, 0, SRCCOPY);

		//Only Painted if NOT Shaded
		if ( shaded == -1 )
		{
			int state = 0;
			int arrowstate = 0;
			for (EntryListIterator eit = entrylist.begin(); eit != entrylist.end(); ++eit)
			{
				if ( currentPart == (*eit)->hiddenPart && (*eit)->entryType != 2 )
				{
					//initialize
					state = 0;
					arrowstate = 0;

					if ( (*eit)->entryType == 1 )
					{
						state = 10;
					}
					else if ( (*eit)->entryType == 0 )
					{
						state = 1;
					}
					else if ( (*eit)->entryType < 0 )
					{
						state = 3;
						arrowstate = 31;
					}
					else
						continue;
				
					//Paint Entry Texture
					buttontexture->apply( defaultDC, (*eit)->x, (*eit)->y, (*eit)->width, (*eit)->height, state);
					//Paint Arrow
					if (arrowstate != 0)
					{
						buttontexture->apply( defaultDC, (*eit)->x, (*eit)->y, (*eit)->width, (*eit)->height, arrowstate);
						buttonfont->apply( defaultDC, (*eit)->x, (*eit)->y, (*eit)->width, (*eit)->height, (*eit)->caption, arrowstate );
					}
					
					//If No Separator, Icon and Text Painting is valid
					if (state != 10)
					{
						buttonicon->apply( defaultDC, (*eit)->x, (*eit)->y, (*eit)->width, (*eit)->height, state, (*eit)->iconPath, (*eit)->hWnd, (*eit)->pidl, DefaultIcon, alphaMap );

						if (quicklaunchMode == 0)
							buttonfont->apply( defaultDC, (*eit)->x, (*eit)->y, (*eit)->width, (*eit)->height, (*eit)->caption, state );
					}
				}
			}
		}
	}

	// double buffer for flicker-free paint
	HDC bufferDC = CreateCompatibleDC(hDC);

	HBITMAP bufferBitmap = CreateCompatibleBitmap(hDC, width, height);
	SelectObject(bufferDC, bufferBitmap);

	// blt default background into double buffer
	BitBlt(bufferDC, 0, 0, width, height, defaultDC, 0, 0, SRCCOPY);
	
	CString helpcaption = captionname.c_str();
	if (helpcaption.Find('&') != helpcaption.Find("& ") && helpcaption.Find('&') != helpcaption.Find("&\r") && helpcaption.Find('&') != helpcaption.Find("&\n") )
		helpcaption.Replace("&", "");
	
	if (showCaption == 1)
		buttonfont->apply( bufferDC, 0, 0, actualwidth, topBorder, (string) helpcaption, 0);
	else if (showCaption == 2)
		buttonfont->apply( bufferDC, 0, actualheight-bottomBorder, actualwidth, bottomBorder, (string) helpcaption, 0);

	//Only Painted if NOT Shaded
	if ( shaded == -1 )
	{
		int state = 0;
		int arrowstate = 0;
		if ( quicklaunchMode == 0 && hasInfoEntry )
		{
			for (EntryListIterator eit = entrylist.begin(); eit != entrylist.end(); ++eit)
			{
				if ( currentPart == (*eit)->hiddenPart && (*eit)->entryType == 2 )
				{
					state = 5;

					//Normal Background Painting behind ActiveEntry
					BitBlt(bufferDC, (*eit)->x, (*eit)->y, (*eit)->width, (*eit)->height, backgroundDC, (*eit)->x, (*eit)->y, SRCCOPY);

					//Paint Entry Texture
					buttontexture->apply( bufferDC, (*eit)->x, (*eit)->y, (*eit)->width, (*eit)->height, state);

					buttonfont->apply( bufferDC, (*eit)->x, (*eit)->y, (*eit)->width, (*eit)->height, (*eit)->caption, state );
				}
			}
		}
		for (EntryListIterator eit = entrylist.begin(); eit != entrylist.end(); ++eit)
		{
			//initialize
			state = 0;
			arrowstate = 0;
		
			if ( currentPart == (*eit)->hiddenPart && (*eit) == hoverentry )
			{
				if ( (*eit)->entryType == 0 )
				{
					state = 2;
				}
				else if ( (*eit)->entryType < 0 )
				{
					state = 4;
					arrowstate = 41;
				}
				else
					continue;
			
				//Normal Background Painting behind ActiveEntry
				BitBlt(bufferDC, (*eit)->x, (*eit)->y, (*eit)->width, (*eit)->height, backgroundDC, (*eit)->x, (*eit)->y, SRCCOPY);

				//Paint Entry Texture
				buttontexture->apply( bufferDC, (*eit)->x, (*eit)->y, (*eit)->width, (*eit)->height, state);
				//Paint Arrow
				if (arrowstate != 0)
				{
					buttontexture->apply( bufferDC, (*eit)->x, (*eit)->y, (*eit)->width, (*eit)->height, arrowstate);
					buttonfont->apply( bufferDC, (*eit)->x, (*eit)->y, (*eit)->width, (*eit)->height, (*eit)->caption, arrowstate );
				}
						
				//If No Info, Icon is valid
				buttonicon->apply( bufferDC, (*eit)->x, (*eit)->y, (*eit)->width, (*eit)->height, state, (*eit)->iconPath, (*eit)->hWnd, (*eit)->pidl, DefaultIcon, alphaMap );

				if (quicklaunchMode == 0)
					buttonfont->apply( bufferDC, (*eit)->x, (*eit)->y, (*eit)->width, (*eit)->height, (*eit)->caption, state );
			}
		}
	}

	// blt the double buffer to the display
	BitBlt(hDC, 0, 0, width, height, bufferDC, 0, 0, SRCCOPY);

	if(backgroundBitmap)
	{
		backgroundBitmap = (HBITMAP) SelectObject(backgroundDC, backgroundBitmap);
		DeleteObject(backgroundBitmap);
	}
	if (backgroundDC)
		DeleteDC(backgroundDC);	

	if(bufferBitmap)
	{
		bufferBitmap = (HBITMAP) SelectObject(bufferDC, bufferBitmap);
		DeleteObject(bufferBitmap);
	}
	if (bufferDC)
		DeleteDC(bufferDC);	
}

bool Popup::PopupActive()
{
	char tmpClass[MAX_PATH_LENGTH] = {0};
			
	POINT pt;
	GetCursorPos(&pt);
			
	GetClassName(WindowFromPoint(pt), tmpClass, MAX_PATH_LENGTH);

	if (stricmp(tmpClass, "xpopup") == NULL	|| stricmp(tmpClass, "xpopupxp") == NULL)
		return true;
			
	GetClassName(GetForegroundWindow(), tmpClass, MAX_PATH_LENGTH);

	if (stricmp(tmpClass, "xpopup") == NULL	|| stricmp(tmpClass, "xpopupxp") == NULL)
		return true;

	GetClassName(GetActiveWindow(), tmpClass, MAX_PATH_LENGTH);

	if (stricmp(tmpClass, "xpopup") == NULL	|| stricmp(tmpClass, "xpopupxp") == NULL)
		return true;

	GetClassName(GetFocus(), tmpClass, MAX_PATH_LENGTH);

	if (stricmp(tmpClass, "xpopup") == NULL	|| stricmp(tmpClass, "xpopupxp") == NULL)
		return true;

	return false;
}


void Popup::onTimer(int timerID)
{
	switch(timerID)
	{	
		case TIMER_MOUSETRACK:
		{
			POINT pt;
			GetCursorPos(&pt);

			if (oldmovetracker.x != pt.x || oldmovetracker.y != pt.y)
			{
				oldmovetracker = pt;
				RECT rc;
				GetWindowRect(hWnd, &rc);

				if(!PtInRect(&rc, pt) || !visible)
				{
					//To be on the save side
					ReleaseCapture();
					mousePressed = false;

					KillTimer(hWnd, TIMER_MOUSETRACK);

					mouseInside = false;
					lastkeyentry = -1;

					if ( hoverentry != NULL && hoverentry->entryType >= 0 )
					{
						hoverentry = NULL;
						repaint();
					}
					else if ( hoverentry != NULL )
					{
						for(PopupListIterator plit = subpopupList.begin(); plit != subpopupList.end(); plit++)
						{
							if (stricmp( hoverentry->caption.c_str(), (*plit)->captionname.c_str()) == NULL)
							{
								if ( (*plit)->isVisible() )
									return;
							}
						}
						hoverentry = NULL;
						repaint();
					}

					break;
				}
			}
		}
		break;

		case TIMER_ALPHAFADEIN:
		{
			if (UpdateAlpha <= alphaTransparency-alphaFadeStep)
			{
				UpdateAlpha += alphaFadeStep;
				UpdateWindowTransparent(hWnd, UpdateAlpha);
			}
			else
			{
				KillTimer(hWnd, TIMER_ALPHAFADEIN);
				if (UpdateAlpha != alphaTransparency)
					UpdateWindowTransparent(hWnd, alphaTransparency);
			}
		}
		break;

		case TIMER_ALPHAFADEOUT:
		{
			if (UpdateAlpha >= alphaFadeStep)
			{
				UpdateAlpha -= alphaFadeStep;
				UpdateWindowTransparent(hWnd, UpdateAlpha);		
			}
			else
			{
				KillTimer(hWnd, TIMER_ALPHAFADEOUT);
				UpdateAlpha = 0;
				UpdateWindowTransparent(hWnd, UpdateAlpha);
				ShowWindow(hWnd, SW_HIDE);

				lastactiveentry = NULL;
				hoverentry = NULL;
				lastkeyentry = -1;

				currentPart = 0;
				
				visible = false;
			}
		}
		break;

		case TIMER_UPDATE:
		{
			if (visible && hasInfoEntry && !isTaskPopup)
			{
				bool needRepaint = false;
				for (EntryListIterator eit = entrylist.begin(); eit != entrylist.end(); ++eit)
				{
					if ( !(*eit)->info.empty() )
					{
						string info = InfoUpdate( (*eit)->info );
						if (!needRepaint && stricmp( (*eit)->caption.c_str() , info.c_str() ) != NULL )
							needRepaint = true;
						(*eit)->caption = info;
					}
				}
				if (needRepaint)
					repaint(false, true);
			}
			else if (visible && isTaskPopup && PinnedMode)
			{
				KillTimer(hWnd, TIMER_UPDATE);
				ScanTasks();
			}

		}
		break;

		case TIMER_ACTIVE:
		{	
			if ( PopupActive() )
				break;

			if ( PinnedMode )
			{
				hoverentry = NULL;
				repaint();
			}
			
			hide();
		}
		break;

		case TIMER_HOVER:
		{
			KillTimer(hWnd, TIMER_HOVER);

			bool alreadyCreated = false;
			for(PopupListIterator plit = subpopupList.begin(); plit != subpopupList.end(); plit++)
			{
				if ( !(*plit)->PinnedMode && (*plit)->isVisible() )
				{
					(*plit)->hide();
					if ( (*plit)->pParentPopup != NULL)
						SetActiveWindow((*plit)->pParentPopup->gethWnd());
				}
			}
			for(plit = subpopupList.begin(); plit != subpopupList.end(); plit++)
			{
				if (stricmp( lastactiveentry->caption.c_str(), (*plit)->captionname.c_str()) == NULL)
				{
					(*plit)->show();

					if (quicklaunchMode != 0 && ToolTips)
					{
						CString worker = TrimAndStripQuote(lastactiveentry->action).c_str();
						worker = worker.Right(worker.GetLength()-worker.ReverseFind('\\')-1);
						if (stricmp(worker.Right(4), ".lnk") == NULL)
							worker = worker.Left(worker.GetLength()-4);
						string tooltip = worker;
						updateHint(hWnd, (char*) tooltip.c_str(), true);
					}

					alreadyCreated = true;		
				}
			}
			if (!alreadyCreated && lastactiveentry->clsid == 0 && lastactiveentry->hWnd == NULL)
				CreateSubMenu( ("sub_"+lastactiveentry->caption), lastactiveentry->action, lastactiveentry);
			else if (!alreadyCreated && lastactiveentry->clsid != 0)
				CreateShellSubMenu( ("sub_"+lastactiveentry->caption), lastactiveentry, lastactiveentry->clsid);
			else if (!alreadyCreated && lastactiveentry->hWnd == hWnd)
				CreateTaskSubMenu( ("sub_"+lastactiveentry->caption), lastactiveentry);
		}
		break;

		case TIMER_CHORDING:
		{
			KillTimer(hWnd, TIMER_CHORDING);
			inChordingTimeOut = false;
		}
		break;
	}
}

void Popup::onWindowPosChanging(WINDOWPOS *windowPos)
{
	if ( (windowPos->flags & SWP_NOMOVE) != 0 )
		return;

	int nDist = 10;
	int result = 0;

	// left edge
	if ( windowPos->x < customWorkArea.left+nDist && (windowPos->x > customWorkArea.left-nDist || restrictToWorkArea) )
	{
		windowPos->x = customWorkArea.left;
		result = 1;
	}

	// top edge
	if ( windowPos->y < customWorkArea.top+nDist && (windowPos->y > customWorkArea.top-nDist || restrictToWorkArea) )
	{
		windowPos->y = customWorkArea.top;
		result = 1;
	}

	// bottom edge
	if ( (windowPos->y+actualheight < customWorkArea.bottom+nDist || restrictToWorkArea) && windowPos->y+actualheight > customWorkArea.bottom-nDist)
	{
		windowPos->y = customWorkArea.bottom - actualheight;
		result = 1;
	}

	// right edge
	if ( (windowPos->x+actualwidth < customWorkArea.right+nDist || restrictToWorkArea) && windowPos->x+actualwidth > customWorkArea.right-nDist)
	{
		windowPos->x = customWorkArea.right - actualwidth;
		result = 1;
	}

	if ( result != 0 )
		return;
			
	POINT pos = {windowPos->x, windowPos->y};
	POINT size = {actualwidth, actualheight};

	for( PopupListIterator plit = visiblepopupList.begin(); plit != visiblepopupList.end(); plit++)
	{
		if ( (*plit) == this )
			continue;

		result = isCloseTo( pos, size, (*plit)->actualx, (*plit)->actualy, (*plit)->actualwidth, (*plit)->actualheight, nDist);
		
		if (result != 0)
			break;
	}

	if (result == 1)
	{
		windowPos->x = pos.x;
	}
	else if (result == 2)
	{
		windowPos->y = pos.y;
	}
	else if (result == 3)
	{
		windowPos->x = pos.x;
		windowPos->y = pos.y;
	}
}

void Popup::onMove(int x, int y)
{
	actualx = x;
	actualy = y;

	if ( !BangToCall.empty() )
	{
		CString help = BangToCall.c_str();
		help.Replace("!", "");

		PrefixedLSSetVariable((string)help, "CurrentX", IntToString( x ));
		PrefixedLSSetVariable((string)help, "CurrentY", IntToString( y ));
	}
}

void Popup::onSize(int width, int height)
{
	actualwidth = width;
	actualheight = height;

	if ( alphaMap )
	{
		HRGN windowRgn = CreateRectRgn(0, 0, actualwidth, actualheight);
		SetWindowRgn(hWnd, windowRgn, false);
		DeleteObject(windowRgn);
	}

	repaint(true);
	
	if ( !BangToCall.empty() )
	{
		CString help = BangToCall.c_str();
		help.Replace("!", "");

		PrefixedLSSetVariable((string)help, "CurrentWidth", IntToString( width ));
		PrefixedLSSetVariable((string)help, "CurrentHeight", IntToString( height ));
	}
}

BOOL Popup::onWindowMessage( UINT nMessage, WPARAM wParam, LPARAM lParam, LRESULT &lResult )
{
	lResult = 0;

	if ((0x8000 & GetKeyState(VK_CONTROL)) && (0x8000 & GetKeyState(VK_SHIFT)) )
	{
		if (HideAllowed && PinnedMode && (0x8000 & GetKeyState(VK_UP)))
		{
			PinnedMode = false;
			setAlwaysOnTop(true);

			return TRUE;
		}
		else if  (!PinnedMode && (0x8000 & GetKeyState(VK_DOWN)))
		{
			PinnedMode = true;
			if (pinnedPopupNotOnTop)
				setAlwaysOnTop(false);
			else 
				setAlwaysOnTop(true);
			SetTimer(hWnd, TIMER_ACTIVE, hideDelay, NULL);

			return TRUE;
		}
	}

	switch ( nMessage )
	{
		case LM_EXTERNALDRAWNOTIFY:
		{
			if ( visible )
				repaint();

			return true;
		}

		case WM_CLOSE:
		{
			lResult = 0;
			return true;
		}

		case WM_LBUTTONDBLCLK:
		{
			onLButtonDblClk((int) (short) LOWORD(lParam), (int) (short) HIWORD(lParam));
			return true;
		}

		case WM_LBUTTONDOWN:
		{
			ontemplateButtonDown(LOWORD(lParam), HIWORD(lParam), titleLClickAction);
			return true;
		}

		case WM_LBUTTONUP:
		{
			ontemplateButtonUp(LOWORD(lParam), HIWORD(lParam), titleLClickAction, 1);
			return true;
		}

		case WM_MBUTTONDOWN:
		{
			ontemplateButtonDown(LOWORD(lParam), HIWORD(lParam), titleMClickAction);
			return true;
		}

		case WM_MBUTTONUP:
		{
			ontemplateButtonUp(LOWORD(lParam), HIWORD(lParam), titleMClickAction, 3);
			return true;
		}

		case WM_RBUTTONDOWN:
		{
			ontemplateButtonDown(LOWORD(lParam), HIWORD(lParam), titleRClickAction);
			return true;
		}

		case WM_RBUTTONUP:
		{
			ontemplateButtonUp(LOWORD(lParam), HIWORD(lParam), titleRClickAction, 2);
			return true;
		}

		case WM_MOUSEWHEEL:
		{
			char tmpClass[MAX_PATH_LENGTH] = {0};
			
			POINT pt;
			GetCursorPos(&pt);
			HWND window = WindowFromPoint(pt);

			if ( window != hWnd )
			{
				SendMessage(window, nMessage, wParam, lParam);
				return true;
			}
			else
			{
				if ((short)(HIWORD(wParam)) < 0)
				{
					onWheelDown((int) (short) LOWORD(lParam), (int) (short) HIWORD(lParam));
					return true;
				}
				else
				{
					onWheelUp((int) (short) LOWORD(lParam), (int) (short) HIWORD(lParam));
					return true;
				}
			}
		}

		case WM_XBUTTONDOWN:
		{
			return true;
		}

		case WM_XBUTTONUP:
		{
			char tmpClass[MAX_PATH_LENGTH] = {0};
			
			POINT pt;
			GetCursorPos(&pt);
			HWND window = WindowFromPoint(pt);

			if ( window != hWnd )
			{
				SendMessage(window, nMessage, wParam, lParam);
				return true;
			}
			else
			{
				if (HIWORD(wParam) == XBUTTON1)
				{
					if (pParentPopup != NULL)
					{
						if (pParentPopup->lastactiveentry != NULL)
						{
							pParentPopup->hoverentry = pParentPopup->lastactiveentry;
							SetCursorPos(pParentPopup->actualx+pParentPopup->lastactiveentry->x+2, pParentPopup->actualy+pParentPopup->lastactiveentry->y+2);	
						}
						else
						{
							pParentPopup->hoverentry = pParentPopup->lastactiveentry;
							SetCursorPos(pParentPopup->actualx+pParentPopup->startentry->x+2, pParentPopup->actualy+pParentPopup->startentry->y+2);	
						}

						if ( !PinnedMode && visible )
							hide();

						pParentPopup->repaint();

						SetForegroundWindow(pParentPopup->gethWnd());
						SetFocus(pParentPopup->gethWnd());
					}
				}
				else if (HIWORD(wParam) == XBUTTON2)
				{
					for (EntryListIterator eit = entrylist.begin(); eit != entrylist.end(); ++eit)
					{
						if ( (*eit) == hoverentry && (*eit)->entryType < 0 )
						{
							lastactiveentry = (*eit);
							bool alreadyCreated = false;
							for(PopupListIterator plit = subpopupList.begin(); plit != subpopupList.end(); plit++)
							{
								if (stricmp( (*eit)->caption.c_str(), (*plit)->captionname.c_str()) == NULL)
								{
									(*plit)->hoverentry = (*plit)->startentry;
									(*plit)->show(true);
									alreadyCreated = true;
								}
								else
								{
									if ( !(*plit)->PinnedMode && (*plit)->isVisible() )
										(*plit)->hide();
								}
							}
							if (!alreadyCreated && (*eit)->clsid == 0 && (*eit)->hWnd == NULL)
								CreateSubMenu( ("sub_"+(*eit)->caption), (*eit)->action, (*eit), true);
							else if (!alreadyCreated && (*eit)->clsid != 0)
								CreateShellSubMenu( ("sub_"+(*eit)->caption), (*eit), (*eit)->clsid, true);
							else if (!alreadyCreated && (*eit)->hWnd == hWnd)
								CreateTaskSubMenu( ("sub_"+(*eit)->caption), (*eit), true );
						
							break;
						}
					}
				}
			}
			
			return true;
		}
	
		case WM_MOUSEMOVE:
		{
			onMouseMove( LOWORD( lParam ), HIWORD( lParam ) );
			return true;
		}
	
		if ( !alphaMap || !IsOS(OS_2KXP) )
		{
			case WM_PAINT:
			{
				PAINTSTRUCT ps;

				HDC hDC;

				hDC = BeginPaint( hWnd, &ps );

				onPaint( hDC );

				EndPaint( hWnd, &ps );
			
				return true;
			}
		}

		case WM_WINDOWPOSCHANGING:
		{
			if ( !manualMove || restrictToWorkArea )
				onWindowPosChanging((WINDOWPOS *) lParam);
			
			return true;
		}

		case WM_MOVE:
		{
			onMove((int) (short) LOWORD(lParam), (int) (short) HIWORD(lParam));
			return true;
		}

		case WM_SIZE:
		{
			onSize( (int) (short) LOWORD( lParam ), (int) (short) HIWORD( lParam ) );
			return true;
		}

		case WM_TIMER:
		{
			onTimer((int) wParam);
			return true;
		}
		
		case WM_KEYDOWN:	// catches arrow keys etc.
		case WM_CHAR:		// catches small letters and swedish keys..
		{
			char tmpClass[MAX_PATH_LENGTH] = {0};
			
			POINT pt;
			GetCursorPos(&pt);
			HWND window = WindowFromPoint(pt);

			GetClassName(window, tmpClass, MAX_PATH_LENGTH);

			if ( window != hWnd && (stricmp(tmpClass, "xpopup") == NULL || stricmp(tmpClass, "xpopupxp") == NULL) )
			{
				SendMessage(window, nMessage, wParam, lParam);
				return true;
			}
			else
			{
				onKey(wParam);
				return true;
			}
		}
		
		case WM_DROPFILES:
		{
			if(dropCommand.empty())	// we shouldn't get here... but somehow we did :p
				break;

			POINT pt;
			GetCursorPos(&pt);
			ScreenToClient(hWnd, &pt);
			for (EntryListIterator eit = entrylist.begin(); eit != entrylist.end(); ++eit)
			{
				if (currentPart != (*eit)->hiddenPart)
					continue;

				if ( pt.x < ((*eit)->x + (*eit)->width) && pt.x >= (*eit)->x )
				{
					if ( pt.y < ((*eit)->y + (*eit)->height) && pt.y >= (*eit)->y )
					{
						hoverentry = *eit;
						break;
					}
				}
			}

			if (hoverentry == NULL || hoverentry->entryType >= 0 || hoverentry->clsid != 0)
				break;

			int intFileAction = 0;
			char szTarget[MAX_PATH_LENGTH] = {0};

			strcpy(szTarget, TrimAndStripQuote(hoverentry->action).c_str());

			if (stricmp(dropCommand.c_str(), "move") == 0)
				intFileAction = 1;
			else if (stricmp(dropCommand.c_str(), "copy") == 0)
				intFileAction = 2;
			else if (stricmp(dropCommand.c_str(), "shortcut") == 0)
				intFileAction = 3;

			string check = szTarget;
			if (check.substr(1, check.length()-1) == "\\")
				check.append( "*" );
			else
				check.append( "\\*" );
				
			//Test Path for Existence
			WIN32_FIND_DATA FindData;
			HANDLE hFindHandle = FindFirstFile(check.c_str(), &FindData);

			if (intFileAction > 0 && intFileAction < 3)
			{
				char *szSourceFiles = NULL;
				char buffer[MAX_PATH_LENGTH];
				int numFiles;
				UINT len,scursor = 0;

				numFiles = DragQueryFile((HDROP) wParam, 0xFFFFFFFF, NULL, NULL);

				szSourceFiles = new char[MAX_PATH_LENGTH*numFiles+1];
				scursor = 0;

				for (int i=0;i<numFiles;i++)
				{
					len = DragQueryFile((HDROP) wParam, i, buffer, MAX_PATH_LENGTH);
					
					strcpy(szSourceFiles+scursor,buffer);
					scursor += len + 1;
				}
					
				DragFinish((HDROP) wParam);

				if( hFindHandle != INVALID_HANDLE_VALUE || intFileAction == 0 )
				{
					szSourceFiles[scursor] = 0;
					strcpy(buffer, szTarget);
					buffer[strlen(buffer)+1] = 0;
					
					SHFILEOPSTRUCT shfop;
					shfop.hwnd = NULL;
					if (intFileAction == 1)
						shfop.wFunc = FO_MOVE;
					else if (intFileAction == 2)
						shfop.wFunc = FO_COPY;
					shfop.pFrom = szSourceFiles;
					shfop.pTo = buffer;
					shfop.fFlags = FOF_NOCONFIRMMKDIR | FOF_ALLOWUNDO;
					SHFileOperation(&shfop);
				}
				
				delete[] szSourceFiles;
			}
			else if (hFindHandle != INVALID_HANDLE_VALUE && intFileAction == 3)
			{
				CString path = szTarget;
				if (path.Right(1) != "\\")
					path += "\\";
				int numDropped;
				char szFile[MAX_PATH_LENGTH];
				
				numDropped = DragQueryFile((HDROP) wParam, 0xFFFFFFFF, NULL, 0);
				for (int i = 0; i < numDropped; i++)
				{
					DragQueryFile((HDROP) wParam, i, (char *)&szFile, MAX_PATH_LENGTH);

					HRESULT hres; 
					IShellLink* psl; 

					hres = CoCreateInstance(CLSID_ShellLink, NULL,
					CLSCTX_INPROC_SERVER, IID_IShellLink, (LPVOID *) &psl); 
					if (SUCCEEDED(hres))
					{
						IPersistFile* ppf; 
						psl->SetPath(szFile); 
						psl->SetDescription("");
 
						hres = psl->QueryInterface(IID_IPersistFile, (LPVOID*)&ppf); 
						if (SUCCEEDED(hres))
						{
							CString work = szFile;
							work = work.Right(work.GetLength()-work.ReverseFind('\\')-1);
							work = work.Left(work.Find("."));
							work += ".lnk";

							WCHAR wsz[MAX_PATH]; 
							MultiByteToWideChar(CP_ACP, 0, (path + work), -1, wsz, MAX_PATH_LENGTH);  
							hres = ppf->Save(wsz, TRUE); 
							ppf->Release(); 
						}
						psl->Release(); 
					}
				}
				DragFinish((HDROP) wParam);	
			}
			FindClose(hFindHandle);
			
			return 0;
		}
	}

	return false;
}

LRESULT WINAPI Popup::windowProcedure(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	Popup *popup = (Popup *) GetWindowLong(hWnd, 0);

	if (message == WM_NCCREATE)
	{
		LPCREATESTRUCT pcs = (LPCREATESTRUCT) lParam;
		popup = (Popup *) pcs->lpCreateParams;

		popup->hWnd = hWnd;
		SetWindowLong(hWnd, 0, (LONG) popup);
	}

	if (popup)
	{
		LRESULT lResult;

		if (popup->onWindowMessage(message, wParam, lParam, lResult))
			return lResult;
	}

	return DefWindowProc(hWnd, message, wParam, lParam);
}

void Popup::ActionAtPoint( int xP, int yP, bool execute, bool context)
{
	for (EntryListIterator eit = entrylist.begin(); eit != entrylist.end(); ++eit)
	{
		if (currentPart != (*eit)->hiddenPart)
			continue;

		if ( xP < ((*eit)->x + (*eit)->width) && xP >= (*eit)->x )
		{
			if ( yP < ((*eit)->y + (*eit)->height) && yP >= (*eit)->y )
			{
				//-------------------------------------------------------------------
				//LEFTCLICK NORMAL EXECUTE AND FOLDEROPENONCLICK (IF ACTIVE)
				//-------------------------------------------------------------------
				if ( execute && !context )
				{
					if ( (*eit)->entryType >= 0 && !(*eit)->action.empty() && (*eit)->pidl == NULL && (*eit)->hWnd == NULL)
					{
						LSExecute(NULL, (*eit)->action.c_str(), NULL);
						
						Popup *tmpPointer = pParentPopup;
						if (tmpPointer != NULL)
						{
							while (tmpPointer->pParentPopup != NULL)
								tmpPointer = tmpPointer->pParentPopup;

							if ( !execSound.empty() )
							{
								tmpPointer->execSoundPlayed = true;
								ParseBangCommand(hWnd, "!playsound", execSound.c_str());
							}
							tmpPointer->hide(true);
						}
						else
						{	
							if ( !execSound.empty() )
							{
								execSoundPlayed = true;
								ParseBangCommand(hWnd, "!playsound", execSound.c_str());
							}
							hide(true);
						}
					}
					else if ( (*eit)->entryType >= 0 && (*eit)->action.empty() && (*eit)->pidl != NULL && (*eit)->hWnd == NULL)
					{
						Invoke( (*eit)->pidl );

						Popup *tmpPointer = pParentPopup;
						if (tmpPointer != NULL)
						{
							while (tmpPointer->pParentPopup != NULL)
								tmpPointer = tmpPointer->pParentPopup;

							if ( !execSound.empty() )
							{
								tmpPointer->execSoundPlayed = true;
								ParseBangCommand(hWnd, "!playsound", execSound.c_str());
							}
							tmpPointer->hide(true);
						}
						else
						{	
							if ( !execSound.empty() )
							{
								execSoundPlayed = true;
								ParseBangCommand(hWnd, "!playsound", execSound.c_str());
							}
							hide(true);
						}
					}
					else if ( (*eit)->entryType >= 0 && (*eit)->action.empty() && (*eit)->pidl == NULL && (*eit)->hWnd != NULL )
					{
						SwitchToThisWindow( (*eit)->hWnd, SW_RESTORE );

						Popup *tmpPointer = pParentPopup;
						if (tmpPointer != NULL)
						{
							while (tmpPointer->pParentPopup != NULL)
								tmpPointer = tmpPointer->pParentPopup;

							if ( !execSound.empty() )
							{
								tmpPointer->execSoundPlayed = true;
								ParseBangCommand(hWnd, "!playsound", execSound.c_str());
							}
							tmpPointer->hide(true);
						}
						else
						{
							if ( !execSound.empty() )
							{
								execSoundPlayed = true;
								ParseBangCommand(hWnd, "!playsound", execSound.c_str());
							}
							hide(true);
						}
					}
					//Jump Cursor into Folder
					else if ( (*eit)->entryType < 0 )
					{
						bool alreadyCreated = false;
						for(PopupListIterator plit = subpopupList.begin(); plit != subpopupList.end(); plit++)
						{
							if (stricmp( (*eit)->caption.c_str(), (*plit)->captionname.c_str()) == NULL)
							{
								if ( (0x8000 & GetKeyState(VK_CONTROL)) )
								{
									if ( !(*eit)->action.empty() )
									{
										if ((*eit)->pipedList.size() == 1)
											LSExecute(NULL, ("explorer.exe /e," + (*eit)->action).c_str(), SW_SHOWNORMAL);
										else
										{
											HMENU GroupMenu = CreatePopupMenu();

											int counter = 1;
											for(StringListIterator sit = (*eit)->pipedList.begin(); sit != (*eit)->pipedList.end(); ++sit)
											{
												AppendMenu( GroupMenu, MF_STRING, counter, (*sit).c_str() );
												counter++;
											}

											// display the menu
											POINT pt;
											GetCursorPos( &pt );

											int command = TrackPopupMenu( GroupMenu,
																		  TPM_RETURNCMD | TPM_RIGHTBUTTON,
																		  pt.x, pt.y, 0,
																		  this->gethWnd(),
																		  NULL );

											if ( command )
											{
												StringListIterator sit = (*eit)->pipedList.begin();
												counter = 1; 
												while (counter < command)
												{
													sit++;
													counter++;
												}

												if ( counter <= (*eit)->pipedList.size() )
												{
													LSExecute(NULL, ("explorer.exe /e," + (*sit)).c_str(), SW_SHOWNORMAL);
												}
											}
											
											if (GroupMenu)
												DestroyMenu(GroupMenu);
										}
									}
								}
								else
									(*plit)->show();
								alreadyCreated = true;
							}
							else
							{
								if ( !(*plit)->PinnedMode && (*plit)->isVisible() )
									(*plit)->hide();
							}
						}
						if (!alreadyCreated && (*eit)->clsid == 0 && (*eit)->hWnd == NULL)
							CreateSubMenu( ("sub_"+(*eit)->caption), (*eit)->action, (*eit));
						else if (!alreadyCreated && (*eit)->clsid != 0)
							CreateShellSubMenu( ("sub_"+(*eit)->caption), (*eit), (*eit)->clsid);
						else if (!alreadyCreated && (*eit)->hWnd == hWnd)
							CreateTaskSubMenu( ("sub_"+(*eit)->caption), (*eit));

						hoverentry = *eit;
					}
				}
				//-------------------------------------------------------------------
				//RIGHTCLICK CONTEXT EXECUTE
				//-------------------------------------------------------------------
				//Normal Context Menu
				else if (!execute && context && !(*eit)->action.empty() && (*eit)->pidl == NULL && (*eit)->hWnd == NULL)
				{
					CString easyWorker = TrimAndStripQuote((*eit)->action).c_str();

					if ( (((*eit)->pipedList.size() == 1 && (*eit)->entryType < 0) || ((*eit)->pipedList.size() <= 1 && (*eit)->entryType >= 0)) && easyWorker.Left(1) != "!" )
						ContextMenu(easyWorker);
					else if ( easyWorker.Left(1) != "!" )
					{
						HMENU GroupMenu = CreatePopupMenu();

						int counter = 1;
						for(StringListIterator sit = (*eit)->pipedList.begin(); sit != (*eit)->pipedList.end(); ++sit)
						{
							AppendMenu( GroupMenu, MF_STRING, counter, (*sit).c_str() );
							counter++;
						}

						// display the menu
						POINT pt;
						GetCursorPos( &pt );

						int command = TrackPopupMenu( GroupMenu,
													  TPM_RETURNCMD | TPM_RIGHTBUTTON,
													  pt.x, pt.y, 0,
													  this->gethWnd(),
													  NULL );

						if ( command )
						{
							StringListIterator sit = (*eit)->pipedList.begin();
							counter = 1; 
							while (counter < command)
							{
								sit++;
								counter++;
							}

							if ( counter <= (*eit)->pipedList.size() )
							{
								easyWorker = TrimAndStripQuote((*sit).c_str()).c_str();
								ContextMenu(easyWorker);
							}
						}
						
						if (GroupMenu)
							DestroyMenu(GroupMenu);
					}
				}
				//ShellFolder Context Menu
				else if (!execute && context && (*eit)->action.empty() && (*eit)->pidl != NULL && (*eit)->hWnd == NULL)
					RInvoke((*eit)->pidl);
				//Tasks ContextMenu
				else if (!execute && context && (*eit)->action.empty() && (*eit)->pidl == NULL && (*eit)->hWnd != NULL )
				{
					HMENU systemMenu;

					systemMenu = GetSystemMenu( (*eit)->hWnd, FALSE );

					WINDOWPLACEMENT wp;
					wp.length = sizeof(WINDOWPLACEMENT);
					GetWindowPlacement( (*eit)->hWnd, &wp );

					// restore is enabled only if minimized or maximized (not normal)
					EnableMenuItem( systemMenu, SC_RESTORE, MF_BYCOMMAND | (wp.showCmd != SW_SHOWNORMAL ? MF_ENABLED : MF_GRAYED) );

					// move is enabled only if normal
					EnableMenuItem( systemMenu, SC_MOVE, MF_BYCOMMAND | (wp.showCmd == SW_SHOWNORMAL ? MF_ENABLED : MF_GRAYED) );

					// size is enabled only if normal
					EnableMenuItem( systemMenu, SC_SIZE, MF_BYCOMMAND | (wp.showCmd == SW_SHOWNORMAL ? MF_ENABLED : MF_GRAYED) );

					// minimize is enabled only if not minimized
					EnableMenuItem( systemMenu, SC_MOVE, MF_BYCOMMAND | (wp.showCmd != SW_SHOWMINIMIZED ? MF_ENABLED : MF_GRAYED) );

					// maximize is enabled only if not maximized
					EnableMenuItem( systemMenu, SC_MOVE, MF_BYCOMMAND | (wp.showCmd != SW_SHOWMAXIMIZED ? MF_ENABLED : MF_GRAYED) );

					// let application modify menu
					PostMessage( (*eit)->hWnd, WM_INITMENUPOPUP, (WPARAM) systemMenu, MAKELPARAM( 0, TRUE ) );
					PostMessage( (*eit)->hWnd, WM_INITMENU, (WPARAM) systemMenu, 0 );

					// display the menu
					POINT pt;
					GetCursorPos( &pt );

					int command = TrackPopupMenu( systemMenu,
												  TPM_RETURNCMD | TPM_RIGHTBUTTON,
												  pt.x, pt.y, 0,
												  hWnd,
												  NULL );

					if ( command )
					{
						if ( command != SC_MINIMIZE && command != SC_CLOSE )
							SwitchToThisWindow((*eit)->hWnd, SW_RESTORE);

						PostMessage( (*eit)->hWnd, WM_SYSCOMMAND, (WPARAM) command, MAKELPARAM( pt.x, pt.y ) );
					}
				}
	
				break;
				//-------------------------------------------------------------------
			}
		}
	}
}

void Popup::HoverActions( int xP, int yP )
{
	if (hoverentry != NULL)
	{
		if ( xP < (hoverentry->x + hoverentry->width) && xP >= hoverentry->x 
				&& yP < (hoverentry->y + hoverentry->height) && yP >= hoverentry->y 
				&& hoverentry->hiddenPart == currentPart )
		{
			return;
		}
	}

	bool quickskip = false;
	for (EntryListIterator eit = entrylist.begin(); eit != entrylist.end(); ++eit)
	{
		if (currentPart != (*eit)->hiddenPart)
			continue;

		if ( xP < ((*eit)->x + (*eit)->width) && xP >= (*eit)->x && !quickskip )
		{
			if ( yP < ((*eit)->y + (*eit)->height) && yP >= (*eit)->y )
			{
				//Separator and InfoEntry cannot be "Active" (Skip)
				if ( (*eit)->entryType == 1 || (*eit)->entryType == 2 )
					continue;

				if (quicklaunchMode != 0 && ToolTips)
				{
					CString worker = TrimAndStripQuote((*eit)->action).c_str();
					worker = worker.Right(worker.GetLength()-worker.ReverseFind('\\')-1);
					if (stricmp(worker.Right(4), ".lnk") == NULL)
						worker = worker.Left(worker.GetLength()-4);
					string tooltip = worker;
					updateHint(hWnd, (char*) tooltip.c_str(), true);
				}
				
				if ( !subSound.empty() )
					ParseBangCommand(hWnd, "!playsound", subSound.c_str());
				
				hoverentry = (*eit);

				if ( (*eit)->entryType >= 0 && !(*eit)->action.empty() )
				{
					KillTimer(hWnd, TIMER_HOVER);

					for(PopupListIterator plit = subpopupList.begin(); plit != subpopupList.end(); plit++)
					{
						if ( !(*plit)->PinnedMode && (*plit)->isVisible() )
							(*plit)->hide();
					}
				}
				else if ( (*eit)->entryType < 0 && !folderOpenOnClick )
				{
					//Hack to avoid AutoOpen on Manual Open (Keyboard, Click)
					if ( xP < (*eit)->x+1 || xP > (*eit)->x+3 || yP < (*eit)->y+1 || yP > (*eit)->y+3 )
					{
						KillTimer(hWnd, TIMER_HOVER);

						lastactiveentry = (*eit);

						SetTimer(hWnd, TIMER_HOVER, openDelay, NULL);
					}
					else
						hoverentry = NULL;
				}

				quickskip = true;

				break;
				//-------------------------------------------------------------------
			}
		}
	}

	if (!quickskip && hoverentry->entryType >= 0)
		hoverentry = NULL;
	
	repaint();
}

int Popup::isCloseTo(POINT &pos, const POINT size, const int x, const int y, const int cx, const int cy, const int span)
{
	bool internalX = false;
	bool internalY = false;

	if ((pos.x > x-span) && (pos.x < x+span))
	{
		if ( ((pos.y > y-span) && (pos.y < y+span)) 
			|| ((pos.y+size.y > y-span) && (pos.y+size.y < y+span)) 
			|| ((pos.y > y+cy-span ) && (pos.y < y+cy+span)) 
			|| ((pos.y+size.y > y+cy-span ) && (pos.y+size.y < y+cy+span)) )
		{
			pos.x = x;
			internalX = true;
		}
		else if ( (pos.y > y) && (pos.y < y+cy) 
				&& (pos.y+size.y > y) && (pos.y+size.y < y+cy) )
		{
			pos.x = x;
			internalX = true;
		}
	}
	else if ((pos.x > x+cx-span ) && (pos.x < x+cx+span))
	{
		if ( ((pos.y > y-span) && (pos.y < y+span)) 
			|| ((pos.y+size.y > y-span) && (pos.y+size.y < y+span)) 
			|| ((pos.y > y+cy-span ) && (pos.y < y+cy+span)) 
			|| ((pos.y+size.y > y+cy-span ) && (pos.y+size.y < y+cy+span)) )
		{
			pos.x = x+cx;
			internalX = true;
		}
		else if ( (pos.y > y) && (pos.y < y+cy) 
				&& (pos.y+size.y > y) && (pos.y+size.y < y+cy) )
		{
			pos.x = x+cx;
			internalX = true;
		}
	}
	else if ((pos.x+size.x > x-span ) && (pos.x+size.x < x+span))
	{
		if ( ((pos.y > y-span) && (pos.y < y+span)) 
			|| ((pos.y+size.y > y-span) && (pos.y+size.y < y+span)) 
			|| ((pos.y > y+cy-span ) && (pos.y < y+cy+span)) 
			|| ((pos.y+size.y > y+cy-span ) && (pos.y+size.y < y+cy+span)) )
		{
			pos.x = x-size.x;
			internalX = true;
		}
		else if ( (pos.y > y) && (pos.y < y+cy) 
				&& (pos.y+size.y > y) && (pos.y+size.y < y+cy) )
		{
			pos.x = x-size.x;
			internalX = true;
		}
	}
	else if ((pos.x+size.x > x+cx-span ) && (pos.x+size.x < x+cx+span))
	{
		if ( ((pos.y > y-span) && (pos.y < y+span)) 
			|| ((pos.y+size.y > y-span) && (pos.y+size.y < y+span)) 
			|| ((pos.y > y+cy-span ) && (pos.y < y+cy+span)) 
			|| ((pos.y+size.y > y+cy-span ) && (pos.y+size.y < y+cy+span)) )
		{
			pos.x = x+cx-size.x;
			internalX = true;
		}
		else if ( (pos.y > y) && (pos.y < y+cy) 
				&& (pos.y+size.y > y) && (pos.y+size.y < y+cy) )
		{
			pos.x = x+cx-size.x;
			internalX = true;
		}
	}

	if ((pos.y > y-span) && (pos.y < y+span))
	{
		if ( ((pos.x > x-span) && (pos.x < x+span)) 
			|| ((pos.x+size.x > x-span) && (pos.x+size.x < x+span)) 
			|| ((pos.x > x+cx-span ) && (pos.x < x+cx+span)) 
			|| ((pos.x+size.x > x+cx-span ) && (pos.x+size.x < x+cx+span)) )
		{
			pos.y = y;
			internalY = true;
		}
		else if ( (pos.x > x) && (pos.x < x+cx) 
				&& (pos.x+size.x > x) && (pos.x+size.x < x+cx) )
		{
			pos.y = y;
			internalY = true;
		}
	}
	else if ((pos.y > y+cy-span ) && (pos.y < y+cy+span))
	{
		if ( ((pos.x > x-span) && (pos.x < x+span)) 
			|| ((pos.x+size.x > x-span) && (pos.x+size.x < x+span)) 
			|| ((pos.x > x+cx-span ) && (pos.x < x+cx+span)) 
			|| ((pos.x+size.x > x+cx-span ) && (pos.x+size.x < x+cx+span)) )
		{
			pos.y = y+cy;
			internalY = true;
		}
		else if ( (pos.x > x) && (pos.x < x+cx) 
				&& (pos.x+size.x > x) && (pos.x+size.x < x+cx) )
		{
			pos.y = y+cy;
			internalY = true;
		}
	}
	else if ((pos.y+size.y > y-span ) && (pos.y+size.y < y+span))
	{
		if ( ((pos.x > x-span) && (pos.x < x+span)) 
			|| ((pos.x+size.x > x-span) && (pos.x+size.x < x+span)) 
			|| ((pos.x > x+cx-span ) && (pos.x < x+cx+span)) 
			|| ((pos.x+size.x > x+cx-span ) && (pos.x+size.x < x+cx+span)) )
		{
			pos.y = y-size.y;
			internalY = true;
		}
		else if ( (pos.x > x) && (pos.x < x+cx) 
				&& (pos.x+size.x > x) && (pos.x+size.x < x+cx) )
		{
			pos.y = y-size.y;
			internalY = true;
		}
	}
	else if ((pos.y+size.y > y+cy-span ) && (pos.y+size.y < y+cy+span))
	{
		if ( ((pos.x > x-span) && (pos.x < x+span)) 
			|| ((pos.x+size.x > x-span) && (pos.x+size.x < x+span)) 
			|| ((pos.x > x+cx-span ) && (pos.x < x+cx+span)) 
			|| ((pos.x+size.x > x+cx-span ) && (pos.x+size.x < x+cx+span)) )
		{
			pos.y = y+cy-size.y;
			internalY = true;
		}
		else if ( (pos.x > x) && (pos.x < x+cx) 
				&& (pos.x+size.x > x) && (pos.x+size.x < x+cx) )
		{
			pos.y = y+cy-size.y;
			internalY = true;
		}
	}

	if (!internalX && !internalY)
		return 0;
	else if (internalX && !internalY)
		return 1;
	else if (!internalX && internalY)
		return 2;
	else
		return 3;
}

void Popup::Sort(vector<string> &inputList)
{
	std::sort( inputList.begin(), inputList.end(), Compare );
}

void Popup::SortTasks(hWndTaskList &inputList)
{
	std::sort( inputList.begin(), inputList.end(), CompareTasks );
}

void Popup::SortShellFolder(vector<LPITEMIDLIST> &inputList)
{
	std::sort( inputList.begin(), inputList.end(), CompareShellFolders );
}

string Popup::InfoUpdate(string infocfg)
{
	if ( strstr(infocfg.c_str(), "[") != NULL && strstr(infocfg.c_str(), "]") != NULL )
	{
		if ( infoExport == 21 )
			infoExport = 1;
		else 
			infoExport++;

		string what = ("specialinfoxpopup" + IntToString( infoExport ));

		ParseBangCommand(hWnd, "!labelinfoexport", ("%#" + what + "%# " + infocfg).c_str());

		return GetxRCString(what.c_str(), "", "");
	}
	else
		return TrimAndStripQuote(infocfg);
}

bool Popup::SelectNextEntry(bool upwards, bool noCursorMove)
{
	bool truerepaint = false;
	bool activeOneFound = false;
	for (EntryListIterator eit = entrylist.begin(); eit != entrylist.end(); ++eit)
	{
		if ( (*eit) == hoverentry )
		{
			if (upwards)
			{
				if (eit != entrylist.begin())
					eit--;
				else
				{
					eit = entrylist.end();
					eit--;
				}
			}
			else
			{
				if (eit != entrylist.end()-1)
					eit++;
				else
					eit = entrylist.begin();
			}

			//Separator and InfoEntry cannot be "Active" (Skip)
			while ((*eit)->entryType == 1 || (*eit)->entryType == 2)
			{
				if (upwards)
				{
					if (eit != entrylist.begin())
						eit--;
					else
					{
						eit = entrylist.end();
						eit--;
					}
				}
				else
				{
					if (eit != entrylist.end()-1)
						eit++;
					else
						eit = entrylist.begin();
				}
			}

			hoverentry = *eit;

			if (!noCursorMove)
				SetCursorPos(actualx+(*eit)->x+2, actualy+(*eit)->y+2);	

			if ( currentPart != (*eit)->hiddenPart)
			{
				currentPart--;
				if (currentPart < 0)
					currentPart = maxPart;

				truerepaint = true;
			}

			activeOneFound = true;

			break;
		}
	}

	if (!activeOneFound)
	{
		//Separator and InfoEntry cannot be "Active" (Skip)
		while (startentry->entryType == 1 || startentry->entryType == 2)
			startentry++;

		hoverentry = startentry;

		if (!noCursorMove)
			SetCursorPos(actualx+startentry->x+2, actualy+startentry->y+2);	

		if ( currentPart != startentry->hiddenPart)
		{
			currentPart++;
			if (currentPart > maxPart)
				currentPart = 0;

			truerepaint = true;
		}
	}

	return truerepaint;
}

//-----------------------------------------------------------------------
// ShellFolder Support
//-----------------------------------------------------------------------

//Normal Action
void Popup::Invoke(LPITEMIDLIST m_pidlFull)
{
	SHELLEXECUTEINFO sei;

	sei.cbSize = sizeof(SHELLEXECUTEINFO);
	sei.fMask = SEE_MASK_INVOKEIDLIST | SEE_MASK_IDLIST | SEE_MASK_FLAG_NO_UI;
	sei.hwnd = NULL;
	sei.lpVerb = NULL;
	sei.lpFile = NULL;
	sei.lpParameters = NULL;
	sei.lpDirectory = NULL;
	sei.nShow = SW_SHOW;
	sei.lpIDList = m_pidlFull;

	ShellExecuteEx(&sei);
}

//Context Menu
void Popup::RInvoke(LPITEMIDLIST m_pidlFull)
{
	LPMALLOC pMalloc;
	LPSHELLFOLDER psfFolder, psfNextFolder;
	LPITEMIDLIST pidlItem, pidlNextItem, *ppidl;
	LPCONTEXTMENU pContextMenu;
	CMINVOKECOMMANDINFO ici;
	UINT nCount, nCmd;
	HMENU hMenu;

	// Get current cursor position & window
	POINT point;
	HWND hwnd;

	GetCursorPos(&point);
	hwnd = WindowFromPoint(point);

	//
	// Get pointers to the shell's IMalloc interface and the desktop's
	// IShellFolder interface.
	//

	if (SUCCEEDED (SHGetMalloc (&pMalloc)))
	{

		if (!SUCCEEDED (SHGetDesktopFolder (&psfFolder)))
		{
			pMalloc->Release();
		}
		else
		{
			//
			// Convert the path name into a pointer to an item ID list (pidl).
			//
			if (m_pidlFull != NULL)
			{

				if (nCount = GetItemCount (m_pidlFull))
				{ // nCount must be > 0
					//
					// Initialize psfFolder with a pointer to the IShellFolder
					// interface of the folder that contains the item whose context
					// menu we're after, and initialize pidlItem with a pointer to
					// the item's item ID. If nCount > 1, this requires us to walk
					// the list of item IDs stored in m_pidlFull and bind to each
					// subfolder referenced in the list.
					//
					pidlItem = m_pidlFull;

					while (--nCount)
					{
						//
						// Create a 1-item item ID list for the next item in m_pidlFull.
						//

						pidlNextItem = DuplicateItem (pMalloc, pidlItem);

						if (pidlNextItem == NULL)
						{
							psfFolder->Release();
							pMalloc->Release();
							return ;
						}

						//
						// Bind to the folder specified in the new item ID list.
						//
						if (!SUCCEEDED (psfFolder->BindToObject (pidlNextItem, NULL, IID_IShellFolder, (void**)&psfNextFolder)))
						{
							pMalloc->Free(pidlNextItem);
							psfFolder->Release();
							pMalloc->Release();
							return ;
						}

						//
						// Release the IShellFolder pointer to the parent folder
						// and set psfFolder equal to the IShellFolder pointer for
						// the current folder.
						//
						psfFolder->Release();
						psfFolder = psfNextFolder;

						//
						// Release the storage for the 1-item item ID list we created
						// just a moment ago and initialize pidlItem so that it points
						// to the next item in m_pidlFull.
						//
						pMalloc->Free(pidlNextItem);
						pidlItem = GetNextItem (pidlItem);
					}

					//
					// Get a pointer to the item's IContextMenu interface and call
					// IContextMenu::QueryContextMenu to initialize a context menu.
					//
					ppidl = &pidlItem;
					if (SUCCEEDED (psfFolder->GetUIObjectOf (hwnd, 1, (const ITEMIDLIST**)ppidl, (const GUID)IID_IContextMenu, NULL, (void**)&pContextMenu)))
					{

						hMenu = CreatePopupMenu ();
						if (SUCCEEDED (pContextMenu->QueryContextMenu (
						                   hMenu, 0, 1, 0x7FFF, CMF_NORMAL)))
						{

							//
							// Display the context menu.
							//
							nCmd = TrackPopupMenu (hMenu, TPM_LEFTALIGN |
							                       TPM_LEFTBUTTON | TPM_RIGHTBUTTON | TPM_RETURNCMD,
							                       point.x, point.y, 0, hwnd, NULL);

							//
							// If a command was selected from the menu, execute it.
							//
							if (nCmd)
							{
								ici.cbSize = sizeof (CMINVOKECOMMANDINFO);
								ici.fMask = SEE_MASK_FLAG_NO_UI;
								ici.hwnd = hwnd;
								ici.lpVerb = MAKEINTRESOURCE (nCmd - 1);
								ici.lpParameters = NULL;
								ici.lpDirectory = NULL;
								ici.nShow = SW_SHOWNORMAL;
								ici.dwHotKey = 0;
								ici.hIcon = NULL;

								pContextMenu->InvokeCommand(&ici);


							}
							DestroyMenu (hMenu);
							pContextMenu->Release();
						}
					}
				}
			}
			//
			// Clean up and return.
			//

			psfFolder->Release();
			pMalloc->Release();
		}
	}
}

//Helpers
LPITEMIDLIST CreateIDList(int cb)
{
	IMalloc *pMalloc;
	LPITEMIDLIST pidl = NULL;

	if (cb <= 0)
		return NULL;

	if (SUCCEEDED(SHGetMalloc(&pMalloc)))
	{
		pidl = (LPITEMIDLIST) pMalloc->Alloc(cb);
		pMalloc->Release();
	}

	if (pidl)
		memset(pidl, 0, cb);

	return pidl;
}

LPITEMIDLIST DuplicateIDList(LPCITEMIDLIST pidl)
{
	LPITEMIDLIST pidlNew;
	int cb = GetIDListSize(pidl);

	if (!pidl)
		return NULL;

	pidlNew = CreateIDList(cb);

	if (pidlNew)
		memcpy(pidlNew, pidl, cb);

	return pidlNew;
}

LPITEMIDLIST JoinIDLists(LPCITEMIDLIST pidlA, LPCITEMIDLIST pidlB)
{
	LPITEMIDLIST pidl;
	int cbA, cbB;

	cbA = pidlA ? GetIDListSize(pidlA) - sizeof(pidlA->mkid.cb) : 0;
	cbB = GetIDListSize(pidlB);

	pidl = CreateIDList(cbA + cbB);

	if (pidl)
	{
		if (pidlA)
			memcpy(pidl, pidlA, cbA);

		memcpy((LPBYTE) pidl + cbA, pidlB, cbB);
	}

	return pidl;
}

int GetIDListSize(LPCITEMIDLIST pidl)
{
	if (!pidl)
		return 0;

	int cb = sizeof(pidl->mkid.cb);

	while (pidl->mkid.cb)
	{
		cb = cb + pidl->mkid.cb;
		pidl = NextID(pidl);
	}

	return cb;
}

LPCITEMIDLIST NextID(LPCITEMIDLIST pidl)
{
	return (LPCITEMIDLIST) ((CONST BYTE *) pidl + pidl->mkid.cb);
}

void FreeIDList(LPITEMIDLIST pidl)
{
	IMalloc *pMalloc;

	if (SUCCEEDED(SHGetMalloc(&pMalloc)))
	{
		pMalloc->Free(pidl);
		pMalloc->Release();
	}
}

// Tooltips for Quicklaunch Mode
void Popup::updateHint(HWND hWnd, LPSTR caption, bool updatesize, bool remove)
{
	if (ToolTips == 0)
		return;	
	
	if (!remove)
	{
		TOOLINFO ti;
		RECT clientRect;

		GetClientRect(hWnd, &clientRect);
		ti.cbSize = sizeof(TOOLINFO);
		ti.uFlags = TTF_SUBCLASS;
		ti.hwnd = hWnd;
		ti.uId = 0;
		ti.rect = clientRect;
		ti.hinst = hInstance;
		ti.lpszText = caption;
		ti.lParam = 0;

		SendMessage(ToolTips, TTM_UPDATETIPTEXT, 0, LPARAM(&ti));

		if (updatesize)
			SendMessage(ToolTips, TTM_NEWTOOLRECT, 0, LPARAM(&ti));
	}
	else
	{
		TOOLINFO ti;
		RECT emptyRect = {0, 0, 0, 0};

		ti.cbSize = sizeof(TOOLINFO);
		ti.uFlags = 0;
		ti.hwnd = hWnd;
		ti.uId = 0;
		ti.rect = emptyRect;
		ti.hinst = hInstance;
		ti.lpszText = NULL;
		ti.lParam = 0;

		SendMessage(ToolTips, TTM_DELTOOL, 0, LPARAM(&ti));
	}
}
